@@ -44,8 +44,7 @@ echo "done."
 if [ -r config.status ]; then
   CMD=`awk '/^#.*\/?configure .*/ { $1 = ""; print; exit }' < config.status`
 else
-  CMD="./configure --enable-maintainer-mode --enable-warn \
-		   --with-sizzle=../sizzle"
+  CMD="./configure --enable-maintainer-mode --enable-warn"
 fi
 echo "Running $CMD $@ ..."
 $CMD "$@"
