@@ -1,7 +1,7 @@
 /*
  * awcs-proto.c - aWCS protocol implementation
  *
- * Copyright (C) 2000 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2000, 2001 Stefan Jahn <stefan@lkcc.org>
  * Copyright (C) 1999 Martin Grabmueller <mgrabmue@cs.tu-berlin.de>
  *
  * This is free software; you can redistribute it and/or modify
@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: awcs-proto.c,v 1.22 2000/12/16 10:57:23 ela Exp $
+ * $Id: awcs-proto.c,v 1.33 2001/06/27 20:38:36 ela Exp $
  *
  */
 
@@ -39,72 +39,39 @@
 #endif
 
 #ifdef __MINGW32__
-# include <winsock.h>
+# include <winsock2.h>
 #endif
 
 #ifndef __MINGW32__
 # include <netinet/in.h>
 #endif
 
-#include "util.h"
-#include "socket.h"
-#include "pipe-socket.h"
-#include "alloc.h"
-#include "server-core.h"
-#include "coserver/coserver.h"
+#include "libserveez.h"
 #include "awcs-proto.h"
-#include "server.h"
-#include "server-socket.h"
-
-/*
- * The aWCS port configuration.
- */
-portcfg_t awcs_net_port = 
-{
-  PROTO_TCP,   /* prefered protocol type */
-  42424,       /* prefered port */
-  "127.0.0.1", /* local ip */
-  NULL,        /* calculated later */
-  NULL,        /* inpipe */
-  NULL         /* outpipe */
-};
-
-portcfg_t awcs_fs_port = 
-{
-  PROTO_PIPE,  /* prefered protocol type */
-  0,           /* prefered port */
-  NULL,        /* local ip */
-  NULL,
-  ".aWCSrecv", /* inpipe */
-  ".aWCSsend"  /* outpipe */
-};
 
 /*
  * The aWCS server instance configuration.
  */
 awcs_config_t awcs_config =
 {
-  &awcs_net_port, /* current network port configuration */
-  &awcs_fs_port,  /* current filesystem port configuration */
   NULL,           /* aWCS Master server */
   0,              /* Was Master server detected ? */
   NULL            /* aWCS clients user base hash */
 };
 
 /*
- * Definition of the configuration items delivered by libsizzle.
+ * Definition of the configuration items delivered by the 
+ * configuration language.
  */
-key_value_pair_t awcs_config_prototype [] =
+svz_key_value_pair_t awcs_config_prototype [] =
 {
-  REGISTER_PORTCFG ("netport", awcs_config.netport, DEFAULTABLE),
-  REGISTER_PORTCFG ("fsport", awcs_config.fsport, DEFAULTABLE),
-  REGISTER_END ()
+  SVZ_REGISTER_END ()
 };
 
 /*
  * The aWCS server definition.
  */
-server_definition_t awcs_server_definition =
+svz_servertype_t awcs_server_definition =
 {
   "aWCS server",         /* server description */
   "aWCS",                /* server prefix used in the config file "aWCS?" */
@@ -120,7 +87,7 @@ server_definition_t awcs_server_definition =
   NULL,                  /* handle request callback */
   &awcs_config,          /* the instance configuration */
   sizeof (awcs_config),  /* sizeof the instance configuration */
-  awcs_config_prototype  /* configuration definition for libsizzle */
+  awcs_config_prototype  /* configuration definitions  */
 };
 
 /*
@@ -143,7 +110,7 @@ awcs_hash_equals (char *id1, char *id2)
 static unsigned long 
 awcs_hash_code (char *id)
 {
-  unsigned long code = UINT32 (id);
+  unsigned long code = SVZ_UINT32 (id);
   return code;
 }
 
@@ -151,22 +118,18 @@ awcs_hash_code (char *id)
  * Local aWCS server instance initialization routine.
  */
 int
-awcs_init (server_t *server)
+awcs_init (svz_server_t *server)
 {
   awcs_config_t *cfg = server->cfg;
 
   /* initialize server instance */
   cfg->master = 0;
-  cfg->clients = hash_create (4);
+  cfg->clients = svz_hash_create (4);
   cfg->clients->code = awcs_hash_code;
   cfg->clients->keylen = awcs_hash_keylen;
   cfg->clients->equals = awcs_hash_equals;
   cfg->server = NULL;
 
-  /* bind the port configuration */
-  server_bind (server, cfg->netport);
-  server_bind (server, cfg->fsport);
-
   return 0;
 }
 
@@ -174,16 +137,15 @@ awcs_init (server_t *server)
  * Local aWCS server instance finalizer.
  */
 int
-awcs_finalize (server_t *server)
+awcs_finalize (svz_server_t *server)
 {
   awcs_config_t *cfg = server->cfg;
   
-  hash_destroy (cfg->clients);
+  svz_hash_destroy (cfg->clients);
   
   return 0;
 }
 
-#if ENABLE_REVERSE_LOOKUP
 /*
  * Gets called when a nslookup coserver has resolved a IP address
  * for socket SOCK to name and has been identified as an aWCS client.
@@ -192,7 +154,7 @@ int
 awcs_nslookup_done (char *host, int id, int version)
 {
   awcs_config_t *cfg;
-  socket_t sock = sock_find (id, version);
+  svz_socket_t *sock = svz_sock_find (id, version);
 
   if (host && sock)
     {
@@ -201,23 +163,23 @@ awcs_nslookup_done (char *host, int id, int version)
       if (!cfg->server)
 	{
 #if ENABLE_DEBUG
-	  log_printf (LOG_DEBUG, "awcs: no master server (nslookup_done)\n");
+	  svz_log (LOG_DEBUG, "awcs: no master server (nslookup_done)\n");
 #endif
 	  return -1;
 	}
 
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "awcs: sending resolved ip to master\n");
+      svz_log (LOG_DEBUG, "awcs: sending resolved ip to master\n");
 #endif
 
-      if (sock_printf (cfg->server, AWCS_ID_FMT " %d " AWCS_ID_FMT " %s%c",
-		       cfg->server->id,
-		       STATUS_NSLOOKUP,
-		       sock->id,
-		       host, '\0'))
+      if (svz_sock_printf (cfg->server, AWCS_ID_FMT " %d " AWCS_ID_FMT " %s%c",
+			   cfg->server->id,
+			   STATUS_NSLOOKUP,
+			   sock->id,
+			   host, '\0'))
 	{
-	  log_printf (LOG_FATAL, "awcs: master write error\n");
-	  sock_schedule_for_shutdown (cfg->server);
+	  svz_log (LOG_FATAL, "awcs: master write error\n");
+	  svz_sock_schedule_for_shutdown (cfg->server);
 	  cfg->server = NULL;
 	  awcs_disconnect_clients (cfg);
 	  return -1;
@@ -226,9 +188,7 @@ awcs_nslookup_done (char *host, int id, int version)
 
   return 0;
 }
-#endif
 
-#if ENABLE_IDENT
 /*
  * Gets called when a ident coserver has resolved a IP address
  * for socket SOCK to name and has been identified as an aWCS client.
@@ -236,7 +196,7 @@ awcs_nslookup_done (char *host, int id, int version)
 int
 awcs_ident_done (char *user, int id, int version)
 {
-  socket_t sock = sock_find (id, version);
+  svz_socket_t *sock = svz_sock_find (id, version);
   awcs_config_t *cfg;
 
   if (user && sock)
@@ -246,23 +206,23 @@ awcs_ident_done (char *user, int id, int version)
       if (!cfg->server)
 	{
 #if ENABLE_DEBUG
-	  log_printf (LOG_DEBUG, "awcs: no master server (ident_done)\n");
+	  svz_log (LOG_DEBUG, "awcs: no master server (ident_done)\n");
 #endif
 	  return -1;
 	}
 
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "awcs: sending identified client to master\n");
+      svz_log (LOG_DEBUG, "awcs: sending identified client to master\n");
 #endif
 
-      if (sock_printf (cfg->server, AWCS_ID_FMT " %d " AWCS_ID_FMT " %s%c",
-		       cfg->server->id,
-		       STATUS_IDENT,
-		       sock->id,
-		       user, '\0'))
+      if (svz_sock_printf (cfg->server, AWCS_ID_FMT " %d " AWCS_ID_FMT " %s%c",
+			   cfg->server->id,
+			   STATUS_IDENT,
+			   sock->id,
+			   user, '\0'))
 	{
-	  log_printf (LOG_FATAL, "awcs: master write error\n");
-	  sock_schedule_for_shutdown (cfg->server);
+	  svz_log (LOG_FATAL, "awcs: master write error\n");
+	  svz_sock_schedule_for_shutdown (cfg->server);
 	  cfg->server = NULL;
 	  awcs_disconnect_clients (cfg);
 	  return -1;
@@ -271,7 +231,6 @@ awcs_ident_done (char *user, int id, int version)
 
   return 0;
 }
-#endif
 
 /*
  * This is called when a valid aWCS client has been connected.
@@ -279,7 +238,7 @@ awcs_ident_done (char *user, int id, int version)
  * a reverse nslookup and a ident request.
  */
 static int
-awcs_status_connected (socket_t sock)
+awcs_status_connected (svz_socket_t *sock)
 {
   unsigned short port;
   unsigned long addr;
@@ -288,7 +247,7 @@ awcs_status_connected (socket_t sock)
   if (!cfg->server)
     {
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "awcs: no master server (status_connected)\n");
+      svz_log (LOG_DEBUG, "awcs: no master server (status_connected)\n");
 #endif
       return -1;
     }
@@ -297,19 +256,19 @@ awcs_status_connected (socket_t sock)
   port = sock->remote_port;
 
 #if ENABLE_DEBUG
-  log_printf (LOG_DEBUG, "awcs: sending connect on socket id %d to master\n",
-	      sock->id);
+  svz_log (LOG_DEBUG, "awcs: sending connect on socket id %d to master\n",
+	   sock->id);
 #endif
 
-  if (sock_printf (cfg->server, AWCS_ID_FMT " %d " AWCS_ID_FMT " %s:%u%c",
-		   cfg->server->id,
-		   STATUS_CONNECT,
-		   sock->id,
-		   util_inet_ntoa (addr),
-		   htons (port), '\0'))
+  if (svz_sock_printf (cfg->server, AWCS_ID_FMT " %d " AWCS_ID_FMT " %s:%u%c",
+		       cfg->server->id,
+		       STATUS_CONNECT,
+		       sock->id,
+		       svz_inet_ntoa (addr),
+		       htons (port), '\0'))
     {
-      log_printf (LOG_FATAL, "awcs: master write error\n");
-      sock_schedule_for_shutdown (cfg->server);
+      svz_log (LOG_FATAL, "awcs: master write error\n");
+      svz_sock_schedule_for_shutdown (cfg->server);
       cfg->server = NULL;
       awcs_disconnect_clients (cfg);
       return -1;
@@ -320,8 +279,8 @@ awcs_status_connected (socket_t sock)
    */
   if (sock != cfg->server)
     {
-      coserver_reverse (addr, awcs_nslookup_done, sock->id, sock->version);
-      coserver_ident (sock, awcs_ident_done, sock->id, sock->version);
+      svz_coserver_rdns (addr, awcs_nslookup_done, sock->id, sock->version);
+      svz_coserver_ident (sock, awcs_ident_done, sock->id, sock->version);
     }
 
   return 0;
@@ -333,31 +292,31 @@ awcs_status_connected (socket_t sock)
  * is an error code telling how the client got disconnected.
  */
 static int
-awcs_status_disconnected (socket_t sock, int reason)
+awcs_status_disconnected (svz_socket_t *sock, int reason)
 {
   awcs_config_t *cfg = sock->cfg;
 
   if (!cfg->server)
     {
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "awcs: no master server (status_disconnected)\n");
+      svz_log (LOG_DEBUG, "awcs: no master server (status_disconnected)\n");
 #endif
       return -1;
     }
 
 #if ENABLE_DEBUG
-  log_printf (LOG_DEBUG, "awcs: sending disconnect on socket %d to master\n",
-	      sock->sock_desc);
+  svz_log (LOG_DEBUG, "awcs: sending disconnect on socket %d to master\n",
+	   sock->sock_desc);
 #endif /* ENABLE_DEBUG */
 
-  if (sock_printf (cfg->server, AWCS_ID_FMT " %d " AWCS_ID_FMT " %d%c", 
-		   cfg->server->id,
-		   STATUS_DISCONNECT,
-		   sock->id, 
-		   reason, '\0'))
+  if (svz_sock_printf (cfg->server, AWCS_ID_FMT " %d " AWCS_ID_FMT " %d%c", 
+		       cfg->server->id,
+		       STATUS_DISCONNECT,
+		       sock->id, 
+		       reason, '\0'))
     {
-      log_printf (LOG_FATAL, "awcs: master write error\n");
-      sock_schedule_for_shutdown (cfg->server);
+      svz_log (LOG_FATAL, "awcs: master write error\n");
+      svz_sock_schedule_for_shutdown (cfg->server);
       cfg->server = NULL;
       awcs_disconnect_clients (cfg);
       return -1;
@@ -372,26 +331,26 @@ awcs_status_disconnected (socket_t sock, int reason)
  * is an error code telling why the client got kicked.
  */
 static int
-awcs_status_kicked (socket_t sock, int reason)
+awcs_status_kicked (svz_socket_t *sock, int reason)
 {
   awcs_config_t *cfg = sock->cfg;
   
   if (!cfg->server)
     {
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "awcs: no master server (status_kicked)\n");
+      svz_log (LOG_DEBUG, "awcs: no master server (status_kicked)\n");
 #endif
       return -1;
     }
 
-  if (sock_printf (cfg->server, AWCS_ID_FMT " %d " AWCS_ID_FMT " %d%c",
-		   cfg->server->id,
-		   STATUS_KICK,
-		   sock->id, 
-		   reason, '\0'))
+  if (svz_sock_printf (cfg->server, AWCS_ID_FMT " %d " AWCS_ID_FMT " %d%c",
+		       cfg->server->id,
+		       STATUS_KICK,
+		       sock->id, 
+		       reason, '\0'))
     {
-      log_printf (LOG_FATAL, "awcs: master write error\n");
-      sock_schedule_for_shutdown (cfg->server);
+      svz_log (LOG_FATAL, "awcs: master write error\n");
+      svz_sock_schedule_for_shutdown (cfg->server);
       cfg->server = NULL;
       awcs_disconnect_clients (cfg);
       return -1;
@@ -410,17 +369,17 @@ awcs_status_alive (awcs_config_t *cfg)
   if (!cfg->server)
     {
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "awcs: no master server (status_alive)\n");
+      svz_log (LOG_DEBUG, "awcs: no master server (status_alive)\n");
 #endif /* ENABLE_DEBUG */
       return -1;
     }
 
-  if (sock_printf (cfg->server, AWCS_ID_FMT " %d 42%c",
-		   cfg->server->id,
-		   STATUS_ALIVE, '\0'))
+  if (svz_sock_printf (cfg->server, AWCS_ID_FMT " %d 42%c",
+		       cfg->server->id,
+		       STATUS_ALIVE, '\0'))
     {
-      log_printf (LOG_FATAL, "awcs: master write error\n");
-      sock_schedule_for_shutdown (cfg->server);
+      svz_log (LOG_FATAL, "awcs: master write error\n");
+      svz_sock_schedule_for_shutdown (cfg->server);
       cfg->server = NULL;
       awcs_disconnect_clients (cfg);
       return -1;
@@ -435,17 +394,17 @@ awcs_status_notify (awcs_config_t *cfg)
   if (!cfg->server)
     {
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "awcs: no master server (status_notify)\n");
+      svz_log (LOG_DEBUG, "awcs: no master server (status_notify)\n");
 #endif /* ENABLE_DEBUG */
       return -1;
     }
 
-  if (sock_printf (cfg->server, AWCS_ID_FMT " %d 42%c",
-		   cfg->server->id,
-		   STATUS_NOTIFY, '\0'))
+  if (svz_sock_printf (cfg->server, AWCS_ID_FMT " %d 42%c",
+		       cfg->server->id,
+		       STATUS_NOTIFY, '\0'))
     {
-      log_printf (LOG_FATAL, "awcs: master write error\n");
-      sock_schedule_for_shutdown (cfg->server);
+      svz_log (LOG_FATAL, "awcs: master write error\n");
+      svz_sock_schedule_for_shutdown (cfg->server);
       cfg->server = NULL;
       awcs_disconnect_clients (cfg);
       return -1;
@@ -460,23 +419,23 @@ awcs_status_notify (awcs_config_t *cfg)
 static int
 awcs_process_broadcast (awcs_config_t *cfg, char *cmd, int cmd_len)
 {
-  socket_t *sock;
+  svz_socket_t **sock;
   int n;
   
 #if ENABLE_DEBUG
-  log_printf (LOG_DEBUG, "awcs: broadcasting\n");
+  svz_log (LOG_DEBUG, "awcs: broadcasting\n");
 #endif /* ENABLE_DEBUG */
 
-  if ((sock = (socket_t *) hash_values (cfg->clients)) != NULL)
+  if ((sock = (svz_socket_t **) svz_hash_values (cfg->clients)) != NULL)
     {
-      for (n = 0; n < hash_size (cfg->clients); n++)
+      for (n = 0; n < svz_hash_size (cfg->clients); n++)
 	{
-	  if (sock_write (sock[n], cmd, cmd_len))
+	  if (svz_sock_write (sock[n], cmd, cmd_len))
 	    {
-	      sock_schedule_for_shutdown (sock[n]);
+	      svz_sock_schedule_for_shutdown (sock[n]);
 	    }
 	}
-      hash_xfree (sock);
+      svz_hash_xfree (sock);
     }
   return 0;
 }
@@ -490,7 +449,7 @@ awcs_process_multicast (awcs_config_t *cfg, char *cmd, int cmd_len)
 {
   char *msg;
   int address;
-  socket_t sock;
+  svz_socket_t *sock;
 
   msg = cmd;
 
@@ -504,22 +463,22 @@ awcs_process_multicast (awcs_config_t *cfg, char *cmd, int cmd_len)
 
   for (;;)
     {
-      address = util_atoi (cmd);
-      sock = (socket_t) hash_get (cfg->clients, (char *) &address);
+      address = svz_atoi (cmd);
+      sock = (svz_socket_t *) svz_hash_get (cfg->clients, (char *) &address);
 
       if (sock)
 	{
-	  if (sock_write (sock, msg, cmd_len))
+	  if (svz_sock_write (sock, msg, cmd_len))
 	    {
-	      sock_schedule_for_shutdown (sock);
+	      svz_sock_schedule_for_shutdown (sock);
 	    }
 	}
 #if ENABLE_DEBUG
       else
 	{
-	  log_printf (LOG_DEBUG, 
-		      "awcs: master sent invalid id (multicast %d)\n", 
-		      address);
+	  svz_log (LOG_DEBUG, 
+		   "awcs: master sent invalid id (multicast %d)\n", 
+		   address);
 	}
 #endif /* ENABLE_DEBUG */
 
@@ -538,7 +497,7 @@ static int
 awcs_process_status (awcs_config_t *cfg, char *cmd, int cmd_len)
 {
 #if ENABLE_DEBUG
-  log_printf (LOG_DEBUG, "awcs: sending status message\n");
+  svz_log (LOG_DEBUG, "awcs: sending status message\n");
 #endif /* ENABLE_DEBUG */
   awcs_status_alive (cfg);
 
@@ -552,19 +511,19 @@ awcs_process_status (awcs_config_t *cfg, char *cmd, int cmd_len)
 static int
 awcs_process_kick (awcs_config_t *cfg, char *cmd, int cmd_len)
 {
-  socket_t sock;
+  svz_socket_t *sock;
   int address;
 
   for (;;)
     {
-      address = util_atoi (cmd);
-      sock = (socket_t) hash_get (cfg->clients, (char *) &address);
+      address = svz_atoi (cmd);
+      sock = (svz_socket_t *) svz_hash_get (cfg->clients, (char *) &address);
 
       if (sock)
 	{
 #if ENABLE_DEBUG
-	  log_printf (LOG_DEBUG, "awcs: kicking socket %d\n",
-		      sock->sock_desc);
+	  svz_log (LOG_DEBUG, "awcs: kicking socket %d\n",
+		   sock->sock_desc);
 #endif /* ENABLE_DEBUG */
 	  /*
 	   * This is a hack.  We set the handler to NULL to be sure
@@ -572,14 +531,14 @@ awcs_process_kick (awcs_config_t *cfg, char *cmd, int cmd_len)
 	   * message for a kick he initiated.
 	   */
 	  sock->kicked_socket = NULL;
-	  sock_schedule_for_shutdown (sock);
+	  svz_sock_schedule_for_shutdown (sock);
 	}
 #if ENABLE_DEBUG
       else
 	{
-	  log_printf (LOG_DEBUG, 
-		      "awcs: master sent invalid id (kick %d)\n",
-		      address);
+	  svz_log (LOG_DEBUG, 
+		   "awcs: master sent invalid id (kick %d)\n",
+		   address);
 	}
 #endif /* ENABLE_DEBUG */
 
@@ -599,20 +558,20 @@ awcs_process_kick (awcs_config_t *cfg, char *cmd, int cmd_len)
 static int
 awcs_process_floodcmd (awcs_config_t *cfg, char *cmd, int cmd_len, int flag)
 {
-  socket_t sock;
+  svz_socket_t *sock;
   int address;
 
   for (;;)
     {
-      address = util_atoi (cmd);
-      sock = (socket_t) hash_get (cfg->clients, (char *) &address);
+      address = svz_atoi (cmd);
+      sock = (svz_socket_t *) svz_hash_get (cfg->clients, (char *) &address);
 
       if (sock)
 	{
 #if ENABLE_DEBUG
-	  log_printf (LOG_DEBUG, "awcs: switching flood "
-		      "control for socket %d %s\n",
-		      sock->sock_desc, flag ? "on" : "off");
+	  svz_log (LOG_DEBUG, "awcs: switching flood "
+		   "control for socket %d %s\n",
+		   sock->sock_desc, flag ? "on" : "off");
 #endif /* ENABLE_DEBUG */
 	  if (flag)
 	    sock->flags &= ~SOCK_FLAG_NOFLOOD;
@@ -622,9 +581,9 @@ awcs_process_floodcmd (awcs_config_t *cfg, char *cmd, int cmd_len, int flag)
 #if ENABLE_DEBUG
       else
 	{
-	  log_printf (LOG_DEBUG, 
-		      "awcs: master sent invalid id (floodcmd %d)\n",
-		      address);
+	  svz_log (LOG_DEBUG, 
+		   "awcs: master sent invalid id (floodcmd %d)\n",
+		   address);
 	}
 #endif /* ENABLE_DEBUG */
       
@@ -648,8 +607,8 @@ awcs_handle_master_request (awcs_config_t *cfg, char *request, int request_len)
     return -1;
 
 #if 0
-  util_hexdump (stdout, "master request", cfg->server->sock_desc,
-		request, request_len, 0);
+  svz_hexdump (stdout, "master request", cfg->server->sock_desc,
+	       request, request_len, 0);
 #endif
 
   request_len--;
@@ -687,10 +646,10 @@ awcs_handle_master_request (awcs_config_t *cfg, char *request, int request_len)
       break;
     case '6':
       /* The following code should not be executed anymore. */
-      log_printf (LOG_NOTICE, "awcs: skipping '6' ...\n");
+      svz_log (LOG_NOTICE, "awcs: skipping '6' ...\n");
       break;
     default:
-      log_printf (LOG_ERROR, "awcs: bad master server request\n");
+      svz_log (LOG_ERROR, "awcs: bad master server request\n");
       break;
     }
   return 0;
@@ -703,16 +662,16 @@ awcs_handle_master_request (awcs_config_t *cfg, char *request, int request_len)
 void
 awcs_disconnect_clients (awcs_config_t *cfg)
 {
-  socket_t *sock;
+  svz_socket_t **sock;
   int n;
 
-  if ((sock = (socket_t *) hash_values (cfg->clients)) != NULL)
+  if ((sock = (svz_socket_t **) svz_hash_values (cfg->clients)) != NULL)
     {
-      for (n = 0; n < hash_size (cfg->clients); n++)
+      for (n = 0; n < svz_hash_size (cfg->clients); n++)
 	{
-	  sock_schedule_for_shutdown (sock[n]);
+	  svz_sock_schedule_for_shutdown (sock[n]);
 	}
-      hash_xfree (sock);
+      svz_hash_xfree (sock);
     }
 }
 
@@ -721,20 +680,20 @@ awcs_disconnect_clients (awcs_config_t *cfg)
  * SOCK.  Return a non-zero value on error.
  */
 static int
-awcs_handle_request (socket_t sock, char *request, int request_len)
+awcs_handle_request (svz_socket_t *sock, char *request, int request_len)
 {
   int ret;
   awcs_config_t *cfg = sock->cfg;
 
 #if 0
-  util_hexdump (stdout, "awcs request", sock->sock_desc, 
-		request, request_len, 1000);
+  svz_hexdump (stdout, "awcs request", sock->sock_desc, 
+	       request, request_len, 1000);
 #endif
 
   if (!cfg->server)
     {
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "awcs: no master server (awcs_handle_request)\n");
+      svz_log (LOG_DEBUG, "awcs: no master server (awcs_handle_request)\n");
 #endif
       awcs_disconnect_clients (cfg);
       return -1;
@@ -746,15 +705,15 @@ awcs_handle_request (socket_t sock, char *request, int request_len)
     }
   else
     {
-      ret = sock_printf (cfg->server, AWCS_ID_FMT " ", sock->id);
+      ret = svz_sock_printf (cfg->server, AWCS_ID_FMT " ", sock->id);
       if (ret == 0)
 	{
-	  ret = sock_write (cfg->server, request, request_len);
+	  ret = svz_sock_write (cfg->server, request, request_len);
 	}
       if (ret)
 	{
-	  log_printf (LOG_FATAL, "awcs: master write error\n");
-	  sock_schedule_for_shutdown (cfg->server);
+	  svz_log (LOG_FATAL, "awcs: master write error\n");
+	  svz_sock_schedule_for_shutdown (cfg->server);
 	  cfg->server = NULL;
 	  awcs_disconnect_clients (cfg);
 	  return -1;
@@ -769,7 +728,7 @@ awcs_handle_request (socket_t sock, char *request, int request_len)
  * request gets handled and removed from the queue.
  */
 int
-awcs_check_request (socket_t sock)
+awcs_check_request (svz_socket_t *sock)
 {
   int retval = 0;
   int request_len = 0;
@@ -809,9 +768,9 @@ awcs_check_request (socket_t sock)
  * server detection.
  */
 int
-awcs_connect_socket (void *config, socket_t sock)
+awcs_connect_socket (svz_server_t *server, svz_socket_t *sock)
 {
-  awcs_config_t *cfg = config;
+  awcs_config_t *cfg = server->cfg;
 
   /*
    * Don't allow clients without master server or multiple master servers.
@@ -821,17 +780,17 @@ awcs_connect_socket (void *config, socket_t sock)
       if (!cfg->server && !cfg->master)
 	{
 #if ENABLE_DEBUG
-	  log_printf (LOG_DEBUG,
-		      "awcs: master not present, cannot connect socket %d\n",
-		      sock->sock_desc);
+	  svz_log (LOG_DEBUG,
+		   "awcs: master not present, cannot connect socket %d\n",
+		   sock->sock_desc);
 #endif
 	}
       if (cfg->server && cfg->master)
 	{
 #if ENABLE_DEBUG
-	  log_printf (LOG_DEBUG,
-		      "awcs: master is present, cannot connect socket %d\n",
-		      sock->sock_desc);
+	  svz_log (LOG_DEBUG,
+		   "awcs: master is present, cannot connect socket %d\n",
+		   sock->sock_desc);
 #endif
 	}
 
@@ -841,13 +800,13 @@ awcs_connect_socket (void *config, socket_t sock)
 #if ENABLE_DEBUG
   if (sock->flags & SOCK_FLAG_PIPE)
     {
-      log_printf (LOG_DEBUG, "awcs: connection on pipe (%d-%d)\n",
-		  sock->pipe_desc[READ], sock->pipe_desc[WRITE]);
+      svz_log (LOG_DEBUG, "awcs: connection on pipe (%d-%d)\n",
+	       sock->pipe_desc[READ], sock->pipe_desc[WRITE]);
     }
   else 
     {
-      log_printf (LOG_DEBUG, "awcs: connection on socket %d\n",
-		  sock->sock_desc);
+      svz_log (LOG_DEBUG, "awcs: connection on socket %d\n",
+	       sock->sock_desc);
     }
 #endif /* ENABLE_DEBUG */
 
@@ -859,30 +818,30 @@ awcs_connect_socket (void *config, socket_t sock)
 #if ENABLE_DEBUG
       if (sock->flags & SOCK_FLAG_PIPE)
 	{
-	  log_printf (LOG_NOTICE, 
-		      "awcs: master server connected on pipe (%d-%d)\n",
-		      sock->pipe_desc[READ], sock->pipe_desc[WRITE]);
+	  svz_log (LOG_NOTICE, 
+		   "awcs: master server connected on pipe (%d-%d)\n",
+		   sock->pipe_desc[READ], sock->pipe_desc[WRITE]);
 	}
       else
 	{
-	  log_printf (LOG_NOTICE, 
-		      "awcs: master server connected on socket %d\n",
-		      sock->sock_desc);
+	  svz_log (LOG_NOTICE, 
+		   "awcs: master server connected on socket %d\n",
+		   sock->sock_desc);
 	}
 #endif /* ENABLE_DEBUG */
       cfg->server = sock;
       sock->idle_func = awcs_idle_func;
       sock->idle_counter = 3;
       sock->flags |= (SOCK_FLAG_NOFLOOD | SOCK_FLAG_PRIORITY);
-      log_printf (LOG_NOTICE,
-		  "awcs: resizing master buffers: %d, %d\n",
-		  MASTER_SEND_BUFSIZE, MASTER_RECV_BUFSIZE);
-      sock_resize_buffers (sock, MASTER_SEND_BUFSIZE, MASTER_RECV_BUFSIZE);
+      svz_log (LOG_NOTICE,
+	       "awcs: resizing master buffers: %d, %d\n",
+	       MASTER_SEND_BUFSIZE, MASTER_RECV_BUFSIZE);
+      svz_sock_resize_buffers (sock, MASTER_SEND_BUFSIZE, MASTER_RECV_BUFSIZE);
       sock->flags |= SOCK_FLAG_INITED;
     }
   else
     {
-      hash_put (cfg->clients, (char *) &sock->id, sock);
+      svz_hash_put (cfg->clients, (char *) &sock->id, sock);
       sock->kicked_socket = awcs_kicked_socket;
     }
 
@@ -899,20 +858,20 @@ awcs_connect_socket (void *config, socket_t sock)
  * message is sent to the master server.
  */
 int
-awcs_disconnected_socket (socket_t sock)
+awcs_disconnected_socket (svz_socket_t *sock)
 {
   awcs_config_t *cfg = sock->cfg;
   
   if (sock == cfg->server)
     {
-      log_printf (LOG_ERROR, "awcs: lost master server\n");
+      svz_log (LOG_ERROR, "awcs: lost master server\n");
       cfg->server = NULL;
       awcs_disconnect_clients (cfg);
     }
   else
     {
       awcs_status_disconnected (sock, 1);
-      hash_delete (cfg->clients, (char *) &sock->id);
+      svz_hash_delete (cfg->clients, (char *) &sock->id);
     }
 
   return 0;
@@ -924,7 +883,7 @@ awcs_disconnected_socket (socket_t sock)
  * buffer overflow or flooding.
  */
 int
-awcs_kicked_socket (socket_t sock, int reason)
+awcs_kicked_socket (svz_socket_t *sock, int reason)
 {
   if (!reason)
     awcs_status_kicked (sock, KICK_FLOODING);
@@ -940,7 +899,7 @@ awcs_kicked_socket (socket_t sock, int reason)
  * and resets its timeout counter.
  */
 int
-awcs_idle_func (socket_t sock)
+awcs_idle_func (svz_socket_t *sock)
 {
   awcs_config_t *cfg = sock->cfg;
 
@@ -958,28 +917,28 @@ awcs_idle_func (socket_t sock)
  * the receive buffer looks like an aWCS identification request.
  */
 int
-awcs_detect_proto (void *config, socket_t sock)
+awcs_detect_proto (svz_server_t *server, svz_socket_t *sock)
 {
-  awcs_config_t *cfg = config;
+  awcs_config_t *cfg = server->cfg;
   int len = 0;
 
 #if 0
-  util_hexdump (stdout, "detecting awcs", sock->sock_desc,
-		sock->recv_buffer, sock->recv_buffer_fill, 0);
+  svz_hexdump (stdout, "detecting awcs", sock->sock_desc,
+	       sock->recv_buffer, sock->recv_buffer_fill, 0);
 #endif
 
   if (sock->recv_buffer_fill >= MASTER_DETECTION && 
       !memcmp (sock->recv_buffer, AWCS_MASTER, MASTER_DETECTION))
     {
       cfg->master = 1;
-      log_printf (LOG_NOTICE, "awcs: master detected\n");
+      svz_log (LOG_NOTICE, "awcs: master detected\n");
       len = MASTER_DETECTION;
     }
   else if (sock->recv_buffer_fill >= CLIENT_DETECTION && 
 	   !memcmp (sock->recv_buffer, AWCS_CLIENT, CLIENT_DETECTION))
     {
       cfg->master = 0;
-      log_printf (LOG_NOTICE, "awcs: client detected\n");
+      svz_log (LOG_NOTICE, "awcs: client detected\n");
       len = CLIENT_DETECTION;
     }
 
