@@ -1,7 +1,7 @@
 /*
  * control-proto.c - control protocol implementation
  *
- * Copyright (C) 2000 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2000, 2001 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: control-proto.c,v 1.36 2001/01/05 01:52:45 ela Exp $
+ * $Id: control-proto.c,v 1.54 2001/06/27 20:38:36 ela Exp $
  *
  */
 
@@ -34,10 +34,11 @@
 #include <time.h>
 
 #ifdef __MINGW32__
-# include <winsock.h>
+# include <winsock2.h>
 #endif
 
 #ifndef __MINGW32__
+# include <sys/types.h>
 # include <netinet/in.h>
 #endif
 
@@ -55,61 +56,37 @@
 # include <sys/times.h>
 #endif
 
-#include "snprintf.h"
-#include "alloc.h"
-#include "hash.h"
-#include "util.h"
-#include "socket.h"
-#include "pipe-socket.h"
-#include "server-core.h"
-#include "server-socket.h"
-#include "server.h"
-#include "serveez.h"
-#include "coserver/coserver.h"
+#include "libserveez.h"
 #include "control-proto.h"
 
 #if ENABLE_HTTP_PROTO
 # include "http-server/http-cache.h"
 #endif
 
-/*
- * The control port configuration.
- */
-portcfg_t ctrl_port =
-{
-  PROTO_TCP,  /* TCP protocol definition */
-  42424,      /* preferred port */
-  "*",        /* preferred local ip address */
-  NULL,       /* calculated automatically later */
-  NULL,       /* no receiving (listening) pipe */
-  NULL        /* no sending pipe */
-};
-
 /*
  * The control server instance configuration.
  */
 ctrl_config_t ctrl_config =
 {
-  &ctrl_port  /* port configuration only (yet) */
+  0 /* nothing */
 };
 
 /*
- * Definition of the configuration items processed by libsizzle (taken
- * from the configuration file).
+ * Definition of the configuration items processed by the configuration 
+ * language.
  */
-key_value_pair_t ctrl_config_prototype [] =
+svz_key_value_pair_t ctrl_config_prototype [] =
 {
-  REGISTER_PORTCFG ("netport", ctrl_config.netport, DEFAULTABLE),
-  REGISTER_END ()
+  SVZ_REGISTER_END ()
 };
 
 /*
  * Definition of the control protocol server.
  */
-server_definition_t ctrl_server_definition =
+svz_servertype_t ctrl_server_definition =
 {
   "control protocol server", /* long server description */
-  "control",                 /* short server description (for libsizzle) */
+  "control",                 /* short server description */
   NULL,                      /* global initializer */
   ctrl_init,                 /* instance initializer */
   ctrl_detect_proto,         /* protocol detection routine */
@@ -122,7 +99,7 @@ server_definition_t ctrl_server_definition =
   NULL,                      /* handle request callback */
   &ctrl_config,              /* default configuration */
   sizeof (ctrl_config),      /* size of the configuration */
-  ctrl_config_prototype      /* configuration prototypes (libsizzle) */
+  ctrl_config_prototype      /* configuration prototypes */
 };
 
 /*
@@ -136,11 +113,10 @@ cpu_state_t cpu_state;
  * server to a given port configuration.
  */
 int
-ctrl_init (server_t *server)
+ctrl_init (svz_server_t *server)
 {
   ctrl_config_t *cfg = server->cfg;
 
-  server_bind (server, cfg->netport);
   return 0;
 }
 
@@ -148,7 +124,7 @@ ctrl_init (server_t *server)
  * Server instance finalizer.
  */
 int
-ctrl_finalize (server_t *server)
+ctrl_finalize (svz_server_t *server)
 {
   return 0;
 }
@@ -157,12 +133,12 @@ ctrl_finalize (server_t *server)
  * Server info callback.
  */
 char *
-ctrl_info_server (server_t *server)
+ctrl_info_server (svz_server_t *server)
 {
   static char info[128];
   ctrl_config_t *cfg = server->cfg;
 
-  sprintf (info, " tcp port : %d", cfg->netport->port);
+  sprintf (info, " nothing to be configured, yet");
   return info;
 }
 
@@ -170,9 +146,8 @@ ctrl_info_server (server_t *server)
  * Client info callback.
  */
 char *
-ctrl_info_client (void *ctrl_cfg, socket_t sock)
+ctrl_info_client (svz_server_t *server, svz_socket_t *sock)
 {
-  ctrl_config_t *cfg = ctrl_cfg;
   static char info[128];
 
   sprintf (info, "This is a control connection client.");
@@ -185,9 +160,8 @@ ctrl_info_client (void *ctrl_cfg, socket_t sock)
  * the receive buffer looks like the control protocol.
  */
 int
-ctrl_detect_proto (void *ctrl_cfg, socket_t sock)
+ctrl_detect_proto (svz_server_t *server, svz_socket_t *sock)
 {
-  ctrl_config_t *cfg = ctrl_cfg;
   int ret = 0;
 
   /* accept both CRLF and CR */
@@ -213,7 +187,7 @@ ctrl_detect_proto (void *ctrl_cfg, socket_t sock)
 	}
       sock->recv_buffer_fill -= ret;
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "control protocol client detected\n");
+      svz_log (LOG_DEBUG, "control protocol client detected\n");
 #endif
       return -1;
     }
@@ -227,12 +201,10 @@ ctrl_detect_proto (void *ctrl_cfg, socket_t sock)
  * routine.
  */
 int
-ctrl_connect_socket (void *ctrlcfg, socket_t sock)
+ctrl_connect_socket (svz_server_t *server, svz_socket_t *sock)
 {
-  ctrl_config_t *cfg = ctrlcfg;
-
-  sock_resize_buffers (sock, CTRL_SEND_BUFSIZE, CTRL_RECV_BUFSIZE);
-  sock->check_request = default_check_request;
+  svz_sock_resize_buffers (sock, CTRL_SEND_BUFSIZE, CTRL_RECV_BUFSIZE);
+  sock->check_request = svz_sock_check_request;
   sock->handle_request = ctrl_handle_request;
   sock->boundary = CTRL_PACKET_DELIMITER;
   sock->boundary_size = CTRL_PACKET_DELIMITER_LEN;
@@ -251,7 +223,7 @@ ctrl_connect_socket (void *ctrlcfg, socket_t sock)
   cpu_state.cpuinfoline = CPU_FORMAT;
 
   /* send welcome message */
-  sock_printf (sock, "%s", CTRL_PASSWD);
+  svz_sock_printf (sock, "%s", CTRL_PASSWD);
 
   return 0;
 }
@@ -261,7 +233,7 @@ ctrl_connect_socket (void *ctrlcfg, socket_t sock)
  * connection will be closed immediately.
  */
 int
-ctrl_quit (socket_t sock, int flag, char *arg)
+ctrl_quit (svz_socket_t *sock, int flag, char *arg)
 {
   return flag;
 }
@@ -272,21 +244,15 @@ ctrl_quit (socket_t sock, int flag, char *arg)
  * of Serveez implements.
  */
 int
-ctrl_help (socket_t sock, int flag, char *arg)
+ctrl_help (svz_socket_t *sock, int flag, char *arg)
 {
-  sock_printf (sock, 
+  svz_sock_printf (sock, 
     "\r\n available commands:\r\n"
     "   * help                - this help screen\r\n"
     "   * quit                - quit this control connection\r\n"
-#if ENABLE_IDENT
     "   * restart ident       - restart the ident coserver\r\n"
-#endif /* ENABLE_IDENT */
-#if ENABLE_REVERSE_LOOKUP
     "   * restart reverse dns - restart reverse DNS lookup coserver\r\n"
-#endif /* ENABLE_REVERSE_LOOKUP */
-#if ENABLE_DNS_LOOKUP
     "   * restart dns         - restart the DNS lookup coserver\r\n"
-#endif /* ENABLE_DNS_LOOKUP */
     "   * killall             - shutdown all client connections\r\n"
     "   * kill id NUM         - shutdown connection NUM\r\n"
     "   * stat                - general statistics\r\n"
@@ -309,30 +275,31 @@ ctrl_help (socket_t sock, int flag, char *arg)
  * socket structure.
  */
 int
-ctrl_stat_id (socket_t sock, int flag, char *arg)
+ctrl_stat_id (svz_socket_t *sock, int flag, char *arg)
 {
   int id, n;
-  socket_t xsock;
+  svz_socket_t *xsock;
   char proto[128];
-  server_t *server;
-  int_coserver_t *coserver;
+  svz_server_t *server;
+  svz_coserver_t *coserver;
 
   /* Find the appropriate client or server connection. */
   id = atoi (arg);
-  if ((xsock = sock_find (id, -1)) == NULL)
+  if ((xsock = svz_sock_find (id, -1)) == NULL)
     {
-      sock_printf (sock, "no such connection: %d\r\n", id);
+      svz_sock_printf (sock, "no such connection: %d\r\n", id);
       return flag;
     }
 
-  sock_printf (sock, "\r\nconnection id %d (version %d) statistics\r\n\r\n", 
-	       id, xsock->version);
+  svz_sock_printf (sock, "\r\nconnection id %d (version %d) "
+		   "statistics\r\n\r\n", 
+		   id, xsock->version);
 
   /* 
    * Process general socket structure's flags. Uppercase words refer
    * to set bits and lowercase to unset bits.
    */
-  sock_printf (sock,
+  svz_sock_printf (sock,
     " flags    : %s %s %s %s %s %s %s\r\n"
     "            %s %s %s %s %s %s %s\r\n",
     xsock->flags & SOCK_FLAG_INBUF ?      "INBUF" : "inbuf",
@@ -350,7 +317,7 @@ ctrl_stat_id (socket_t sock, int flag, char *arg)
     xsock->flags & SOCK_FLAG_ENQUEUED ?   "ENQUEUED" : "enqueued",
     xsock->flags & SOCK_FLAG_PRIORITY ?   "PRIORITY" : "priority");
 
-  sock_printf (sock, " protocol : ");
+  svz_sock_printf (sock, " protocol : ");
 
   /* process connection type server flags */
   if (xsock->flags & SOCK_FLAG_LISTENING)
@@ -368,55 +335,55 @@ ctrl_stat_id (socket_t sock, int flag, char *arg)
       if (xsock->proto & PROTO_RAW)
 	strcat (proto, "raw ");
 
-      sock_printf (sock, "%s\r\n", proto);
-      for (n = 0; (server = SERVER (xsock->data, n)) != NULL; n++)
+      svz_sock_printf (sock, "%s\r\n", proto);
+      svz_array_foreach (xsock->data, server, n)
 	{
-	  sock_printf (sock, "            %d. %s (%s)\r\n", 
-		       n + 1, server->name, server->description);
+	  svz_sock_printf (sock, "            %d. %s (%s)\r\n", 
+			   n + 1, server->name, server->description);
 	}
     }
   /* process client info */
   else
     {
       /* usual client */
-      if ((server = server_find (xsock->cfg)) != NULL)
+      if ((server = svz_server_find (xsock->cfg)) != NULL)
 	{
-	  sock_printf (sock, "%s client\r\n", server->name);
+	  svz_sock_printf (sock, "%s client\r\n", server->name);
 	  if (server->info_client)
 	    {
-	      sock_printf (sock, "            %s\r\n",
-			   server->info_client (server->cfg, xsock));
+	      svz_sock_printf (sock, "            %s\r\n",
+			       server->info_client (server, xsock));
 	    }
 	}
       /* coserver */
       else if (xsock->flags & SOCK_FLAG_COSERVER)
 	{
 	  coserver = xsock->data;
-	  sock_printf (sock, "internal %s coserver\r\n",
-		       int_coserver_type[coserver->type].name);
+	  svz_sock_printf (sock, "internal %s coserver\r\n",
+			   svz_coservertypes[coserver->type].name);
 	}
       /* unidentified */
       else
 	{
-	  sock_printf (sock, "not yet identified\r\n");
+	  svz_sock_printf (sock, "not yet identified\r\n");
 	}
     }
 
   /* print all previously collected statistics of this connection */
-  sock_printf (sock, 
-	       " sock fd  : %d\r\n"
-	       " file fd  : %d\r\n"
-	       " pipe fd  : %d (recv), %d (send)\r\n"
-	       " foreign  : %s:%u\r\n",
-	       xsock->sock_desc,
-	       xsock->file_desc,
-	       xsock->pipe_desc[READ],
-	       xsock->pipe_desc[WRITE],
-	       util_inet_ntoa (xsock->remote_addr),
-	       ntohs (xsock->remote_port));
+  svz_sock_printf (sock, 
+		   " sock fd  : %d\r\n"
+		   " file fd  : %d\r\n"
+		   " pipe fd  : %d (recv), %d (send)\r\n"
+		   " foreign  : %s:%u\r\n",
+		   xsock->sock_desc,
+		   xsock->file_desc,
+		   xsock->pipe_desc[READ],
+		   xsock->pipe_desc[WRITE],
+		   svz_inet_ntoa (xsock->remote_addr),
+		   ntohs (xsock->remote_port));
 
-  /* that is why `util_inet_ntoa' cannot be used in the same call */
-  sock_printf (sock, 
+  /* that is why `svz_inet_ntoa' cannot be used in the same call */
+  svz_sock_printf (sock, 
 	       " local    : %s:%u\r\n"
 	       " sendbuf  : %d (size), %d (fill), %s (last send)\r\n"
 	       " recvbuf  : %d (size), %d (fill), %s (last recv)\r\n"
@@ -425,14 +392,14 @@ ctrl_stat_id (socket_t sock, int flag, char *arg)
 	       " flood    : %d (points), %d (limit)\r\n"
 #endif /* ENABLE_FLOOD_PROTECTION */
 	       " avail    : %s\r\n\r\n",
-	       util_inet_ntoa (xsock->local_addr),
+	       svz_inet_ntoa (xsock->local_addr),
 	       ntohs (xsock->local_port),
 	       xsock->send_buffer_size,
 	       xsock->send_buffer_fill,
-	       util_time (xsock->last_send),
+	       svz_time (xsock->last_send),
 	       xsock->recv_buffer_size,
 	       xsock->recv_buffer_fill,
-	       util_time (xsock->last_recv),
+	       svz_time (xsock->last_recv),
 	       xsock->idle_counter,
 #if ENABLE_FLOOD_PROTECTION
 	       xsock->flood_points,
@@ -450,37 +417,37 @@ ctrl_stat_id (socket_t sock, int flag, char *arg)
  * server and give information about it if so.
  */
 int
-ctrl_stat (socket_t sock, int flag, char *arg)
+ctrl_stat (svz_socket_t *sock, int flag, char *arg)
 {
-  server_t *server;
-  int n;
+  svz_server_t *server;
+  char *p;
 
-  /* go through all server instances */
-  for (n = 0; n < server_instances; n++)
+  /* find an appropriate server instance */
+  p = arg;
+  while (*p && *p != '\r' && *p != '\n')
+    p++;
+  if (*p)
+    *p = '\0';
+  if ((server = svz_hash_get (svz_servers, arg)) != NULL)
     {
-      server = servers[n];
-      if (!memcmp (server->name, arg, strlen (server->name)))
-	{
-	  sock_printf (sock, "\r\n%s (%s):\r\n",
+      svz_sock_printf (sock, "\r\n%s (%s):\r\n",
 		       server->description, server->name);
-	  if (server->info_server)
-	    {
-	      sock_printf (sock, "%s\r\n", server->info_server (server));
-	    }
-	  sock_printf (sock, "\r\n");
-	  return flag;
+      if (server->info_server)
+	{
+	  svz_sock_printf (sock, "%s\r\n", server->info_server (server));
 	}
+      svz_sock_printf (sock, "\r\n");
+      return flag;
     }
 
   /* print a standard output */
-  sock_printf (sock, 
-	       "\r\nThis is %s version %s running since %s.\r\n", 
-	       serveez_config.program_name, 
-	       serveez_config.version_string,
-	       util_time (serveez_config.start_time));
+  svz_sock_printf (sock, 
+		   "\r\nThis is %s version %s running since %s.\r\n", 
+		   svz_library, svz_version,
+		   svz_time (svz_config.start));
 
   /* display compile time feature list */
-  sock_printf (sock, "Features  :"
+  svz_sock_printf (sock, "Features  : FOO"
 #ifdef ENABLE_AWCS_PROTO
 	       " AWCS"
 #endif
@@ -508,16 +475,10 @@ ctrl_stat (socket_t sock, int flag, char *arg)
 	       "\r\n");
   
   /* second feature line */
-  sock_printf (sock, "           "
-#ifdef ENABLE_IDENT
+  svz_sock_printf (sock, "           "
 	       " IDENT"
-#endif
-#ifdef ENABLE_REVERSE_LOOKUP
 	       " REVERSE-DNS"
-#endif
-#ifdef ENABLE_DNS_LOOKUP
 	       " DNS"
-#endif
 #ifdef ENABLE_FLOOD_PROTECTION
 	       " FLOOD"
 #endif
@@ -530,20 +491,20 @@ ctrl_stat (socket_t sock, int flag, char *arg)
 	       "\r\n");
 
   /* display system and process information */
-  sock_printf (sock, "Os        : %s\r\n", util_version ());
-  sock_printf (sock, "Sys-Load  : %s\r\n", cpu_state.info);
-  sock_printf (sock, "Proc-Load : %s\r\n", cpu_state.pinfo);
+  svz_sock_printf (sock, "Os        : %s\r\n", svz_sys_version ());
+  svz_sock_printf (sock, "Sys-Load  : %s\r\n", cpu_state.info);
+  svz_sock_printf (sock, "Proc-Load : %s\r\n", cpu_state.pinfo);
 
   /* show general state */
-  sock_printf (sock, "\r\n * %d connected sockets (hard limit is %d)\r\n",
-	       connected_sockets, serveez_config.max_sockets);
-  sock_printf (sock, " * uptime is %s\r\n", 
-	       util_uptime (time (NULL) - serveez_config.start_time));
+  svz_sock_printf (sock, "\r\n * %d connected sockets (hard limit is %d)\r\n",
+		   svz_sock_connections, svz_config.max_sockets);
+  svz_sock_printf (sock, " * uptime is %s\r\n", 
+		   svz_uptime (time (NULL) - svz_config.start));
 #if ENABLE_DEBUG
-  sock_printf (sock, " * %d bytes of memory in %d blocks allocated\r\n", 
-	       allocated_bytes, allocated_blocks);
-#endif
-  sock_printf (sock, "\r\n");
+  svz_sock_printf (sock, " * %d bytes of memory in %d blocks allocated\r\n", 
+		   svz_allocated_bytes, svz_allocated_blocks);
+#endif /* ENABLE_DEBUG */
+  svz_sock_printf (sock, "\r\n");
 
   return flag;
 }
@@ -553,46 +514,46 @@ ctrl_stat (socket_t sock, int flag, char *arg)
  * each socket structure currently within the socket list.
  */
 int
-ctrl_stat_con (socket_t sock, int flag, char *arg)
+ctrl_stat_con (svz_socket_t *sock, int flag, char *arg)
 {
-  socket_t xsock;
+  svz_socket_t *xsock;
   char *id;
   char linet[64];  
   char rinet[64];
-  server_t *server;
+  svz_server_t *server;
 
-  sock_printf (sock, "\r\n%s", 
-	       "Proto              Id  RecvQ  SendQ "
-	       "Local                Foreign\r\n");
+  svz_sock_printf (sock, "\r\n%s", 
+		   "Proto              Id  RecvQ  SendQ "
+		   "Local                Foreign\r\n");
 
   /* go through all the socket list */
-  for (xsock = socket_root; xsock; xsock = xsock->next)
+  svz_sock_foreach (xsock)
     {
       /* get type of socket */
       id = "None";
       if (xsock->flags & SOCK_FLAG_LISTENING)
 	id = "Listener";
-      else if ((server = server_find (xsock->cfg)) != NULL)
+      else if ((server = svz_server_find (xsock->cfg)) != NULL)
 	id = server->name;
       else if (xsock->flags & SOCK_FLAG_COSERVER)
 	id = "Co-Server";
 
       /* print local and remote end of the connection */
       sprintf (linet, "%s:%u",
-	       util_inet_ntoa (xsock->local_addr),
+	       svz_inet_ntoa (xsock->local_addr),
 	       ntohs (xsock->local_port));
 
       sprintf (rinet, "%s:%u",
-	       util_inet_ntoa (xsock->remote_addr),
+	       svz_inet_ntoa (xsock->remote_addr),
 	       ntohs (xsock->remote_port));
       
       /* gather all information from above */
-      sock_printf (sock, 
-		   "%-16s %4d %6d %6d %-20s %-20s\r\n", id,
-		   xsock->id, xsock->recv_buffer_fill,
-		   xsock->send_buffer_fill, linet, rinet);
+      svz_sock_printf (sock, 
+		       "%-16s %4d %6d %6d %-20s %-20s\r\n", id,
+		       xsock->id, xsock->recv_buffer_fill,
+		       xsock->send_buffer_fill, linet, rinet);
     }
-  sock_printf (sock, "\r\n");
+  svz_sock_printf (sock, "\r\n");
 
   return flag;
 }
@@ -603,21 +564,21 @@ ctrl_stat_con (socket_t sock, int flag, char *arg)
  * visual representation of the http cache structures.
  */
 int
-ctrl_stat_cache (socket_t sock, int flag, char *arg)
+ctrl_stat_cache (svz_socket_t *sock, int flag, char *arg)
 {
   int n, total, files;
   char *p;
   http_cache_entry_t **cache;
 
-  sock_printf (sock, "\r\n%s", 
-	       "File                             "
-	       "Size  Usage  Hits Recent Ready\r\n");
+  svz_sock_printf (sock, "\r\n%s", 
+		   "File                             "
+		   "Size  Usage  Hits Recent Ready\r\n");
 
   files = total = 0;
-  if ((cache = (http_cache_entry_t **) hash_values (http_cache)) != NULL)
+  if ((cache = (http_cache_entry_t **) svz_hash_values (http_cache)) != NULL)
     {
       /* go through each cache entry */
-      for (n = 0; n < hash_size (http_cache); n++)
+      for (n = 0; n < svz_hash_size (http_cache); n++)
 	{
 	  files++;
 	  total += cache[n]->size;
@@ -625,19 +586,19 @@ ctrl_stat_cache (socket_t sock, int flag, char *arg)
 	  p += strlen (cache[n]->file);
 	  while (*p != '/' && *p != '\\' && p != cache[n]->file) p--;
 	  if (p != cache[n]->file) p++;
-	  sock_printf (sock, "%-30s %6d %6d %5d %6d %-5s\r\n", p,
-		       cache[n]->size,
-		       cache[n]->usage,
-		       cache[n]->hits,
-		       cache[n]->urgent,
-		       cache[n]->ready ? "Yes" : "No");
+	  svz_sock_printf (sock, "%-30s %6d %6d %5d %6d %-5s\r\n", p,
+			   cache[n]->size,
+			   cache[n]->usage,
+			   cache[n]->hits,
+			   cache[n]->urgent,
+			   cache[n]->ready ? "Yes" : "No");
 	}
-      hash_xfree (cache);
+      svz_hash_xfree (cache);
     }
 
   /* print cache summary */
-  sock_printf (sock, "\r\nTotal : %d byte in %d cache entries\r\n\r\n",
-	       total, files);
+  svz_sock_printf (sock, "\r\nTotal : %d byte in %d cache entries\r\n\r\n",
+		   total, files);
 
   return flag;
 }
@@ -646,10 +607,10 @@ ctrl_stat_cache (socket_t sock, int flag, char *arg)
  * Free all HTTP cache entries.
  */
 int
-ctrl_kill_cache (socket_t sock, int flag, char *arg)
+ctrl_kill_cache (svz_socket_t *sock, int flag, char *arg)
 {
-  sock_printf (sock, "%d HTTP cache entries reinitialized.\r\n",
-	       http_cache_entries);
+  svz_sock_printf (sock, "%d HTTP cache entries reinitialized.\r\n",
+		   http_cache_entries);
   http_free_cache ();
   http_alloc_cache (http_cache_entries);
   return flag;
@@ -660,31 +621,30 @@ ctrl_kill_cache (socket_t sock, int flag, char *arg)
  * Show all Co-Server instances statistics.
  */
 int
-ctrl_stat_coservers (socket_t sock, int flag, char *arg)
+ctrl_stat_coservers (svz_socket_t *sock, int flag, char *arg)
 {
   int n;
-  int_coserver_t *coserver;
+  svz_coserver_t *coserver;
 
   /* go through all internal coserver instances */
-  for (n = 0; n < int_coservers; n++)
+  svz_array_foreach (svz_coservers, coserver, n)
     {
-      coserver = int_coserver[n];
-      sock_printf (sock, "\r\ninternal %s coserver:\r\n",
-		   int_coserver_type[coserver->type].name);
-      sock_printf (sock, 
-		   " socket id  : %d\r\n"
-		   " %s %d\r\n"
-		   " requests   : %d\r\n",
-		   coserver->sock->id,
+      svz_sock_printf (sock, "\r\ninternal %s coserver:\r\n",
+		       svz_coservertypes[coserver->type].name);
+      svz_sock_printf (sock, 
+		       " socket id  : %d\r\n"
+		       " %s %d\r\n"
+		       " requests   : %d\r\n",
+		       coserver->sock->id,
 #ifndef __MINGW32__
-		   "process id :", coserver->pid,
+		       "process id :", coserver->pid,
 #else /* __MINGW32__ */
-		   "thread id  :", coserver->tid,
+		       "thread id  :", coserver->tid,
 #endif /* __MINGW32__ */
-		   coserver->busy);
+		       coserver->busy);
     }
 
-  sock_printf (sock, "\r\n");
+  svz_sock_printf (sock, "\r\n");
   return flag;
 }
 
@@ -692,20 +652,19 @@ ctrl_stat_coservers (socket_t sock, int flag, char *arg)
  * Server and Co-Server instance statistics.
  */
 int
-ctrl_stat_all (socket_t sock, int flag, char *arg)
+ctrl_stat_all (svz_socket_t *sock, int flag, char *arg)
 {
   int n;
-  server_t *server;
+  svz_server_t **server;
 
   /* go through all server instances */
-  for (n = 0; n < server_instances; n++)
+  svz_hash_foreach_value (svz_servers, server, n)
     {
-      server = servers[n];
-      sock_printf (sock, "\r\n%s (%s):\r\n",
-		   server->description, server->name);
-      if (server->info_server)
+      svz_sock_printf (sock, "\r\n%s (%s):\r\n",
+		       server[n]->description, server[n]->name);
+      if (server[n]->info_server)
 	{
-	  sock_printf (sock, "%s\r\n", server->info_server (server));
+	  svz_sock_printf (sock, "%s\r\n", server[n]->info_server (server[n]));
 	}
     }
 
@@ -721,20 +680,20 @@ ctrl_stat_all (socket_t sock, int flag, char *arg)
  * So you want to be *very* careful with this command.
  */
 int
-ctrl_kill_id (socket_t sock, int flag, char *arg)
+ctrl_kill_id (svz_socket_t *sock, int flag, char *arg)
 {
   int id;
-  socket_t xsock;
+  svz_socket_t *xsock;
 
   id = atoi (arg);
-  if ((xsock = sock_find (id, -1)) == NULL)
+  if ((xsock = svz_sock_find (id, -1)) == NULL)
     {
-      sock_printf (sock, "no such connection: %d\r\n", id);
+      svz_sock_printf (sock, "no such connection: %d\r\n", id);
       return flag;
     }
 
-  sock_schedule_for_shutdown (xsock);
-  sock_printf (sock, "scheduled socket id %d for shutdown\r\n", id);
+  svz_sock_schedule_for_shutdown (xsock);
+  svz_sock_printf (sock, "scheduled socket id %d for shutdown\r\n", id);
   return flag;
 }
 
@@ -743,22 +702,22 @@ ctrl_kill_id (socket_t sock, int flag, char *arg)
  * coservers and sockets with the priority flag set.
  */
 int
-ctrl_killall (socket_t sock, int flag, char *arg)
+ctrl_killall (svz_socket_t *sock, int flag, char *arg)
 {
-  socket_t xsock;
+  svz_socket_t *xsock;
   int n = 0;
 
-  for (xsock = socket_root; xsock; xsock = xsock->next)
+  svz_sock_foreach (xsock)
     {
       if (xsock != sock &&
 	  !(xsock->flags & (SOCK_FLAG_LISTENING | SOCK_FLAG_COSERVER | 
 			    SOCK_FLAG_PRIORITY)))
 	{
-	  sock_schedule_for_shutdown (xsock);
+	  svz_sock_schedule_for_shutdown (xsock);
 	  n++;
 	}
     }
-  sock_printf (sock, "killed %d network connections\r\n", n);
+  svz_sock_printf (sock, "killed %d network connections\r\n", n);
 
   return flag;
 }
@@ -767,29 +726,28 @@ ctrl_killall (socket_t sock, int flag, char *arg)
  * Restart coservers.
  */
 int
-ctrl_restart (socket_t sock, int type, char *arg)
+ctrl_restart (svz_socket_t *sock, int type, char *arg)
 {
-  int_coserver_t *coserver;
+  svz_coserver_t *coserver;
   int n;
 
   /* find an appropriate coserver to kill */
-  for (n = 0; n < int_coservers; n++)
+  svz_array_foreach (svz_coservers, coserver, n)
     {
-      coserver = int_coserver[n];
       if (coserver->type == type)
 	{
-	  coserver_destroy (type);
-	  coserver_create (type);
-	  sock_printf (sock, "internal %s coserver restarted\r\n",
-		       int_coserver_type[type].name);
+	  svz_coserver_destroy (type);
+	  svz_coserver_create (type);
+	  svz_sock_printf (sock, "internal %s coserver restarted\r\n",
+			   svz_coservertypes[type].name);
 	  return 0;
 	}
     }
 
   /* start a new internal coserver if there has none found */
-  coserver_create (type);
-  sock_printf (sock, "internal %s coserver invoked\r\n",
-	       int_coserver_type[type].name);
+  svz_coserver_create (type);
+  svz_sock_printf (sock, "internal %s coserver invoked\r\n",
+		   svz_coservertypes[type].name);
   return 0;
 }
 
@@ -799,9 +757,9 @@ ctrl_restart (socket_t sock, int type, char *arg)
  */
 struct
 {
-  char *command;                      /* the complete command string */
-  int (*func)(socket_t, int, char *); /* callback routine */
-  int flag;                           /* second argument */
+  char *command;                            /* the complete command string */
+  int (*func)(svz_socket_t *, int, char *); /* callback routine */
+  int flag;                                 /* second argument */
 }
 ctrl[] =
 {
@@ -820,15 +778,9 @@ ctrl[] =
   { CTRL_CMD_STAT,          ctrl_stat, 0 },
   { CTRL_CMD_KILLALL,       ctrl_killall, 0 },
   { CTRL_CMD_KILL_ID,       ctrl_kill_id, 0 },
-#if ENABLE_REVERSE_LOOKUP
   { CTRL_CMD_RESTART_RDNS,  ctrl_restart, COSERVER_REVERSE_DNS },
-#endif
-#if ENABLE_IDENT
   { CTRL_CMD_RESTART_IDENT, ctrl_restart, COSERVER_IDENT },
-#endif
-#if ENABLE_REVERSE_LOOKUP
   { CTRL_CMD_RESTART_DNS,   ctrl_restart, COSERVER_DNS },
-#endif
   { NULL, NULL, 0 }
 };
 
@@ -837,7 +789,7 @@ ctrl[] =
  * routine of the control protocol.
  */
 int
-ctrl_handle_request (socket_t sock, char *request, int len)
+ctrl_handle_request (svz_socket_t *sock, char *request, int len)
 {
   static char last_request[CTRL_RECV_BUFSIZE];
   static int last_len;
@@ -858,15 +810,16 @@ ctrl_handle_request (socket_t sock, char *request, int len)
       if (len <= 2) return -1;
 #if ENABLE_CRYPT && HAVE_CRYPT
       request[len] = '\0';
-      if (!strcmp (crypt (request, serveez_config.server_password), 
-		   serveez_config.server_password))
+      if (svz_config.password == NULL ||
+	  !strcmp (crypt (request, svz_config.password), svz_config.password))
 #else
-      if (!memcmp (request, serveez_config.server_password, len) &&
-	  (unsigned) len >= strlen (serveez_config.server_password))
+      if (svz_config.password == NULL ||
+	  (!memcmp (request, svz_config.password, len) &&
+	   (unsigned) len >= strlen (svz_config.password)))
 #endif
 	{
 	  sock->userflags |= CTRL_FLAG_PASSED;
-	  sock_printf (sock, "Login ok.\r\n%s", CTRL_PROMPT);
+	  svz_sock_printf (sock, "Login ok.\r\n%s", CTRL_PROMPT);
 	}
       else return -1;
     }
@@ -890,7 +843,7 @@ ctrl_handle_request (socket_t sock, char *request, int len)
 
 	      /* execute valid command and give the prompt */
 	      ret = ctrl[n].func (sock, ctrl[n].flag, &request[l + 1]);
-	      sock_printf (sock, "%s", CTRL_PROMPT);
+	      svz_sock_printf (sock, "%s", CTRL_PROMPT);
 	      return ret;
 	    }
 	  n++;
@@ -898,12 +851,12 @@ ctrl_handle_request (socket_t sock, char *request, int len)
       l = 0;
       while (l < len && request[l] >= ' ') l++;
       request[l] = 0;
-      sock_printf (sock, "no such command: %s\r\n", request);
-      sock_printf (sock, "%s", CTRL_PROMPT);
+      svz_sock_printf (sock, "no such command: %s\r\n", request);
+      svz_sock_printf (sock, "%s", CTRL_PROMPT);
     }
   else
     {
-      sock_printf (sock, "%s", CTRL_PROMPT);
+      svz_sock_printf (sock, "%s", CTRL_PROMPT);
     }
   return ret;
 }
@@ -966,8 +919,8 @@ ctrl_get_cpu_state (void)
     {
       if (kstat_read (kc, ksp, &cs) == -1) 
 	{
-	  snprintf (cpu_state.info, STAT_BUFFER_SIZE, 
-		    "kstat_read() failed");
+	  svz_snprintf (cpu_state.info, STAT_BUFFER_SIZE, 
+			"kstat_read() failed");
 	  return -1;
 	}
 
@@ -980,10 +933,10 @@ ctrl_get_cpu_state (void)
 #elif HAVE_PROC_STAT /* Linux */
 
   /* open the statistics file */
-  if ((f = fopen (cpu_state.cpufile, "r")) == NULL)
+  if ((f = svz_fopen (cpu_state.cpufile, "r")) == NULL)
     {
-      snprintf (cpu_state.info, STAT_BUFFER_SIZE, 
-		"%s not available", cpu_state.cpufile);
+      svz_snprintf (cpu_state.info, STAT_BUFFER_SIZE, 
+		    "%s not available", cpu_state.cpufile);
       return -1;
     }
 
@@ -996,15 +949,15 @@ ctrl_get_cpu_state (void)
 		       &cpu_state.cpu[n][2], 
 		       &cpu_state.cpu[n][3]))
 	{
-	  fclose (f);
+	  svz_fclose (f);
 	  return 0;
 	}
     }
 
   /* cpu line not found */
-  snprintf (cpu_state.info, STAT_BUFFER_SIZE, 
-	    "cpu line not found in %s", cpu_state.cpufile);
-  fclose (f);
+  svz_snprintf (cpu_state.info, STAT_BUFFER_SIZE, 
+		"cpu line not found in %s", cpu_state.cpufile);
+  svz_fclose (f);
 
 #elif HAVE_PSTAT /* HP Unix */
 
@@ -1038,7 +991,7 @@ ctrl_get_cpu_state (void)
 #define CPU_DIFF(x) (c->cpu[n][x] - c->cpu[old][x])
 
 int
-ctrl_idle (socket_t sock)
+ctrl_idle (svz_socket_t *sock)
 {
   int n, old;
   unsigned long all;
@@ -1054,15 +1007,15 @@ ctrl_idle (socket_t sock)
       all = c->ptotal[n] - c->ptotal[old]; 
       if (all != 0)
 	{
-	  snprintf (c->pinfo, STAT_BUFFER_SIZE, PROC_FORMAT,
-		    PROC_DIFF (0) * 100 / all,
-		    PROC_DIFF (0) * 1000 / all % 10,
-		    PROC_DIFF (1) * 100 / all,
-		    PROC_DIFF (1) * 1000 / all % 10,
-		    PROC_DIFF (2) * 100 / all,
-		    PROC_DIFF (2) * 1000 / all % 10,
-		    PROC_DIFF (3) * 100 / all,
-		    PROC_DIFF (3) * 1000 / all % 10);
+	  svz_snprintf (c->pinfo, STAT_BUFFER_SIZE, PROC_FORMAT,
+			PROC_DIFF (0) * 100 / all,
+			PROC_DIFF (0) * 1000 / all % 10,
+			PROC_DIFF (1) * 100 / all,
+			PROC_DIFF (1) * 1000 / all % 10,
+			PROC_DIFF (2) * 100 / all,
+			PROC_DIFF (2) * 1000 / all % 10,
+			PROC_DIFF (3) * 100 / all,
+			PROC_DIFF (3) * 1000 / all % 10);
 	}
 
       /* calculate cpu specific info */
@@ -1070,15 +1023,15 @@ ctrl_idle (socket_t sock)
       all = c->total[n] - c->total[old];
       if (all != 0)
 	{
-	  snprintf (c->info, STAT_BUFFER_SIZE, c->cpuinfoline,
-		    CPU_DIFF (0) * 100 / all,
-		    CPU_DIFF (0) * 1000 / all % 10,
-		    CPU_DIFF (1) * 100 / all,
-		    CPU_DIFF (1) * 1000 / all % 10,
-		    CPU_DIFF (2) * 100 / all,
-		    CPU_DIFF (2) * 1000 / all % 10,
-		    CPU_DIFF (3) * 100 / all,
-		    CPU_DIFF (3) * 1000 / all % 10);
+	  svz_snprintf (c->info, STAT_BUFFER_SIZE, c->cpuinfoline,
+			CPU_DIFF (0) * 100 / all,
+			CPU_DIFF (0) * 1000 / all % 10,
+			CPU_DIFF (1) * 100 / all,
+			CPU_DIFF (1) * 1000 / all % 10,
+			CPU_DIFF (2) * 100 / all,
+			CPU_DIFF (2) * 1000 / all % 10,
+			CPU_DIFF (3) * 100 / all,
+			CPU_DIFF (3) * 1000 / all % 10);
 	}
     }
   
