@@ -1,7 +1,7 @@
 /*
  * http-cgi.c - http cgi implementation
  *
- * Copyright (C) 2000 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2000, 2001 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: http-cgi.c,v 1.30 2001/01/03 20:02:42 ela Exp $
+ * $Id: http-cgi.c,v 1.45 2001/07/05 21:35:26 ela Exp $
  *
  */
 
@@ -46,7 +46,7 @@
 #endif
 
 #ifdef __MINGW32__
-# include <winsock.h>
+# include <winsock2.h>
 # include <io.h>
 # include <shellapi.h>
 #endif
@@ -61,11 +61,7 @@
 # endif
 #endif
 
-#include "snprintf.h"
-#include "util.h"
-#include "pipe-socket.h"
-#include "alloc.h"
-#include "serveez.h"
+#include "libserveez.h"
 #include "http-proto.h"
 #include "http-core.h"
 #include "http-cgi.h"
@@ -76,7 +72,7 @@
  * functionality.
  */
 int
-http_cgi_disconnect (socket_t sock)
+http_cgi_disconnect (svz_socket_t *sock)
 {
   http_socket_t *http = sock->data;
 
@@ -89,14 +85,14 @@ http_cgi_disconnect (socket_t sock)
   if (sock->pipe_desc[READ] != INVALID_HANDLE)
     {
       if (closehandle (sock->pipe_desc[READ]) == -1)
-	log_printf (LOG_ERROR, "close: %s\n", SYS_ERROR);
+	svz_log (LOG_ERROR, "close: %s\n", SYS_ERROR);
       sock->pipe_desc[READ] = INVALID_HANDLE;
       sock->flags &= ~SOCK_FLAG_RECV_PIPE;
     }
   if (sock->pipe_desc[WRITE] != INVALID_HANDLE)
     {
       if (closehandle (sock->pipe_desc[WRITE]) == -1)
-	log_printf (LOG_ERROR, "close: %s\n", SYS_ERROR);
+	svz_log (LOG_ERROR, "close: %s\n", SYS_ERROR);
       sock->pipe_desc[WRITE] = INVALID_HANDLE;
       sock->flags &= ~SOCK_FLAG_SEND_PIPE;
     }
@@ -108,9 +104,9 @@ http_cgi_disconnect (socket_t sock)
   if (http->pid != INVALID_HANDLE)
     {
       if (!TerminateProcess (http->pid, 0))
-	log_printf (LOG_ERROR, "TerminateProcess: %s\n", SYS_ERROR);
+	svz_log (LOG_ERROR, "TerminateProcess: %s\n", SYS_ERROR);
       if (closehandle (http->pid) == -1)
-	log_printf (LOG_ERROR, "CloseHandle: %s\n", SYS_ERROR);
+	svz_log (LOG_ERROR, "CloseHandle: %s\n", SYS_ERROR);
       http->pid = INVALID_HANDLE;
     }
 #else /* not __MINGW32__ */
@@ -120,11 +116,12 @@ http_cgi_disconnect (socket_t sock)
   if (http->pid != INVALID_HANDLE)
     {
       if (kill (http->pid, SIGKILL) == -1)
-	log_printf (LOG_ERROR, "kill: %s\n", SYS_ERROR);
+	svz_log (LOG_ERROR, "kill: %s\n", SYS_ERROR);
 #if HAVE_WAITPID
       /* Test if the cgi is still running and cleanup. */
-      waitpid (http->pid, NULL, 0);
-#endif
+      else if (waitpid (http->pid, NULL, 0) == -1)
+	svz_log (LOG_ERROR, "waitpid: %s\n", SYS_ERROR);
+#endif /* not HAVE_WAITPID */
       http->pid = INVALID_HANDLE;
     }
 #endif /* not __MINGW32__ */
@@ -132,6 +129,66 @@ http_cgi_disconnect (socket_t sock)
   return http_disconnect (sock);
 }
 
+/*
+ * This is the default idle function for http connections. It checks 
+ * whether any died child was a cgi script.
+ */
+int
+http_cgi_died (svz_socket_t *sock)
+{
+  http_socket_t *http = sock->data;
+#ifdef __MINGW32__
+  DWORD result;
+#endif
+
+  if (sock->flags & SOCK_FLAG_PIPE)
+    {
+#ifndef __MINGW32__
+      /* Check if a died child is this cgi. */
+      if (svz_child_died && http->pid == svz_child_died)
+	{
+	  svz_log (LOG_NOTICE, "cgi script pid %d died\n", 
+		   (int) svz_child_died);
+	  svz_child_died = 0;
+	}
+#if HAVE_WAITPID
+      /* Test if the cgi is still running. */
+      if (waitpid (http->pid, NULL, WNOHANG) == http->pid)
+	{
+	  svz_log (LOG_NOTICE, "cgi script pid %d died\n", 
+		   (int) http->pid);
+	  http->pid = INVALID_HANDLE;
+	}
+#endif /* HAVE_WAITPID */
+
+#else /* __MINGW32__ */
+
+      /*
+       * Check if there died a process handle in Win32, this has to be
+       * done regularly here because there is no SIGCHLD in Win32 !
+       */
+      if (http->pid != INVALID_HANDLE)
+	{
+	  result = WaitForSingleObject (http->pid, LEAST_WAIT_OBJECT);
+	  if (result == WAIT_FAILED)
+	    {
+	      svz_log (LOG_ERROR, "WaitForSingleObject: %s\n", SYS_ERROR);
+	    }
+	  else if (result != WAIT_TIMEOUT)
+	    {
+	      if (closehandle (http->pid) == -1)
+		svz_log (LOG_ERROR, "CloseHandle: %s\n", SYS_ERROR);
+	      svz_child_died = http->pid;
+	      http->pid = INVALID_HANDLE;
+	    }
+	}
+#endif /* __MINGW32__ */
+    }
+  
+  sock->idle_counter = 1;
+  return 0;
+}
+
 /*
  * The http cgi reader gets data from the stdout of a cgi
  * program and stores the data into the send buffer of
@@ -139,7 +196,7 @@ http_cgi_disconnect (socket_t sock)
  * to indicate there was no more data.
  */
 int
-http_cgi_read (socket_t sock)
+http_cgi_read (svz_socket_t *sock)
 {
   int do_read;
   int num_read;
@@ -157,7 +214,7 @@ http_cgi_read (socket_t sock)
   if (!PeekNamedPipe (sock->pipe_desc[READ], NULL, 0, 
 		      NULL, (DWORD *) &num_read, NULL))
     {
-      log_printf (LOG_ERROR, "cgi: PeekNamedPipe: %s\n", SYS_ERROR);
+      svz_log (LOG_ERROR, "cgi: PeekNamedPipe: %s\n", SYS_ERROR);
       return -1;
     }
 
@@ -170,7 +227,7 @@ http_cgi_read (socket_t sock)
 		 sock->send_buffer + sock->send_buffer_fill,
 		 do_read, (DWORD *) &num_read, NULL))
     {
-      log_printf (LOG_ERROR, "cgi: ReadFile: %s\n", SYS_ERROR);
+      svz_log (LOG_ERROR, "cgi: ReadFile: %s\n", SYS_ERROR);
       num_read = -1;
     }
 #else /* not __MINGW32__ */
@@ -178,7 +235,9 @@ http_cgi_read (socket_t sock)
 			sock->send_buffer + sock->send_buffer_fill,
 			do_read)) == -1)
     {
-      log_printf (LOG_ERROR, "cgi: read: %s\n", SYS_ERROR);
+      svz_log (LOG_ERROR, "cgi: read: %s\n", SYS_ERROR);
+      if (svz_errno == EAGAIN)
+	return 0;
       num_read = -1;
     }
 #endif /* not __MINGW32__ */
@@ -207,7 +266,7 @@ http_cgi_read (socket_t sock)
   if (sock->send_buffer_fill == 0)
     {
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "cgi: data successfully received and resent\n");
+      svz_log (LOG_DEBUG, "cgi: data successfully received and resent\n");
 #endif
       sock->userflags &= ~HTTP_FLAG_CGI;
       sock->flags &= ~SOCK_FLAG_RECV_PIPE;
@@ -223,7 +282,7 @@ http_cgi_read (socket_t sock)
  * It directly reads from the RECV_BUFFER of the socket structure.
  */
 int
-http_cgi_write (socket_t sock)
+http_cgi_write (svz_socket_t *sock)
 {
   int do_write;
   int num_written;
@@ -242,14 +301,14 @@ http_cgi_write (socket_t sock)
   if (!WriteFile (sock->pipe_desc[WRITE], sock->recv_buffer, 
 		  do_write, (DWORD *) &num_written, NULL))
     {
-      log_printf (LOG_ERROR, "cgi: WriteFile: %s\n", SYS_ERROR);
+      svz_log (LOG_ERROR, "cgi: WriteFile: %s\n", SYS_ERROR);
       num_written = -1;
     }
 #else /* !__MINGW32__ */
   if ((num_written = write (sock->pipe_desc[WRITE], 
 			    sock->recv_buffer, do_write)) == -1)
     {
-      log_printf (LOG_ERROR, "cgi: write: %s\n", SYS_ERROR);
+      svz_log (LOG_ERROR, "cgi: write: %s\n", SYS_ERROR);
     }
 #endif /* !__MINGW32__ */
 
@@ -280,7 +339,7 @@ http_cgi_write (socket_t sock)
   if (http->contentlength <= 0)
     {
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "cgi: post data sent to cgi\n");
+      svz_log (LOG_DEBUG, "cgi: post data sent to cgi\n");
 #endif
       sock->userflags &= ~HTTP_FLAG_POST;
       sock->flags &= ~SOCK_FLAG_SEND_PIPE;
@@ -301,7 +360,7 @@ http_cgi_write (socket_t sock)
  * we really do it depends on the system we compile with.
  */
 static void
-http_insert_env (ENV_BLOCK_TYPE env, /* the block to add the variable to */
+http_insert_env (svz_envblock_t env, /* the block to add the variable to */
 		 int *length,        /* pointer to the current length of it */
 		 char *fmt,          /* format string */
 		 ...)                /* arguments for the format string */
@@ -313,25 +372,25 @@ http_insert_env (ENV_BLOCK_TYPE env, /* the block to add the variable to */
   /* Windows: */
   if (*length >= ENV_BLOCK_SIZE)
     {
-      log_printf (LOG_WARNING, "cgi: env block size %d reached\n", 
-		  ENV_BLOCK_SIZE);
+      svz_log (LOG_WARNING, "cgi: env block size %d reached\n", 
+	       ENV_BLOCK_SIZE);
     }
   else
     {
-      vsnprintf (&env[*length], ENV_BLOCK_SIZE - (*length), fmt, args);
+      svz_vsnprintf (&env[*length], ENV_BLOCK_SIZE - (*length), fmt, args);
       *length += strlen (&env[*length]) + 1;
     }
 #else
   /* Unices: */
   if (*length >= ENV_ENTRIES - 1)
     {
-      log_printf (LOG_WARNING, "cgi: all env entries %d filled\n", 
-		  ENV_ENTRIES);
+      svz_log (LOG_WARNING, "cgi: all env entries %d filled\n", 
+	       ENV_ENTRIES);
     }
   else
     {
-      env[*length] = xmalloc (ENV_LENGTH);
-      vsnprintf (env[*length], ENV_LENGTH, fmt, args);
+      env[*length] = svz_malloc (ENV_LENGTH);
+      svz_vsnprintf (env[*length], ENV_LENGTH, fmt, args);
       (*length)++;
     }
 #endif
@@ -347,8 +406,8 @@ http_insert_env (ENV_BLOCK_TYPE env, /* the block to add the variable to */
  * or the size of the block in bytes.
  */
 static int
-http_create_cgi_envp (socket_t sock,      /* socket for this request */
-		      ENV_BLOCK_TYPE env, /* env block */
+http_create_cgi_envp (svz_socket_t *sock, /* socket for this request */
+		      svz_envblock_t env, /* env block */
 		      char *script,       /* the cgi script's filename */
 		      int type)           /* the cgi type */
 {
@@ -393,7 +452,7 @@ http_create_cgi_envp (socket_t sock,      /* socket for this request */
   if (http->property)
     for (c = 0; env_var[c].property; c++)
       for (n = 0; http->property[n]; n += 2)
-	if (!util_strcasecmp (http->property[n], env_var[c].property))
+	if (!svz_strcasecmp (http->property[n], env_var[c].property))
 	  {
 	    http_insert_env (env, &size, "%s=%s", 
 			     env_var[c].env, http->property[n + 1]);
@@ -405,17 +464,16 @@ http_create_cgi_envp (socket_t sock,      /* socket for this request */
    * necessary for the cgi script
    */
   http_insert_env (env, &size, "SERVER_NAME=%s", 
-		   cfg->host ? cfg->host : util_inet_ntoa (sock->local_addr));
+		   cfg->host ? cfg->host : svz_inet_ntoa (sock->local_addr));
   http_insert_env (env, &size, "SERVER_PORT=%u", ntohs (sock->local_port));
   http_insert_env (env, &size, "REMOTE_ADDR=%s", http->host ? http->host :
-		   util_inet_ntoa (sock->remote_addr));
+		   svz_inet_ntoa (sock->remote_addr));
   http_insert_env (env, &size, "REMOTE_PORT=%u", ntohs (sock->remote_port));
   http_insert_env (env, &size, "SCRIPT_NAME=%s%s", cfg->cgiurl, script);
   http_insert_env (env, &size, "GATEWAY_INTERFACE=%s", CGI_VERSION);
   http_insert_env (env, &size, "SERVER_PROTOCOL=%s", HTTP_VERSION);
   http_insert_env (env, &size, "SERVER_SOFTWARE=%s/%s", 
-		   serveez_config.program_name, 
-		   serveez_config.version_string);
+		   svz_library, svz_version);
   http_insert_env (env, &size, "REQUEST_METHOD=%s", request_type[type]);
 
 #ifdef __MINGW32__
@@ -436,10 +494,10 @@ http_create_cgi_envp (socket_t sock,      /* socket for this request */
  * parses the text of the request and delivers the real file to be
  * invoked. This function makes sure that the cgi script file exists
  * and is executable. On success it delivers a pointer which must be
- * xfree()ed after use.
+ * svz_free()ed after use.
  */
 char *
-http_check_cgi (socket_t sock, char *request)
+http_check_cgi (svz_socket_t *sock, char *request)
 {
 #ifndef __MINGW32__
   struct stat buf;
@@ -466,7 +524,7 @@ http_check_cgi (socket_t sock, char *request)
 
   /* store the request in a local variable */
   len = strlen (request) + 1 - strlen (cfg->cgiurl);
-  saverequest = xmalloc (len);
+  saverequest = svz_malloc (len);
   strcpy (saverequest, request + strlen (cfg->cgiurl));
 
   /* find the actual URL */
@@ -476,15 +534,15 @@ http_check_cgi (socket_t sock, char *request)
   *p = 0;
 
   size = strlen (cfg->cgidir) + len;
-  file = xmalloc (size);
+  file = svz_malloc (size);
   sprintf (file, "%s%s", cfg->cgidir, saverequest);
 
   /* test if the file really exists and close it again */
   if ((fd = open (file, O_RDONLY)) == -1)
     {
-      log_printf (LOG_ERROR, "cgi: open: %s (%s)\n", SYS_ERROR, file);
-      xfree (file);
-      xfree (saverequest);
+      svz_log (LOG_ERROR, "cgi: open: %s (%s)\n", SYS_ERROR, file);
+      svz_free (file);
+      svz_free (saverequest);
       return NULL;
     }
 
@@ -492,30 +550,30 @@ http_check_cgi (socket_t sock, char *request)
   /* test the file being an executable */
   if (fstat (fd, &buf) == -1)
     {
-      log_printf (LOG_ERROR, "cgi: fstat: %s\n", SYS_ERROR);
+      svz_log (LOG_ERROR, "cgi: fstat: %s\n", SYS_ERROR);
       close (fd);
-      xfree (file);
-      xfree (saverequest);
+      svz_free (file);
+      svz_free (saverequest);
       return NULL;
     }
 
   if (!(buf.st_mode & S_IFREG) ||
       !(buf.st_mode & S_IXUSR) || !(buf.st_mode & S_IRUSR))  
     {
-      log_printf (LOG_ERROR, "cgi: no executable: %s\n", file);
+      svz_log (LOG_ERROR, "cgi: no executable: %s\n", file);
       close (fd);
-      xfree (file);
-      xfree (saverequest);
+      svz_free (file);
+      svz_free (saverequest);
       return NULL;
     }
 #endif
   if (close (fd) == -1)
-    log_printf (LOG_ERROR, "cgi: close: %s\n", SYS_ERROR);
+    svz_log (LOG_ERROR, "cgi: close: %s\n", SYS_ERROR);
 
   /* return a pointer referring to the actual plain cgi file */
   strcpy (file, saverequest);
-  file = xrealloc (file, strlen (file) + 1);
-  xfree (saverequest);
+  file = svz_realloc (file, strlen (file) + 1);
+  svz_free (saverequest);
   return file;
 }
 
@@ -526,8 +584,8 @@ http_check_cgi (socket_t sock, char *request)
  * cgi file (including the path). This MUST be freed afterwards.
  */
 char *
-http_pre_exec (socket_t sock,       /* socket structure */
-	       ENV_BLOCK_TYPE envp, /* environment block to be filled */
+http_pre_exec (svz_socket_t *sock,  /* socket structure */
+	       svz_envblock_t envp, /* environment block to be filled */
 	       char *file,          /* plain executable name */
 	       char *request,       /* original http request */
 	       int type)            /* POST or GET ? */
@@ -541,28 +599,28 @@ http_pre_exec (socket_t sock,       /* socket structure */
   /* change into the CGI directory temporarily */
   if (chdir (cfg->cgidir) == -1)
     {
-      log_printf (LOG_ERROR, "cgi: chdir: %s\n", SYS_ERROR);
+      svz_log (LOG_ERROR, "cgi: chdir: %s\n", SYS_ERROR);
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "cgi: cannot change dir: %s\n", cfg->cgidir);
+      svz_log (LOG_DEBUG, "cgi: cannot change dir: %s\n", cfg->cgidir);
 #endif
       return NULL;
     }
 
   /* reserve buffer for the directory */
-  cgidir = xmalloc (MAX_CGI_DIR_LEN);
+  cgidir = svz_malloc (MAX_CGI_DIR_LEN);
 
   /* get the current directory and concate the cgifile */
   if (getcwd (cgidir, MAX_CGI_DIR_LEN) == NULL)
     {
-      log_printf (LOG_ERROR, "cgi: getcwd: %s\n", SYS_ERROR);
-      xfree (cgidir);
+      svz_log (LOG_ERROR, "cgi: getcwd: %s\n", SYS_ERROR);
+      svz_free (cgidir);
       return NULL;
     }
   
   /* put the directory and file together */
-  cgifile = xmalloc (strlen (cgidir) + strlen (file) + 1);
+  cgifile = svz_malloc (strlen (cgidir) + strlen (file) + 1);
   sprintf (cgifile, "%s%s", cgidir, file);
-  xfree (cgidir);
+  svz_free (cgidir);
 
   /* create the environment block for the CGI script */
   size = http_create_cgi_envp (sock, envp, file, type);
@@ -589,12 +647,17 @@ http_pre_exec (socket_t sock,       /* socket structure */
  * right after the the actual CGI script has been invoked.
  */
 int
-http_cgi_accepted (socket_t sock)
+http_cgi_accepted (svz_socket_t *sock)
 {
   http_socket_t *http = sock->data;
 
   http->response = 202;
-  return sock_printf (sock, HTTP_ACCEPTED);
+  return svz_sock_printf (sock, HTTP_OK
+			  "Date: %s\r\n"
+			  "Server: %s/%s\r\n"
+			  "Connection: close\r\n",
+			  http_asc_date (time (NULL)),
+			  svz_library, svz_version);
 }
 
 /*
@@ -604,22 +667,17 @@ http_cgi_accepted (socket_t sock)
 void
 http_free_cgi_apps (http_config_t *cfg)
 {
+#if 0
+  /* FIXME: Is hat necessary ? */
   char **app;
   int n;
 
-  if (*(cfg->cgiapps))
-    {
-      if ((app = (char **) hash_values (*(cfg->cgiapps))) != NULL)
-	{
-	  for (n = 0; n < hash_size (*(cfg->cgiapps)); n++)
-	    {
-	      xfree (app[n]);
-	    }
-	  hash_xfree (app);
-	}
-      hash_destroy (*(cfg->cgiapps));
-      *(cfg->cgiapps) = NULL;
-    }
+  svz_hash_foreach_value (cfg->cgiapps, app, n)
+    svz_free (app[n]);
+      
+  svz_hash_destroy (cfg->cgiapps);
+  cfg->cgiapps = NULL;
+#endif
 }
 
 /*
@@ -629,16 +687,22 @@ http_free_cgi_apps (http_config_t *cfg)
 void
 http_gen_cgi_apps (http_config_t *cfg)
 {
+  char *p;
+
   /* create the cgi association hash table if necessary */
-  if (*(cfg->cgiapps) == NULL)
-    {
-      *(cfg->cgiapps) = hash_create (4);
-    }
+  if (cfg->cgiapps == NULL)
+    cfg->cgiapps = svz_hash_create (4);
 
   /* the associations need to be in the hash to be executed at all */
-  hash_put (*(cfg->cgiapps), "exe", xstrdup (DEFAULT_CGIAPP));
-  hash_put (*(cfg->cgiapps), "com", xstrdup (DEFAULT_CGIAPP));
-  hash_put (*(cfg->cgiapps), "bat", xstrdup (DEFAULT_CGIAPP));
+  if ((p = svz_hash_put (cfg->cgiapps, "exe", svz_strdup (DEFAULT_CGIAPP))) 
+      != NULL)
+    svz_free (p);
+  if ((p = svz_hash_put (cfg->cgiapps, "com", svz_strdup (DEFAULT_CGIAPP)))
+      != NULL)
+    svz_free (p);
+  if ((p = svz_hash_put (cfg->cgiapps, "bat", svz_strdup (DEFAULT_CGIAPP)))
+      != NULL)
+    svz_free (p);
 }
 
 /*
@@ -646,7 +710,7 @@ http_gen_cgi_apps (http_config_t *cfg)
  * CreateProcess().
  */
 int
-http_cgi_exec (socket_t sock,  /* the socket structure */
+http_cgi_exec (svz_socket_t *sock, /* the socket structure */
 	       HANDLE in,      /* here the cgi reads from or NULL if GET */
 	       HANDLE out,     /* here the cgi writes to */
 	       char *file,     /* cgi script file */
@@ -656,9 +720,9 @@ http_cgi_exec (socket_t sock,  /* the socket structure */
   HANDLE pid;    /* the pid from fork() or the process handle in Win32 */
   char *cgifile; /* path including the name of the cgi script */
   http_socket_t *http;
-  http_config_t *cfg = sock->cfg;
 
 #ifdef __MINGW32__
+  http_config_t *cfg = sock->cfg;
   STARTUPINFO StartupInfo;         /* store here the inherited handles */
   PROCESS_INFORMATION ProcessInfo; /* where we get the process handle from */
   char *savedir;                   /* save the original directory */
@@ -684,24 +748,25 @@ http_cgi_exec (socket_t sock,  /* the socket structure */
   StartupInfo.cb = sizeof (StartupInfo);
   StartupInfo.dwFlags = STARTF_USESTDHANDLES;
   StartupInfo.hStdOutput = out;
+  /* StartupInfo.hStdError = out; */
   if (type == POST_METHOD)
     StartupInfo.hStdInput = in;
 
   /* reserve buffer space for the environment block */
-  envp = xmalloc (ENV_BLOCK_SIZE);
-  savedir = xmalloc (MAX_CGI_DIR_LEN);
+  envp = svz_malloc (ENV_BLOCK_SIZE);
+  savedir = svz_malloc (MAX_CGI_DIR_LEN);
 
   /* save the current directory */
   getcwd (savedir, MAX_CGI_DIR_LEN);
 
   if ((cgifile = http_pre_exec (sock, envp, file, request, type)) == NULL)
     {
-      sock_printf (sock, HTTP_INTERNAL_ERROR "\r\n");
+      svz_sock_printf (sock, HTTP_INTERNAL_ERROR "\r\n");
       http_error_response (sock, 500);
       sock->userflags |= HTTP_FLAG_DONE;
       chdir (savedir);
-      xfree (envp);
-      xfree (savedir);
+      svz_free (envp);
+      svz_free (savedir);
       return -1;
     }
 
@@ -711,13 +776,13 @@ http_cgi_exec (socket_t sock,  /* the socket structure */
     p--;
   suffix = p + 1;
 
-  if ((p = hash_get (*(cfg->cgiapps), util_tolower (suffix))) != NULL)
+  if ((p = svz_hash_get (cfg->cgiapps, svz_tolower (suffix))) != NULL)
     {
       if (strcmp (p, DEFAULT_CGIAPP))
 	{
-	  cgiapp = xmalloc (strlen (cgifile) + strlen (p) + 2);
+	  cgiapp = svz_malloc (strlen (cgifile) + strlen (p) + 2);
 	  sprintf (cgiapp, "%s %s", p, cgifile);
-	  xfree (cgifile);
+	  svz_free (cgifile);
 	  cgifile = cgiapp;
 	}
     }
@@ -725,28 +790,24 @@ http_cgi_exec (socket_t sock,  /* the socket structure */
   else
     {
       /* find an appropriate system association */
-      cgiapp = xmalloc (MAX_PATH);
+      cgiapp = svz_malloc (MAX_PATH);
       if (FindExecutable (cgifile, NULL, cgiapp) <= (HINSTANCE) 32)
-	{
-	  log_printf (LOG_ERROR, "FindExecutable: %s\n", SYS_ERROR);
-	}
+	svz_log (LOG_ERROR, "FindExecutable: %s\n", SYS_ERROR);
 #if ENABLE_DEBUG
       /* if this is enabled you could learn about the system */
       else
-	{
-	  log_printf (LOG_DEBUG, "FindExecutable: %s\n", cgiapp);
-	}
+	svz_log (LOG_DEBUG, "FindExecutable: %s\n", cgiapp);
 #endif
-      xfree (cgiapp);
+      svz_free (cgiapp);
 
       /* print some error message */
-      sock_printf (sock, HTTP_ACCESS_DENIED "\r\n");
+      svz_sock_printf (sock, HTTP_ACCESS_DENIED "\r\n");
       http_error_response (sock, 403);
       sock->userflags |= HTTP_FLAG_DONE;
       chdir (savedir);
-      xfree (cgifile);
-      xfree (envp);
-      xfree (savedir);
+      svz_free (cgifile);
+      svz_free (envp);
+      svz_free (savedir);
       return -1;
     }
 
@@ -755,9 +816,9 @@ http_cgi_exec (socket_t sock,  /* the socket structure */
     {
       sock->userflags |= HTTP_FLAG_DONE;
       chdir (savedir);
-      xfree (cgifile);
-      xfree (envp);
-      xfree (savedir);
+      svz_free (cgifile);
+      svz_free (envp);
+      svz_free (savedir);
       return -1;
     }
 
@@ -772,29 +833,29 @@ http_cgi_exec (socket_t sock,  /* the socket structure */
 		      NULL,                /* CurrentDirectory */
 		      &StartupInfo, &ProcessInfo))
     {
-      log_printf (LOG_ERROR, "cgi: CreateProcess: %s\n", SYS_ERROR);
+      svz_log (LOG_ERROR, "cgi: CreateProcess: %s\n", SYS_ERROR);
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "cgi: cannot execute: %s\n", cgifile);
+      svz_log (LOG_DEBUG, "cgi: cannot execute: %s\n", cgifile);
 #endif
-      sock_printf (sock, "\r\n");
+      svz_sock_printf (sock, "\r\n");
       sock->userflags |= HTTP_FLAG_DONE;
       chdir (savedir);
-      xfree (cgifile);
-      xfree (envp);
-      xfree (savedir);
+      svz_free (cgifile);
+      svz_free (envp);
+      svz_free (savedir);
       return -1;
     }
   
   /* reenter the actual directory and free reserved space */
   chdir (savedir);
-  xfree (cgifile);
-  xfree (envp);
-  xfree (savedir);
+  svz_free (cgifile);
+  svz_free (envp);
+  svz_free (savedir);
   pid = ProcessInfo.hProcess;
 
 #ifdef ENABLE_DEBUG
-  log_printf (LOG_DEBUG, "http: cgi %s got pid 0x%08X\n", 
-	      file + 1, ProcessInfo.dwProcessId);
+  svz_log (LOG_DEBUG, "http: cgi %s got pid 0x%08X\n", 
+	   file + 1, ProcessInfo.dwProcessId);
 #endif
 
 #else /* not __MINGW32__ */
@@ -816,50 +877,72 @@ http_cgi_exec (socket_t sock,  /* the socket structure */
       /* make the output blocking */
       if (fcntl (out, F_SETFL, ~O_NONBLOCK) == -1)
 	{
-	  log_printf (LOG_ERROR, "cgi: fcntl: %s\n", SYS_ERROR);
+	  svz_log (LOG_ERROR, "cgi: fcntl: %s\n", SYS_ERROR);
 	  exit (0);
 	}
 
       /* duplicate the receiving pipe descriptor to stdout */
       if (dup2 (out, 1) != 1)
 	{
-	  log_printf (LOG_ERROR, "cgi: dup2: %s\n", SYS_ERROR);
+	  svz_log (LOG_ERROR, "cgi: dup2: %s\n", SYS_ERROR);
 	  exit (0);
 	}
+#ifndef ENABLE_DEBUG
+      /* duplicate stderr to the cgi output */
+      if (dup2 (out, 2) != 2)
+	{
+	  svz_log (LOG_ERROR, "cgi: dup2: %s\n", SYS_ERROR);
+	  exit (0);
+	}
+#endif /* !ENABLE_DEBUG */
 
+      /* handle post method */
       if (type == POST_METHOD)
 	{
 	  /* make the input blocking */
 	  if (fcntl (in, F_SETFL, ~O_NONBLOCK) == -1)
 	    {
-	      log_printf (LOG_ERROR, "cgi: fcntl: %s\n", SYS_ERROR);
+	      svz_log (LOG_ERROR, "cgi: fcntl: %s\n", SYS_ERROR);
 	      exit (0);
 	    }
 
 	  /* duplicate the sending pipe descriptor to stdin */
 	  if (dup2 (in, 0) != 0)
 	    {
-	      log_printf (LOG_ERROR, "cgi: dup2: %s\n", SYS_ERROR);
+	      svz_log (LOG_ERROR, "cgi: dup2: %s\n", SYS_ERROR);
 	      exit (0);
 	    }
+
+	  /* close the old file descriptors */
+	  if (close (in) < 0)
+	    svz_log (LOG_ERROR, "cgi: close: %s\n", SYS_ERROR);
 	}
+      /* close remaining stdin in get method */
+      else
+	{
+	  close (0);
+	}
+
+      /* close the old file descriptors */
+      if (close (out) < 0)
+	svz_log (LOG_ERROR, "cgi: close: %s\n", SYS_ERROR);
 
       /* get the cgi scripts permissions */
       if (stat (cgifile, &buf) == -1)
 	{
-	  log_printf (LOG_ERROR, "cgi: stat: %s\n", SYS_ERROR);
+	  svz_log (LOG_ERROR, "cgi: stat: %s\n", SYS_ERROR);
 	  exit (0);
 	}
 
       /* set the appropriate user permissions */
       if (setgid (buf.st_gid) == -1)
 	{
-	  log_printf (LOG_ERROR, "cgi: setgid: %s\n", SYS_ERROR);
+	  svz_log (LOG_ERROR, "cgi: setgid: %s\n", SYS_ERROR);
 	  exit (0);
 	}
       if (setuid (buf.st_uid) == -1)
 	{
-	  log_printf (LOG_ERROR, "cgi: setuid: %s\n", SYS_ERROR);
+	  svz_log (LOG_ERROR, "cgi: setuid: %s\n", SYS_ERROR);
 	  exit (0);
 	}
 
@@ -873,7 +956,7 @@ http_cgi_exec (socket_t sock,  /* the socket structure */
        */
       if (execve (cgifile, argv, envp) == -1)
 	{
-	  log_printf (LOG_ERROR, "cgi: execve: %s\n", SYS_ERROR);
+	  svz_log (LOG_ERROR, "cgi: execve: %s\n", SYS_ERROR);
 	  exit (0);
 	}
     }
@@ -886,8 +969,8 @@ http_cgi_exec (socket_t sock,  /* the socket structure */
 	}
 
       /* ------ error forking new process ------ */
-      log_printf (LOG_ERROR, "cgi: fork: %s\n", SYS_ERROR);
-      sock_printf (sock, HTTP_BAD_REQUEST "\r\n");
+      svz_log (LOG_ERROR, "cgi: fork: %s\n", SYS_ERROR);
+      svz_sock_printf (sock, HTTP_BAD_REQUEST "\r\n");
       http_error_response (sock, 400);
       sock->userflags |= HTTP_FLAG_DONE;
       return -1;
@@ -896,7 +979,7 @@ http_cgi_exec (socket_t sock,  /* the socket structure */
   /* ------ still current (parent) process here ------ */
 
 #ifdef ENABLE_DEBUG
-  log_printf (LOG_DEBUG, "http: cgi %s got pid %d\n", file + 1, pid);
+  svz_log (LOG_DEBUG, "http: cgi %s got pid %d\n", file + 1, pid);
 #endif
 
   /* send http header response */
@@ -906,7 +989,7 @@ http_cgi_exec (socket_t sock,  /* the socket structure */
       return -1;
     }
 
-#endif
+#endif /* not __MINGW32__ */
 
   /* save the process id */
   http = sock->data;
@@ -915,25 +998,75 @@ http_cgi_exec (socket_t sock,  /* the socket structure */
   /* close the inherited http data handles */
   if (closehandle (out) == -1)
     {
-      log_printf (LOG_ERROR, "cgi: close: %s\n", SYS_ERROR);
+      svz_log (LOG_ERROR, "cgi: close: %s\n", SYS_ERROR);
     }
   if (type == POST_METHOD)
     {
       /* close the reading end of the pipe for the post data */
       if (closehandle (in) == -1)
 	{
-	  log_printf (LOG_ERROR, "cgi: close: %s\n", SYS_ERROR);
+	  svz_log (LOG_ERROR, "cgi: close: %s\n", SYS_ERROR);
 	}
     }
 
   return 0;
 }
 
+/*
+ * The http GET cgi request response.
+ */
+int
+http_cgi_get_response (svz_socket_t *sock, char *request, int flags)
+{
+  HANDLE cgi2s[2];
+  char *file;
+
+  /* check if this is a cgi request at all */
+  if ((file = http_check_cgi (sock, request)) == HTTP_NO_CGI)
+    return -2;
+
+  if (file == NULL)
+    {
+      svz_sock_printf (sock, HTTP_INTERNAL_ERROR "\r\n");
+      http_error_response (sock, 500);
+      sock->userflags |= HTTP_FLAG_DONE;
+      return -1;
+    }
+
+  /* create a pipe for the cgi script process */
+  if (svz_pipe_create_pair (cgi2s) == -1)
+    {
+      svz_sock_printf (sock, HTTP_INTERNAL_ERROR "\r\n");
+      http_error_response (sock, 500);
+      sock->userflags |= HTTP_FLAG_DONE;
+      svz_free (file);
+      return -1;
+    }
+
+  /* execute the cgi script in FILE */
+  sock->userflags |= HTTP_FLAG_CGI;
+  sock->flags |= SOCK_FLAG_RECV_PIPE;
+  sock->read_socket = http_cgi_read;
+  sock->pipe_desc[READ] = cgi2s[READ];
+  svz_fd_cloexec ((int) cgi2s[READ]);
+
+  if (http_cgi_exec (sock, INVALID_HANDLE, cgi2s[WRITE], 
+		     file, request, GET_METHOD))
+    {
+      /* some error occurred here */
+      sock->read_socket = svz_tcp_read_socket;
+      svz_free (file);
+      return -1;
+    }
+  svz_free (file);
+  return 0;
+}
+
 /*
  * The http POST request response.
  */
 int
-http_post_response (socket_t sock, char *request, int flags)
+http_post_response (svz_socket_t *sock, char *request, int flags)
 {
   char *file;
   char *length;
@@ -948,53 +1081,55 @@ http_post_response (socket_t sock, char *request, int flags)
   file = http_check_cgi (sock, request);
   if (file == NULL || file == HTTP_NO_CGI)
     {
-      sock_printf (sock, HTTP_INTERNAL_ERROR "\r\n");
+      svz_sock_printf (sock, HTTP_INTERNAL_ERROR "\r\n");
       http_error_response (sock, 500);
       sock->userflags |= HTTP_FLAG_DONE;
       return -1;
     }
 
   /* create a pair of pipes for the cgi script process */
-  if (pipe_create_pair (cgi2s) == -1)
+  if (svz_pipe_create_pair (cgi2s) == -1)
     {
-      sock_printf (sock, HTTP_INTERNAL_ERROR "\r\n");
+      svz_sock_printf (sock, HTTP_INTERNAL_ERROR "\r\n");
       http_error_response (sock, 500);
       sock->userflags |= HTTP_FLAG_DONE;
-      xfree (file);
+      svz_free (file);
       return -1;
     }
-  if (pipe_create_pair (s2cgi) == -1)
+  if (svz_pipe_create_pair (s2cgi) == -1)
     {
-      sock_printf (sock, HTTP_INTERNAL_ERROR "\r\n");
+      svz_sock_printf (sock, HTTP_INTERNAL_ERROR "\r\n");
       http_error_response (sock, 500);
       sock->userflags |= HTTP_FLAG_DONE;
-      xfree (file);
+      svz_free (file);
       return -1;
     }
 
   /* get the content length from the header information */
   if ((length = http_find_property (http, "Content-length")) == NULL)
     {
-      sock_printf (sock, HTTP_BAD_REQUEST "\r\n");
+      svz_sock_printf (sock, HTTP_BAD_REQUEST "\r\n");
       http_error_response (sock, 411);
       sock->userflags |= HTTP_FLAG_DONE;
-      xfree (file);
+      svz_free (file);
       return -1;
     }
-  http->contentlength = util_atoi (length);
+  http->contentlength = svz_atoi (length);
 
   /* prepare everything for the cgi pipe handling */
   sock->pipe_desc[WRITE] = s2cgi[WRITE];
   sock->pipe_desc[READ] = cgi2s[READ];
+  svz_fd_cloexec ((int) s2cgi[WRITE]);
+  svz_fd_cloexec ((int) cgi2s[READ]);
 
   /* execute the cgi script in FILE */
   if (http_cgi_exec (sock, s2cgi[READ], cgi2s[WRITE], 
 		     file, request, POST_METHOD))
     {
       /* some error occurred here */
-      sock->read_socket = default_read;
+      sock->read_socket = svz_tcp_read_socket;
       sock->write_socket = http_default_write;
-      xfree (file);
+      svz_free (file);
       return -1;
     }
 
@@ -1002,7 +1137,7 @@ http_post_response (socket_t sock, char *request, int flags)
   sock->flags |= SOCK_FLAG_SEND_PIPE;
   sock->userflags |= HTTP_FLAG_POST;
 
-  xfree (file);
+  svz_free (file);
   return 0;
 }
 
