@@ -1,7 +1,7 @@
 /*
  * http-core.c - http core functionality
  *
- * Copyright (C) 2000 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2000, 2001 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: http-core.c,v 1.23 2000/12/31 13:23:36 ela Exp $
+ * $Id: http-core.c,v 1.38 2001/07/05 21:35:26 ela Exp $
  *
  */
 
@@ -47,18 +47,13 @@
 #endif
 
 #ifdef __MINGW32__
-# include <winsock.h>
+# include <winsock2.h>
 # include <io.h>
 # include <lm.h>
+# include <lmerr.h>
 #endif
 
-#include "util.h"
-#include "alloc.h"
-#include "windoze.h"
-#include "hash.h"
-#include "snprintf.h"
-#include "serveez.h"
-#include "server-core.h"
+#include "libserveez.h"
 #include "http-proto.h"
 #include "http-core.h"
 
@@ -70,7 +65,16 @@ http_header_t http_header;
  */
 #if defined (__CYGWIN__) || defined (__MINGW32__)
 # define timezone _timezone
-# define daylight _daylight
+# ifndef daylight
+#  define daylight _daylight
+# endif
+
+#elif !HAVE_TIMEZONE
+/*
+ * For some reason FreeBSD 3.2 does not provide `timezone' and `daylight'.
+ */
+# define timezone ((long int) 0)
+# define daylight ((int) 0)
 #endif
 
 #ifdef __MINGW32__
@@ -113,7 +117,7 @@ http_start_netapi (void)
 	GetProcAddress (netapiHandle, "NetUserGetInfo");
       if (GetUserInfo == NULL)
 	{
-	  log_printf (LOG_ERROR, "http: GetProcAddress: %s\n", SYS_ERROR);
+	  svz_log (LOG_ERROR, "http: GetProcAddress: %s\n", SYS_ERROR);
 	  FreeLibrary (netapiHandle);
 	  netapiHandle = NULL;
 	}
@@ -126,7 +130,7 @@ http_start_netapi (void)
  * user's home directory and put it in front of the request.
  */
 char *
-http_userdir (socket_t sock, char *uri)
+http_userdir (svz_socket_t *sock, char *uri)
 {
   http_config_t *cfg = sock->cfg;
   char *p = uri + 1;
@@ -146,30 +150,30 @@ http_userdir (socket_t sock, char *uri)
       if (p - uri <= 2)
 	return NULL;
 
-      user = xmalloc (p - uri - 1);
+      user = svz_malloc (p - uri - 1);
       memcpy (user, uri + 2, p - uri - 2);
       user[p - uri - 2] = '\0';
       
 #if HAVE_GETPWNAM
       if ((entry = getpwnam (user)) != NULL)
 	{
-	  file = xmalloc (strlen (entry->pw_dir) + strlen (cfg->userdir) + 
-			  strlen (p) + 2);
+	  file = svz_malloc (strlen (entry->pw_dir) + strlen (cfg->userdir) + 
+			     strlen (p) + 2);
 	  sprintf (file, "%s/%s%s", entry->pw_dir, cfg->userdir, p);
-	  xfree (user);
+	  svz_free (user);
 	  return file;
 	}
 #elif defined (__MINGW32__)
       if (GetUserInfo == NULL)
 	{
-	  xfree (user);
+	  svz_free (user);
 	  return NULL;
 	}
 
-      status = GetUserInfo (NULL,                   /* server name */
-			    windoze_asc2uni (user), /* user name */
-			    1,                      /* type of info */
-			    (LPBYTE *) &entry);     /* info buffer */
+      status = GetUserInfo (NULL,                       /* server name */
+			    svz_windoze_asc2uni (user), /* user name */
+			    1,                          /* type of info */
+			    (LPBYTE *) &entry);         /* info buffer */
 
       if (status != NERR_Success)
 	{
@@ -190,29 +194,31 @@ http_userdir (socket_t sock, char *uri)
 	      error = "Unknown error.";
 	      break;
 	    }
-	  log_printf (LOG_ERROR, "NetUserGetInfo: %s\n", error);
+	  svz_log (LOG_ERROR, "NetUserGetInfo: %s\n", error);
 	}
       /* successfully got the user information ? */
       else if (entry && entry->usri1_home_dir && entry->usri1_home_dir[0])
 	{
-	  file = xmalloc (strlen (windoze_uni2asc (entry->usri1_home_dir)) +
-			  strlen (cfg->userdir) + strlen (p) + 2);
+	  file = 
+	    svz_malloc (strlen (svz_windoze_uni2asc (entry->usri1_home_dir)) +
+			strlen (cfg->userdir) + strlen (p) + 2);
 	  sprintf (file, "%s/%s%s", 
-		   windoze_uni2asc (entry->usri1_home_dir), cfg->userdir, p);
+		   svz_windoze_uni2asc (entry->usri1_home_dir), 
+		   cfg->userdir, p);
 	  FreeUserInfo (entry);
-	  xfree (user);
+	  svz_free (user);
 	  return file;
 	}
 #if ENABLE_DEBUG
       else if (entry)
 	{
-	  log_printf (LOG_DEBUG, "http: home directory for %s not set\n",
-		      windoze_uni2asc (entry->usri1_name));
+	  svz_log (LOG_DEBUG, "http: home directory for %s not set\n",
+		   svz_windoze_uni2asc (entry->usri1_name));
 	}
 #endif /* ENABLE_DEBUG */
 #endif /* not HAVE_GETPWNAM and not __MINGW32__ */
 
-      xfree (user);
+      svz_free (user);
     }
   return NULL;
 }
@@ -226,13 +232,13 @@ int
 http_identification (char *ident, int id, int version)
 {
   http_socket_t *http;
-  socket_t sock = sock_find (id, version);
+  svz_socket_t *sock = svz_sock_find (id, version);
 
   if (ident && sock)
     {
       http = sock->data;
       if (!http->ident)
-	http->ident = xstrdup (ident);
+	http->ident = svz_strdup (ident);
     } 
 
   return 0;
@@ -245,13 +251,13 @@ int
 http_remotehost (char *host, int id, int version)
 {
   http_socket_t *http;
-  socket_t sock = sock_find (id, version);
+  svz_socket_t *sock = svz_sock_find (id, version);
 
   if (host && sock)
     {
       http = sock->data;
       if (!http->host)
-	http->host = xstrdup (host);
+	http->host = svz_strdup (host);
     } 
 
   return 0;
@@ -266,7 +272,7 @@ http_localhost (char *host, http_config_t *cfg)
 {
   if (host && !cfg->host)
     {
-      cfg->host = xpstrdup (host);
+      cfg->host = svz_pstrdup (host);
     }
   return 0;
 }
@@ -276,7 +282,7 @@ http_localhost (char *host, http_config_t *cfg)
  * and necessary.
  */
 void
-http_log (socket_t sock)
+http_log (svz_socket_t *sock)
 {
   http_config_t *cfg = sock->cfg;
   http_socket_t *http = sock->data;
@@ -319,18 +325,18 @@ http_log (socket_t sock)
 	      break;
 	      /* %l - delivered content length */
 	    case 'l':
-	      strcat (line, util_itoa (http->length));
+	      strcat (line, svz_itoa (http->length));
 	      p++;
 	      break;
 	      /* %c - http response code */
 	    case 'c':
-	      strcat (line, util_itoa (http->response));
+	      strcat (line, svz_itoa (http->response));
 	      p++;
 	      break;
 	      /* %h - host name */
 	    case 'h':
 	      strcat (line, http->host ? http->host :
-		      util_inet_ntoa (sock->remote_addr));
+		      svz_inet_ntoa (sock->remote_addr));
 	      p++;
 	      break;
 	      /* %t - request time stamp */
@@ -363,8 +369,18 @@ http_log (socket_t sock)
 	  start = p;
 	}
       strcat (line, "\n");
-      fprintf (cfg->log, line);
-      fflush (cfg->log);
+
+      if (!ferror (cfg->log) && !feof (cfg->log))
+	{
+	  fprintf (cfg->log, line);
+	  fflush (cfg->log);
+	}
+      else
+	{
+	  svz_log (LOG_ERROR, "http: access logfile died\n");
+	  svz_fclose (cfg->log);
+	  cfg->log = NULL;
+	}
     }
 }
 
@@ -392,7 +408,7 @@ http_add_header (const char *fmt, ...)
   if (len >= HTTP_HEADER_SIZE)
     return;
   va_start (args, fmt);
-  vsnprintf (p, HTTP_HEADER_SIZE - len, fmt, args);
+  svz_vsnprintf (p, HTTP_HEADER_SIZE - len, fmt, args);
   va_end (args);
 }
 
@@ -400,24 +416,23 @@ http_add_header (const char *fmt, ...)
  * Send the http header.
  */
 int
-http_send_header (socket_t sock)
+http_send_header (svz_socket_t *sock)
 {
   int ret = 0;
 
   /* send first part of header including response field and static texts */
-  ret = sock_printf (sock, 
-		     "%s"
-		     "Date: %s\r\n"
-		     "Server: %s/%s\r\n",
-		     http_header.response,
-		     http_asc_date (time (NULL)),
-		     serveez_config.program_name,
-		     serveez_config.version_string);
+  ret = svz_sock_printf (sock, 
+			 "%s"
+			 "Date: %s\r\n"
+			 "Server: %s/%s\r\n",
+			 http_header.response,
+			 http_asc_date (time (NULL)),
+			 svz_library, svz_version);
   if (ret)
     return ret;
 
   /* send header fields and trailing line break */
-  ret = sock_printf (sock, "%s\r\n", http_header.field);
+  ret = svz_sock_printf (sock, "%s\r\n", http_header.field);
   if (ret)
     return ret;
   
@@ -456,7 +471,7 @@ http_get_range (char *line, http_range_t *range)
   if (memcmp (p, "bytes", 5))
     {
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "http: invalid byte-range specifier (%s)\n", p);
+      svz_log (LOG_DEBUG, "http: invalid byte-range specifier (%s)\n", p);
 #endif
       return -1;
     }
@@ -511,7 +526,7 @@ http_get_range (char *line, http_range_t *range)
  * not data." if this occurs.
  */
 int
-http_error_response (socket_t sock, int response)
+http_error_response (svz_socket_t *sock, int response)
 {
   http_config_t *cfg = sock->cfg;
   http_socket_t *http = sock->data;
@@ -598,19 +613,18 @@ http_error_response (socket_t sock, int response)
   http->response = response;
 
   /* Send some standard error message. */
-  return sock_printf (sock, 
-		      "<html><body bgcolor=white text=black><br>"
-		      "<h1>%d %s</h1>"
-		      "<hr noshade><i>%s/%s server at %s port %d, "
-		      "please send email to <a href=\"mailto:%s\">%s</a> "
-		      "for reporting errors</i>"
-		      "</body></html>",
-		      response, txt, 
-		      serveez_config.program_name, 
-		      serveez_config.version_string,
-		      cfg->host ? cfg->host : 
-		      util_inet_ntoa (sock->local_addr),
-		      ntohs (sock->local_port), cfg->admin, cfg->admin);
+  return svz_sock_printf (sock, 
+			  "<html><body bgcolor=white text=black><br>"
+			  "<h1>%d %s</h1>"
+			  "<hr noshade><i>%s/%s server at %s port %d, "
+			  "please send email to <a href=\"mailto:%s\">%s</a> "
+			  "for reporting errors</i>"
+			  "</body></html>",
+			  response, txt, 
+			  svz_library, svz_version,
+			  cfg->host ? cfg->host : 
+			  svz_inet_ntoa (sock->local_addr),
+			  ntohs (sock->local_port), cfg->admin, cfg->admin);
 }
 
 /*
@@ -618,20 +632,20 @@ http_error_response (socket_t sock, int response)
  * Keep-Alive connections. Return -1 if it is not 'Keep'able.
  */
 int
-http_keep_alive (socket_t sock)
+http_keep_alive (svz_socket_t *sock)
 {
   if (sock->userflags & HTTP_FLAG_KEEP)
     {
       http_free_socket (sock);
 
       sock->userflags &= ~HTTP_FLAG; 
-      sock->read_socket = default_read;
+      sock->read_socket = svz_tcp_read_socket;
       sock->check_request = http_check_request;
       sock->write_socket = http_default_write;
       sock->send_buffer_fill = 0;
       sock->idle_func = http_idle;
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "http: keeping connection alive\n");
+      svz_log (LOG_DEBUG, "http: keeping connection alive\n");
 #endif
       return 0;
     }
@@ -643,7 +657,7 @@ http_keep_alive (socket_t sock)
  * Keep-Alive connection and sends the appropriate HTTP header property.
  */
 void
-http_check_keepalive (socket_t sock)
+http_check_keepalive (svz_socket_t *sock)
 {
   http_socket_t *http = sock->data;
   http_config_t *cfg = sock->cfg;
@@ -683,8 +697,8 @@ http_clf_date (time_t t)
 	   tm->tm_mday, months[tm->tm_mon], tm->tm_year + 1900,
 	   tm->tm_hour, tm->tm_min, tm->tm_sec,
 	   timezone > 0 ? '+' : '-',
-	   timezone > 0 ? timezone / 3600 : -timezone / 3600,
-	   timezone > 0 ? (timezone / 60) % 60 : -(timezone / 60) % 60);
+	   timezone > 0 ? timezone / 3600 : - timezone / 3600,
+	   timezone > 0 ? (timezone / 60) % 60 : - (timezone / 60) % 60);
   return date;
 }
 
@@ -768,7 +782,7 @@ http_parse_date (char *date)
  * properties found in the request.
  */
 int
-http_parse_property (socket_t sock, char *request, char *end)
+http_parse_property (svz_socket_t *sock, char *request, char *end)
 {
   int properties, n;
   char *p;
@@ -778,12 +792,12 @@ http_parse_property (socket_t sock, char *request, char *end)
   http = sock->data;
 
   /* reserve data space for the http properties */
-  http->property = xmalloc (MAX_HTTP_PROPERTIES * 2 * sizeof (char *));
+  http->property = svz_malloc (MAX_HTTP_PROPERTIES * 2 * sizeof (char *));
   properties = 0;
   n = 0;
 
   /* find out properties if necessary */
-  while (INT16 (request) != CRLF && properties < MAX_HTTP_PROPERTIES - 1)
+  while (SVZ_INT16 (request) != CRLF && properties < MAX_HTTP_PROPERTIES - 1)
     {
       /* get property entity identifier */
       p = request;
@@ -791,18 +805,18 @@ http_parse_property (socket_t sock, char *request, char *end)
 	p++;
       if (p == end)
 	break;
-      http->property[n] = xmalloc (p - request + 1);
+      http->property[n] = svz_malloc (p - request + 1);
       strncpy (http->property[n], request, p - request);
       http->property[n][p - request] = 0;
       n++;
       request = p + 2;
 
       /* get property entity body */
-      while (INT16 (p) != CRLF && p < end)
+      while (SVZ_INT16 (p) != CRLF && p < end)
 	p++;
       if (p == end || p <= request)
 	break;
-      http->property[n] = xmalloc (p - request + 1);
+      http->property[n] = svz_malloc (p - request + 1);
       strncpy (http->property[n], request, p - request);
       http->property[n][p - request] = 0;
       n++;
@@ -838,7 +852,7 @@ http_find_property (http_socket_t *http, char *key)
   n = 0;
   while (http->property[n])
     {
-      if (!util_strcasecmp (http->property[n], key))
+      if (!svz_strcasecmp (http->property[n], key))
 	{
 	  return http->property[n + 1];
 	}
@@ -876,7 +890,10 @@ http_process_uri (char *uri)
 	  /* Copy rest of URI. */
 	  uri = ++p;
 	  while (*(p + 2))
-	    *p++ = *(p + 2);
+	    {
+	      *p = *(p + 2);
+	      p++;
+	    }
 	  *p = '\0';
 	}
       else
@@ -890,22 +907,17 @@ http_process_uri (char *uri)
 void
 http_free_types (http_config_t *cfg)
 {
+#if 0
+  /* FIXME: Is that still neccessary ? */
   char **type;
   int n;
 
-  if (*(cfg->types))
-    {
-      if ((type = (char **) hash_values (*(cfg->types))) != NULL)
-	{
-	  for (n = 0; n < hash_size (*(cfg->types)); n++)
-	    {
-	      xfree (type[n]);
-	    }
-	  hash_xfree (type);
-	}
-      hash_destroy (*(cfg->types));
-      *(cfg->types) = NULL;
-    }
+  svz_hash_foreach_value (cfg->types, type, n)
+    svz_free (type[n]);
+      
+  svz_hash_destroy (cfg->types);
+  cfg->types = NULL;
+#endif
 }
 
 /*
@@ -925,19 +937,16 @@ http_read_types (http_config_t *cfg)
   char *content_type;
 
   /* create the content type hash table if necessary */
-  if (*(cfg->types) == NULL)
-    {
-      *(cfg->types) = hash_create (4);
-    }
+  if (cfg->types == NULL)
+    cfg->types = svz_hash_create (4);
 
   /* try open the file */
-  if ((f = fopen (cfg->type_file, "rt")) == NULL)
+  if ((f = svz_fopen (cfg->type_file, "rt")) == NULL)
     {
-      log_printf (LOG_ERROR, "fopen: %s\n", SYS_ERROR);
       return -1;
     }
 
-  line = xmalloc (TYPES_LINE_SIZE);
+  line = svz_malloc (TYPES_LINE_SIZE);
 
   /* read all lines within the file */
   while ((fgets (line, TYPES_LINE_SIZE, f)) != NULL)
@@ -976,17 +985,17 @@ http_read_types (http_config_t *cfg)
 	       * add the given content type to the hash if it does not
 	       * contain it already
 	       */
-	      if (!hash_get (*(cfg->types), suffix))
+	      if (!svz_hash_get (cfg->types, suffix))
 		{
-		  content_type = xmalloc (strlen (content) + 1);
+		  content_type = svz_malloc (strlen (content) + 1);
 		  strcpy (content_type, content);
-		  hash_put (*(cfg->types), suffix, content_type);
+		  svz_hash_put (cfg->types, suffix, content_type);
 		}
 	    }
 	}
     }
-  fclose (f);
-  xfree (line);
+  svz_fclose (f);
+  svz_free (line);
   return 0;
 }
 
@@ -996,7 +1005,7 @@ http_read_types (http_config_t *cfg)
  * type if the suffix could not be found.
  */
 char *
-http_find_content_type (socket_t sock, char *file)
+http_find_content_type (svz_socket_t *sock, char *file)
 {
   http_config_t *cfg = sock->cfg;
   char *suffix = file + strlen (file) - 1;
@@ -1009,7 +1018,7 @@ http_find_content_type (socket_t sock, char *file)
     suffix++;
 
   /* find this file suffix in the content type hash */
-  if ((type = hash_get (*(cfg->types), suffix)) != NULL)
+  if ((type = svz_hash_get (cfg->types, suffix)) != NULL)
     {
       return type;
     }
@@ -1044,16 +1053,16 @@ http_absolute_file (char *file)
     }
 
   /* save the filename within a buffer */
-  savefile = xmalloc (strlen (p) + 1);
+  savefile = svz_malloc (strlen (p) + 1);
   strcpy (savefile, p);
 
   /* get current work directory */
-  savedir = xmalloc (MAX_DIR_LEN);
+  savedir = svz_malloc (MAX_DIR_LEN);
   if ((getcwd (savedir, MAX_DIR_LEN)) == NULL)
     {
-      log_printf (LOG_ERROR, "getcwd: %s\n", SYS_ERROR);
-      xfree (savefile);
-      xfree (savedir);
+      svz_log (LOG_ERROR, "getcwd: %s\n", SYS_ERROR);
+      svz_free (savefile);
+      svz_free (savedir);
       return file;
     }
   
@@ -1065,8 +1074,8 @@ http_absolute_file (char *file)
     {
       strcat (savedir, "/");
       strcat (savedir, savefile);
-      savedir = xrealloc (savedir, strlen (savedir) + 1);
-      xfree (file);
+      savedir = svz_realloc (savedir, strlen (savedir) + 1);
+      svz_free (file);
       return savedir;
     }
   
@@ -1075,37 +1084,37 @@ http_absolute_file (char *file)
   if (chdir (file) == -1)
     {
       *p = '/';
-      log_printf (LOG_ERROR, "chdir: %s\n", SYS_ERROR);
+      svz_log (LOG_ERROR, "chdir: %s\n", SYS_ERROR);
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "cannot change dir: %s\n", file);
+      svz_log (LOG_DEBUG, "cannot change dir: %s\n", file);
 #endif
-      xfree (savefile);
-      xfree (savedir);
+      svz_free (savefile);
+      svz_free (savedir);
       return file;
     }
   *p = '/';
 
   /* get now the current work directory */
-  dir = xmalloc (MAX_DIR_LEN);
+  dir = svz_malloc (MAX_DIR_LEN);
   if ((getcwd (dir, MAX_DIR_LEN)) == NULL)
     {
-      log_printf (LOG_ERROR, "getcwd: %s\n", SYS_ERROR);
-      xfree (dir);
-      xfree (savefile);
-      xfree (savedir);
+      svz_log (LOG_ERROR, "getcwd: %s\n", SYS_ERROR);
+      svz_free (dir);
+      svz_free (savefile);
+      svz_free (savedir);
       return file;
     }
 
   /* concate new work directory with given filename */
   strcat (dir, "/");
   strcat (dir, savefile);
-  dir = xrealloc (dir, strlen (dir) + 1);
-  xfree (savefile);
-  xfree (file);
+  dir = svz_realloc (dir, strlen (dir) + 1);
+  svz_free (savefile);
+  svz_free (file);
 
   /* change back to the original work directory */
   chdir (savedir);
-  xfree (savedir);
+  svz_free (savedir);
   return dir;
 }
 
