@@ -1,7 +1,7 @@
 /*
  * http-proto.c - http protocol implementation
  *
- * Copyright (C) 2000 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2000, 2001 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: http-proto.c,v 1.51 2001/01/02 00:52:46 raimi Exp $
+ * $Id: http-proto.c,v 1.73 2001/07/11 18:02:59 ela Exp $
  *
  */
 
@@ -29,6 +29,7 @@
 #if ENABLE_HTTP_PROTO
 
 #define _GNU_SOURCE
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -37,7 +38,6 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <time.h>
-#include <signal.h>
 
 #if HAVE_STRINGS_H
 # include <strings.h>
@@ -45,12 +45,9 @@
 #if HAVE_UNISTD_H
 # include <unistd.h>
 #endif
-#if HAVE_SYS_SENDFILE_H
-# include <sys/sendfile.h>
-#endif
 
 #ifdef __MINGW32__
-# include <winsock.h>
+# include <winsock2.h>
 # include <io.h>
 #endif
 
@@ -58,53 +55,20 @@
 # include <sys/types.h>
 # include <sys/socket.h>
 # include <netinet/in.h>
-# if HAVE_WAIT_H
-#  include <wait.h>
-# endif
-# if HAVE_SYS_WAIT_H
-#  include <sys/wait.h>
-# endif
 #endif
 
-#if HAVE_NETINET_TCP_H
-# include <netinet/tcp.h>
-#endif
-
-#include "util.h"
-#include "alloc.h"
-#include "hash.h"
-#include "socket.h"
-#include "pipe-socket.h"
-#include "serveez.h"
-#include "server.h"
-#include "server-socket.h"
-#include "server-core.h"
+#include "libserveez.h"
 #include "http-proto.h"
 #include "http-core.h"
 #include "http-cgi.h"
 #include "http-dirlist.h"
 #include "http-cache.h"
-#include "coserver/coserver.h"
-
-/*
- * The http port configuration.
- */
-portcfg_t http_port =
-{
-  PROTO_TCP,  /* TCP protocol definition */
-  42424,      /* preferred port */
-  "*",        /* preferred local ip address */
-  NULL,       /* calculated automatically later */
-  NULL,       /* no receiving (listening) pipe */
-  NULL        /* no sending pipe */
-};
 
 /*
  * The HTTP server instance configuration.
  */
 http_config_t http_config =
 {
-  &http_port,         /* port configuration */
   "index.html",       /* standard index file  for GET request */
   "../show",          /* document root */
   "/cgi-bin",         /* how cgi-requests are detected */
@@ -128,41 +92,43 @@ http_config_t http_config =
 };
 
 /*
- * Definition of the configuration items processed by libsizzle (taken
- * from the config file).
+ * Definition of the configuration items processed by the configuration
+ * callbacks.
  */
-key_value_pair_t http_config_prototype[] =
+svz_key_value_pair_t http_config_prototype[] =
 {
-  REGISTER_PORTCFG ("netport", http_config.port, DEFAULTABLE),
-  REGISTER_STR ("indexfile", http_config.indexfile, DEFAULTABLE),
-  REGISTER_STR ("docs", http_config.docs, DEFAULTABLE),
-  REGISTER_STR ("cgi-url", http_config.cgiurl, DEFAULTABLE),
-  REGISTER_STR ("cgi-dir", http_config.cgidir, DEFAULTABLE),
-  REGISTER_INT ("cache-size", http_config.cachesize, DEFAULTABLE),
-  REGISTER_INT ("cache-entries", http_config.cacheentries, DEFAULTABLE),
-  REGISTER_INT ("timeout", http_config.timeout, DEFAULTABLE),
-  REGISTER_INT ("keepalive", http_config.keepalive, DEFAULTABLE),
-  REGISTER_STR ("default-type", http_config.default_type, DEFAULTABLE),
-  REGISTER_STR ("type-file", http_config.type_file, DEFAULTABLE),
-  REGISTER_HASH ("types", http_config.types, DEFAULTABLE),
-  REGISTER_HASH ("cgi-application", http_config.cgiapps, DEFAULTABLE),
-  REGISTER_STR ("admin", http_config.admin, DEFAULTABLE),
-  REGISTER_STR ("host", http_config.host, DEFAULTABLE),
-  REGISTER_STR ("logfile", http_config.logfile, DEFAULTABLE),
-  REGISTER_STR ("logformat", http_config.logformat, DEFAULTABLE),
-  REGISTER_STR ("userdir", http_config.userdir, DEFAULTABLE),
-  REGISTER_INT ("nslookup", http_config.nslookup, DEFAULTABLE),
-  REGISTER_INT ("ident", http_config.ident, DEFAULTABLE),
-  REGISTER_END ()
+  SVZ_REGISTER_STR ("indexfile", http_config.indexfile, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_STR ("docs", http_config.docs, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_STR ("cgi-url", http_config.cgiurl, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_STR ("cgi-dir", http_config.cgidir, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_INT ("cache-size", http_config.cachesize, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_INT ("cache-entries", http_config.cacheentries, 
+		    SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_INT ("timeout", http_config.timeout, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_INT ("keepalive", http_config.keepalive, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_STR ("default-type", http_config.default_type, 
+		    SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_STR ("type-file", http_config.type_file, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_HASH ("types", http_config.types, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_HASH ("cgi-application", http_config.cgiapps, 
+		     SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_STR ("admin", http_config.admin, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_STR ("host", http_config.host, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_STR ("logfile", http_config.logfile, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_STR ("logformat", http_config.logformat, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_STR ("userdir", http_config.userdir, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_BOOL ("nslookup", http_config.nslookup, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_BOOL ("ident", http_config.ident, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_END ()
 };
 
 /*
  * Definition of the http server.
  */
-server_definition_t http_server_definition =
+svz_servertype_t http_server_definition =
 {
   "http server",         /* long server description */
-  "http",                /* short server description (for libsizzle) */
+  "http",                /* short server description */
   http_global_init,      /* global initializer */
   http_init,             /* instance initializer */
   http_detect_proto,     /* protocol detection routine */
@@ -175,7 +141,7 @@ server_definition_t http_server_definition =
   NULL,                  /* handle request callback */
   &http_config,          /* default configuration */
   sizeof (http_config),  /* size of the configuration */
-  http_config_prototype  /* configuration prototypes (libsizzle) */
+  http_config_prototype  /* configuration prototypes */
 };
 
 /*
@@ -185,9 +151,9 @@ server_definition_t http_server_definition =
  */
 struct
 {
-  char *ident;                            /* identification string */
-  int len;                                /* the length of this string */
-  int (*response)(socket_t, char *, int); /* the callback routine */
+  char *ident;                                  /* identification string */
+  int len;                                      /* the length of this string */
+  int (*response)(svz_socket_t *, char *, int); /* the callback routine */
 } 
 http_request [HTTP_REQUESTS] = 
 {
@@ -207,7 +173,7 @@ http_request [HTTP_REQUESTS] =
  * Global http server initializer.
  */
 int
-http_global_init (void)
+http_global_init (svz_servertype_t *server)
 {
 #ifdef __MINGW32__
   http_start_netapi ();
@@ -220,7 +186,7 @@ http_global_init (void)
  * Global http server finalizer.
  */
 int
-http_global_finalize (void)
+http_global_finalize (svz_servertype_t *server)
 {
   http_free_cache ();
 #ifdef __MINGW32__
@@ -233,48 +199,54 @@ http_global_finalize (void)
  * Local http server instance initializer.
  */
 int
-http_init (server_t *server)
+http_init (svz_server_t *server)
 {
   int types = 0;
   char *p;
-  unsigned long host;
+  unsigned long host = INADDR_ANY;
   http_config_t *cfg = server->cfg;
+  svz_array_t *ports;
+  struct sockaddr_in *addr;
 
   /* resolve localhost if server name is not set */
   if (!cfg->host)
     {
-      host = cfg->port->addr->sin_addr.s_addr;
+      if ((ports = svz_server_portcfg (server)) != NULL)
+	{
+	  addr = svz_portcfg_addr ((svz_portcfg_t *) svz_array_get (ports, 0));
+	  host = addr->sin_addr.s_addr;
+	  svz_array_destroy (ports);
+	}
       if (host == INADDR_ANY)
 	host = htonl (INADDR_LOOPBACK);
-      coserver_reverse (host, http_localhost, cfg, NULL);
+      svz_coserver_rdns (host, http_localhost, cfg, NULL);
     }
 
   /* start http logging system */
   if (cfg->logfile)
     {
-      if ((cfg->log = fopen (cfg->logfile, "at")) == NULL)
+      if ((cfg->log = svz_fopen (cfg->logfile, "at")) == NULL)
 	{
-	  log_printf (LOG_ERROR, "http: fopen: %s\n", SYS_ERROR);
-	  log_printf (LOG_ERROR, "http: cannot open access logfile %s\n",
-		      cfg->logfile);
+	  svz_log (LOG_ERROR, "http: cannot open access logfile %s\n",
+		   cfg->logfile);
 	}
     }
   
   /* create content type hash */
-  if (*(cfg->types))
-    types = hash_size (*(cfg->types));
+  if (cfg->types)
+    types = svz_hash_size (cfg->types);
   
   if (http_read_types (cfg))
     {
-      log_printf (LOG_ERROR, "http: unable to load %s\n", cfg->type_file);
+      svz_log (LOG_ERROR, "http: unable to load %s\n", cfg->type_file);
     }
-  log_printf (LOG_NOTICE, "http: %d+%d known content types\n",
-	      types, hash_size (*(cfg->types)) - types);
+  svz_log (LOG_NOTICE, "http: %d+%d known content types\n",
+	   types, svz_hash_size (cfg->types) - types);
 
   /* check user directory path, snip trailing '/' or '\' */
   if (!cfg->userdir || !strlen (cfg->userdir))
     {
-      log_printf (LOG_ERROR, "http: not a valid user directory\n");
+      svz_log (LOG_ERROR, "http: not a valid user directory\n");
       return -1;
     }
   p = cfg->userdir + strlen (cfg->userdir) - 1;
@@ -284,7 +256,7 @@ http_init (server_t *server)
   /* check document root path */
   if (!strlen (cfg->docs))
     {
-      log_printf (LOG_ERROR, "http: not a valid document root\n");
+      svz_log (LOG_ERROR, "http: not a valid document root\n");
       return -1;
     }
 
@@ -299,7 +271,6 @@ http_init (server_t *server)
   /* generate cgi associations */
   http_gen_cgi_apps (cfg);
   
-  server_bind (server, cfg->port);
   return 0;
 }
 
@@ -307,14 +278,14 @@ http_init (server_t *server)
  * Local http server instance finalizer.
  */
 int
-http_finalize (server_t *server)
+http_finalize (svz_server_t *server)
 {
   http_config_t *cfg = server->cfg;
 
   http_free_types (cfg);
   http_free_cgi_apps (cfg);
   if (cfg->log)
-    fclose (cfg->log);
+    svz_fclose (cfg->log);
 
   return 0;
 }
@@ -325,7 +296,7 @@ http_finalize (server_t *server)
  * socket structure SOCK should still be usable for keep-alive connections.
  */
 void
-http_free_socket (socket_t sock)
+http_free_socket (svz_socket_t *sock)
 {
   http_socket_t *http = sock->data;
   int n;
@@ -334,7 +305,7 @@ http_free_socket (socket_t sock)
   http_log (sock);
   if (http->request)
     {
-      xfree (http->request);
+      svz_free (http->request);
       http->request = NULL;
     }
   http->timestamp = 0;
@@ -348,10 +319,10 @@ http_free_socket (socket_t sock)
       n = 0;
       while (http->property[n])
 	{
-	  xfree (http->property[n]);
+	  svz_free (http->property[n]);
 	  n++;
 	}
-      xfree (http->property);
+      svz_free (http->property);
       http->property = NULL;
     }
 
@@ -363,16 +334,13 @@ http_free_socket (socket_t sock)
 
   /* is the cache entry used ? */
   if (http->cache)
-    {
-      xfree (http->cache);
-      http->cache = NULL;
-    }
+    svz_free_and_zero (http->cache);
 
   /* close the file descriptor for usual http file transfer */
   if (sock->file_desc != -1)
     {
-      if (close (sock->file_desc) == -1)
-	log_printf (LOG_ERROR, "close: %s\n", SYS_ERROR);
+      if (svz_close (sock->file_desc) == -1)
+	svz_log (LOG_ERROR, "close: %s\n", SYS_ERROR);
       sock->file_desc = -1;
     }
 }
@@ -382,7 +350,7 @@ http_free_socket (socket_t sock)
  * socket structure entry "disconnected_socket".
  */
 int
-http_disconnect (socket_t sock)
+http_disconnect (svz_socket_t *sock)
 {
   /* get http socket structure */
   http_socket_t *http = sock->data;
@@ -393,83 +361,23 @@ http_disconnect (socket_t sock)
   if (http)
     {
       if (http->host)
-	xfree (http->host);
+	svz_free (http->host);
       if (http->ident)
-	xfree (http->ident);
-      xfree (http);
+	svz_free (http->ident);
+      svz_free (http);
       sock->data = NULL;
     }
 
   return 0;
 }
 
-/*
- * This is the default idle function for http connections. It checks 
- * whether any died child was a cgi script.
- */
-int
-http_cgi_died (socket_t sock)
-{
-  http_socket_t *http = sock->data;
-#ifdef __MINGW32__
-  DWORD result;
-#endif
-
-  if (sock->flags & SOCK_FLAG_PIPE)
-    {
-#ifndef __MINGW32__
-      /* Check if a died child is this cgi. */
-      if (server_child_died && http->pid == server_child_died)
-	{
-	  log_printf (LOG_NOTICE, "cgi script pid %d died\n", 
-		      (int) server_child_died);
-	  server_child_died = 0;
-	}
-#if HAVE_WAITPID
-      /* Test if the cgi is still running. */
-      if (waitpid (http->pid, NULL, WNOHANG | WUNTRACED) == http->pid)
-	{
-	  log_printf (LOG_NOTICE, "cgi script pid %d died\n", 
-		      (int) http->pid);
-	  http->pid = INVALID_HANDLE;
-	}
-#endif /* HAVE_WAITPID */
-
-#else /* __MINGW32__ */
-
-      /*
-       * Check if there died a process handle in Win32, this has to be
-       * done regularly here because there is no SIGCHLD in Win32 !
-       */
-      if (http->pid != INVALID_HANDLE)
-	{
-	  result = WaitForSingleObject (http->pid, LEAST_WAIT_OBJECT);
-	  if (result == WAIT_FAILED)
-	    {
-	      log_printf (LOG_ERROR, "WaitForSingleObject: %s\n", SYS_ERROR);
-	    }
-	  else if (result != WAIT_TIMEOUT)
-	    {
-	      if (closehandle (http->pid) == -1)
-		log_printf (LOG_ERROR, "CloseHandle: %s\n", SYS_ERROR);
-	      server_child_died = http->pid;
-	      http->pid = INVALID_HANDLE;
-	    }
-	}
-#endif /* __MINGW32__ */
-    }
-  
-  sock->idle_counter = 1;
-  return 0;
-}
-
 /*
  * Idle function for HTTP Keep-Alive connections. It simply returns -1
  * if a certain time has elapsed and the main server loop will shutdown
  * the connection therefore.
  */
 int
-http_idle (socket_t sock)
+http_idle (svz_socket_t *sock)
 {
   time_t now;
   http_config_t *cfg = sock->cfg;
@@ -483,42 +391,8 @@ http_idle (socket_t sock)
   return http_cgi_died (sock);
 }
 
-#if HAVE_SENDFILE
-/*
- * Enable and disable the TCP_CORK socket option. This is useful for 
- * performance reasons when using sendfile() with a prepending header
- * to deliver a http document.
- */
-static int
-http_tcp_cork (SOCKET sock, int set)
-{
-#ifdef TCP_CORK
-  int flags;
-
-  /* get current socket options */
-  if ((flags = fcntl (sock, F_GETFL)) < 0)
-    {
-      log_printf (LOG_ERROR, "fcntl: %s\n", NET_ERROR);
-      return -1;
-    }
-
-  /* set / unset cork option */
-  if (set)
-    flags |= TCP_CORK;
-  else
-    flags &= ~TCP_CORK;
-
-  /* set new socket option */
-  if (fcntl (sock, F_SETFL, flags) < 0)
-    {
-      log_printf (LOG_ERROR, "fcntl: %s\n", NET_ERROR);
-      return -1;
-    }
-
-#endif /* TCP_CORK */
-  return 0;
-}
-
+#if ENABLE_SENDFILE
+#if defined (HAVE_SENDFILE) || defined (__MINGW32__)
 /*
  * This routine is using sendfile() to transport large file's content
  * to a network socket. It is replacing HTTP_DEFAULT_WRITE on systems where
@@ -526,19 +400,23 @@ http_tcp_cork (SOCKET sock, int set)
  * the READ_SOCKET callback HTTP_FILE_READ.
  */
 int
-http_send_file (socket_t sock)
+http_send_file (svz_socket_t *sock)
 {
   http_socket_t *http = sock->data;
-  int num_written;
+  int num_written, do_write;
+
+  /* Limitate the number of bytes to write at once. */
+  do_write = http->filelength > SOCK_MAX_WRITE 
+    ? SOCK_MAX_WRITE : http->filelength;
 
   /* Try sending throughout file descriptor to socket. */
-  num_written = sendfile (sock->sock_desc, sock->file_desc,
-			  &http->fileoffset, SOCK_MAX_WRITE);
-  
+  num_written = svz_sendfile (sock->sock_desc, sock->file_desc,
+			      &http->fileoffset, do_write);
+
   /* Some error occurred. */
   if (num_written < 0)
     {
-      log_printf (LOG_ERROR, "http: sendfile: %s\n", SYS_ERROR);
+      svz_log (LOG_ERROR, "http: sendfile: %s\n", SYS_ERROR);
       return -1;
     }
 
@@ -556,24 +434,25 @@ http_send_file (socket_t sock)
   if (http->filelength <= 0)
     {
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "http: file successfully sent\n");
+      svz_log (LOG_DEBUG, "http: file successfully sent\n");
 #endif
       /* 
        * no further read()s from the file descriptor, signaling 
        * the writers there will not be additional data from now on
        */
-      sock->read_socket = default_read;
+      sock->read_socket = svz_tcp_read_socket;
       sock->recv_buffer_fill = 0;
       sock->send_buffer_fill = 0;
       sock->write_socket = http_default_write;
       sock->userflags &= ~HTTP_FLAG_SENDFILE;
       num_written = http_keep_alive (sock);
-      http_tcp_cork (sock->sock_desc, 0);
+      svz_tcp_cork (sock->sock_desc, 0);
     }
 
   return (num_written < 0) ? -1 : 0;
 }
 #endif /* HAVE_SENDFILE */
+#endif /* ENABLE_SENDFILE */
 
 /*
  * HTTP_DEFAULT_WRITE will shutdown the connection immediately when 
@@ -582,7 +461,7 @@ http_send_file (socket_t sock)
  * actual file is within the cache and if this is a keep-alive connection.
  */
 int
-http_default_write (socket_t sock)
+http_default_write (svz_socket_t *sock)
 {
   int num_written;
 
@@ -610,8 +489,8 @@ http_default_write (socket_t sock)
   /* write error occurred */
   else if (num_written < 0)
     {
-      log_printf (LOG_ERROR, "http: send: %s\n", NET_ERROR);
-      if (last_errno == SOCK_UNAVAILABLE)
+      svz_log (LOG_ERROR, "http: send: %s\n", NET_ERROR);
+      if (svz_errno == SOCK_UNAVAILABLE)
 	{
 	  sock->unavailable = time (NULL) + RELAX_FD_TIME;
 	  num_written = 0;
@@ -626,7 +505,7 @@ http_default_write (socket_t sock)
   if ((sock->userflags & HTTP_FLAG_DONE) && sock->send_buffer_fill == 0)
     {
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "http: response successfully sent\n");
+      svz_log (LOG_DEBUG, "http: response successfully sent\n");
 #endif
       num_written = http_keep_alive (sock);
     }
@@ -642,13 +521,20 @@ http_default_write (socket_t sock)
 	  sock->send_buffer_fill = 42;
 	  sock->write_socket = http_cache_write;
 	}
-#if HAVE_SENDFILE
+#if ENABLE_SENDFILE
+#if defined (HAVE_SENDFILE) || defined (__MINGW32__)
+# ifdef __MINGW32__
+      else if (sock->userflags & HTTP_FLAG_SENDFILE && 
+	       svz_os_version >= WinNT4x)
+# else
       else if (sock->userflags & HTTP_FLAG_SENDFILE)
+# endif
 	{
 	  sock->send_buffer_fill = 42;
 	  sock->write_socket = http_send_file;
 	}
-#endif /* HAVE_SENDFILE */
+#endif /* HAVE_SENDFILE || __MINGW32__ */
+#endif /* ENABLE_SENDFILE */
     }
 
   /*
@@ -664,7 +550,7 @@ http_default_write (socket_t sock)
  * HTTP_FLAG_DONE is set.
  */
 int
-http_file_read (socket_t sock)
+http_file_read (svz_socket_t *sock)
 {
   int num_read;
   int do_read;
@@ -682,6 +568,7 @@ http_file_read (socket_t sock)
       return 0;
     }
 
+#ifndef __MINGW32__
   /*
    * Try to read as much data as possible from the file.
    */
@@ -691,9 +578,18 @@ http_file_read (socket_t sock)
   /* Read error occurred. */
   if (num_read < 0)
     {
-      log_printf (LOG_ERROR, "http: read: %s\n", SYS_ERROR);
+      svz_log (LOG_ERROR, "http: read: %s\n", SYS_ERROR);
       return -1;
     }
+#else
+  if (!ReadFile ((HANDLE) sock->file_desc,
+		 sock->send_buffer + sock->send_buffer_fill,
+		 do_read, (DWORD *) &num_read, NULL))
+    {
+      svz_log (LOG_ERROR, "http: ReadFile: %s\n", SYS_ERROR);
+      return -1;
+    }
+#endif
 
   /* Bogus file. File size from stat() was not true. */
   if (num_read == 0 && http->filelength != 0)
@@ -710,13 +606,13 @@ http_file_read (socket_t sock)
   if (http->filelength <= 0)
     {
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "http: file successfully read\n");
+      svz_log (LOG_DEBUG, "http: file successfully read\n");
 #endif
       /* 
        * no further read()s from the file descriptor, signaling 
        * the writers there will not be additional data from now on
        */
-      sock->read_socket = default_read;
+      sock->read_socket = svz_tcp_read_socket;
       sock->userflags |= HTTP_FLAG_DONE;
       sock->flags &= ~SOCK_FLAG_FILE;
     }
@@ -730,7 +626,7 @@ http_file_read (socket_t sock)
  * the receive buffer looks like an HTTP request.
  */
 int
-http_detect_proto (void *cfg, socket_t sock)
+http_detect_proto (svz_server_t *server, svz_socket_t *sock)
 {
   int n;
 
@@ -743,7 +639,7 @@ http_detect_proto (void *cfg, socket_t sock)
 		       http_request[n].len))
 	    {
 #if ENABLE_DEBUG
-	      log_printf (LOG_DEBUG, "http client detected\n");
+	      svz_log (LOG_DEBUG, "http client detected\n");
 #endif
 	      return -1;
 	    }
@@ -758,15 +654,15 @@ http_detect_proto (void *cfg, socket_t sock)
  * be called to set all the appropriate callbacks and socket flags.
  */
 int
-http_connect_socket (void *http_cfg, socket_t sock)
+http_connect_socket (svz_server_t *server, svz_socket_t *sock)
 {
   http_socket_t *http;
-  http_config_t *cfg = http_cfg;
+  http_config_t *cfg = server->cfg;
 
   /*
    * initialize the http socket structure
    */
-  http = xmalloc (sizeof (http_socket_t));
+  http = svz_malloc (sizeof (http_socket_t));
   memset (http, 0, sizeof (http_socket_t));
   http->pid = INVALID_HANDLE;
   http->keepalive = cfg->keepalive;
@@ -775,13 +671,13 @@ http_connect_socket (void *http_cfg, socket_t sock)
   /* start reverse dns lookup for logging purposes if necessary */
   if (cfg->nslookup)
     {
-      coserver_reverse (sock->remote_addr, http_remotehost, 
-			sock->id, sock->version);
+      svz_coserver_rdns (sock->remote_addr, http_remotehost, 
+			 sock->id, sock->version);
     }
   /* start user identification if necessary */
   if (cfg->ident)
     {
-      coserver_ident (sock, http_identification, sock->id, sock->version);
+      svz_coserver_ident (sock, http_identification, sock->id, sock->version);
     }
 
   /* 
@@ -803,7 +699,7 @@ http_connect_socket (void *http_cfg, socket_t sock)
  * seen a full HTTP request (ends with a double CRLF).
  */
 int
-http_handle_request (socket_t sock, int len)
+http_handle_request (svz_socket_t *sock, int len)
 {
   http_socket_t *http = sock->data;
   int n;
@@ -826,7 +722,7 @@ http_handle_request (socket_t sock, int len)
       return -1;
     }
   *p = 0;
-  request = xmalloc (p - line + 1);
+  request = svz_malloc (p - line + 1);
   strcpy (request, line);
   line = p + 1;
 
@@ -835,7 +731,7 @@ http_handle_request (socket_t sock, int len)
     p++;
   if (p == end)
     {
-      xfree (request);
+      svz_free (request);
       return -1;
     }
 
@@ -849,7 +745,7 @@ http_handle_request (socket_t sock, int len)
       flag |= HTTP_FLAG_SIMPLE;
       while (*p != '\r')
 	p++;
-      uri = xmalloc (p - line + 1);
+      uri = svz_malloc (p - line + 1);
       strncpy (uri, line, p - line);
       uri[p - line] = 0;
       line = p;
@@ -861,19 +757,19 @@ http_handle_request (socket_t sock, int len)
     {
       if (p <= line)
 	{
-	  xfree (request);
+	  svz_free (request);
 	  return -1;
 	}
       *p = 0;
-      uri = xmalloc (p - line + 1);
+      uri = svz_malloc (p - line + 1);
       strcpy (uri, line);
       line = p + 1;
   
       /* scan the version string of the HTTP request */
       if (memcmp (line, "HTTP/", 5))
 	{
-	  xfree (request);
-	  xfree (uri);
+	  svz_free (request);
+	  svz_free (uri);
 	  return -1;
 	}
       line += 5;
@@ -886,10 +782,10 @@ http_handle_request (socket_t sock, int len)
   /* check the remaining part of the first line the version */
   if (((version[MAJOR_VERSION] != HTTP_MAJOR_VERSION ||
 	version[MINOR_VERSION] > 1 || *(line - 2) != '.') && !flag) || 
-      INT16 (line) != CRLF)
+      SVZ_INT16 (line) != CRLF)
     {
-      xfree (request);
-      xfree (uri);
+      svz_free (request);
+      svz_free (uri);
       return -1;
     }
   line += 2;
@@ -902,7 +798,7 @@ http_handle_request (socket_t sock, int len)
 
   /* assign request properties to http structure */
   http->timestamp = time (NULL);
-  http->request = xmalloc (strlen (request) + strlen (uri) + 11);
+  http->request = svz_malloc (strlen (request) + strlen (uri) + 11);
   sprintf (http->request, "%s %s HTTP/%d.%d",
 	   request, uri, version[MAJOR_VERSION], version[MINOR_VERSION]);
 
@@ -912,7 +808,7 @@ http_handle_request (socket_t sock, int len)
       if (!memcmp (request, http_request[n].ident, http_request[n].len))
 	{
 #if ENABLE_DEBUG
-	  log_printf (LOG_DEBUG, "http: %s received\n", request);
+	  svz_log (LOG_DEBUG, "http: %s received\n", request);
 #endif
 	  http_request[n].response (sock, uri, flag);
 	  break;
@@ -925,8 +821,8 @@ http_handle_request (socket_t sock, int len)
       http_default_response (sock, uri, 0);
     }
 
-  xfree (request);
-  xfree (uri);
+  svz_free (request);
+  svz_free (uri);
   return 0;
 }
 
@@ -935,7 +831,7 @@ http_handle_request (socket_t sock, int len)
  * http request and call http_handle_request if necessary.
  */
 int 
-http_check_request (socket_t sock)
+http_check_request (svz_socket_t *sock)
 {
   char *p;
   int len;
@@ -943,10 +839,10 @@ http_check_request (socket_t sock)
   p = sock->recv_buffer;
 
   while (p < sock->recv_buffer + sock->recv_buffer_fill - 3 && 
-	 INT32 (p) != CRLF2)
+	 SVZ_INT32 (p) != CRLF2)
     p++;
   
-  if (INT32 (p) == CRLF2 && 
+  if (SVZ_INT32 (p) == CRLF2 && 
       p < sock->recv_buffer + sock->recv_buffer_fill - 3)
     {
       len = p - sock->recv_buffer + 4;
@@ -969,13 +865,13 @@ http_check_request (socket_t sock)
  * it for displaying the server configuration within the control protocol.
  */
 char *
-http_info_server (server_t *server)
+http_info_server (svz_server_t *server)
 {
   http_config_t *cfg = server->cfg;
   static char info[80 * 12];
   
   sprintf (info,
-	   " tcp port        : %d\r\n"
+	   " tcp bindings    : %s\r\n"
 	   " index file      : %s\r\n"
 	   " document root   : %s/\r\n"
 	   " cgi url         : %s/\r\n"
@@ -987,7 +883,7 @@ http_info_server (server_t *server)
 	   " default type    : %s\r\n"
 	   " type file       : %s\r\n"
 	   " content types   : %d",
-	   cfg->port->port,
+	   svz_server_bindings (server),
 	   cfg->indexfile,
 	   cfg->docs,
 	   cfg->cgiurl,
@@ -998,7 +894,7 @@ http_info_server (server_t *server)
 	   cfg->keepalive,
 	   cfg->default_type,
 	   cfg->type_file,
-	   hash_size (*(cfg->types)));
+	   svz_hash_size (cfg->types));
 
   return info;
 }
@@ -1007,23 +903,24 @@ http_info_server (server_t *server)
  * Client info callback for the http protocol.
  */
 char *
-http_info_client (void *http_cfg, socket_t sock)
+http_info_client (svz_server_t *server, svz_socket_t *sock)
 {
-  http_config_t *cfg = http_cfg;
   http_socket_t *http = sock->data;
   http_cache_t *cache = http->cache;
   static char info[80 * 32], text[80 * 8];
   int n;
 
   sprintf (info, "This is a http client connection.\r\n\r\n");
-#if HAVE_SENDFILE
+#if ENABLE_SENDFILE
+#if defined (HAVE_SENDFILE) || defined (__MINGW32__)
   if (sock->userflags & HTTP_FLAG_SENDFILE)
     {
       sprintf (text, "  * delivering via sendfile() (offset: %lu)\r\n",
 	       http->fileoffset);
       strcat (info, text);
     }
-#endif /* HAVE_SENDFILE */
+#endif /* HAVE_SENDFILE || __MINGW32__ */
+#endif /* ENABLE_SENDFILE */
   if (sock->userflags & HTTP_FLAG_KEEP)
     {
       sprintf (text, 
@@ -1092,70 +989,31 @@ http_info_client (void *http_cfg, socket_t sock)
  * request or a CGI request.
  */
 int
-http_get_response (socket_t sock, char *request, int flags)
+http_get_response (svz_socket_t *sock, char *request, int flags)
 {
   int fd;
   int size, status, partial = 0;
   struct stat buf;
-  char *cgifile, *dir, *host, *p, *file;
+  char *dir, *host, *p, *file;
   time_t date;
-  HANDLE cgi2s[2];
   http_cache_t *cache;
-  http_socket_t *http;
+  http_socket_t *http = sock->data;
   http_config_t *cfg = sock->cfg;
 
   /* reset current http header */
   http_reset_header ();
 
-  /* get the http socket structure */
-  http = sock->data;
-
   /* check if this is a cgi request */
-  cgifile = http_check_cgi (sock, request);
-  if (cgifile != HTTP_NO_CGI)
-    {
-      if (cgifile == NULL)
-	{
-	  sock_printf (sock, HTTP_INTERNAL_ERROR "\r\n");
-	  http_error_response (sock, 500);
-	  sock->userflags |= HTTP_FLAG_DONE;
-	  return -1;
-	}
-
-      /* create a pipe for the cgi script process */
-      if (pipe_create_pair (cgi2s) == -1)
-	{
-	  sock_printf (sock, HTTP_INTERNAL_ERROR "\r\n");
-	  http_error_response (sock, 500);
-	  sock->userflags |= HTTP_FLAG_DONE;
-	  xfree (cgifile);
-	  return -1;
-	}
-
-      /* execute the cgi script in CGIFILE */
-      sock->userflags |= HTTP_FLAG_CGI;
-      sock->flags |= SOCK_FLAG_RECV_PIPE;
-      sock->read_socket = http_cgi_read;
-      sock->pipe_desc[READ] = cgi2s[READ];
-      if (http_cgi_exec (sock, INVALID_HANDLE, cgi2s[WRITE], 
-			 cgifile, request, GET_METHOD))
-	{
-	  /* some error occurred here */
-	  sock->read_socket = default_read;
-	  xfree (cgifile);
-	  return -1;
-	}
-      xfree (cgifile);
-      return 0;
-    }
+  if ((status = http_cgi_get_response (sock, request, 0)) != -2)
+    return status;
 
   /* check for "~user" syntax here */
   if ((p = http_userdir (sock, request)) != NULL)
     {
       size = strlen (p) + strlen (cfg->indexfile) + 1;
-      file = xmalloc (size);
+      file = svz_malloc (size);
       strcpy (file, p);
-      xfree (p);
+      svz_free (p);
       status = 1;
     }
   /* this is a usual file request */
@@ -1164,7 +1022,7 @@ http_get_response (socket_t sock, char *request, int flags)
       size = 
 	strlen (cfg->docs) + strlen (request) + strlen (cfg->indexfile) + 4;
 
-      file = xmalloc (size);
+      file = svz_malloc (size);
       strcpy (file, cfg->docs);
       strcat (file, request);
       status = 0;
@@ -1183,23 +1041,23 @@ http_get_response (socket_t sock, char *request, int flags)
 	  if ((dir = http_dirlist (file, cfg->docs, 
 				   status ? request : NULL)) == NULL)
 	    {
-	      log_printf (LOG_ERROR, "http: dirlist: %s: %s\n", 
-			  file, SYS_ERROR);
-	      sock_printf (sock, HTTP_FILE_NOT_FOUND "\r\n");
+	      svz_log (LOG_ERROR, "http: dirlist: %s: %s\n", 
+		       file, SYS_ERROR);
+	      svz_sock_printf (sock, HTTP_FILE_NOT_FOUND "\r\n");
 	      http_error_response (sock, 404);
 	      sock->userflags |= HTTP_FLAG_DONE;
-	      xfree (file);
+	      svz_free (file);
 	      return -1;
 	    }
 	  /* send the directory listing */
 	  http->response = 200;
 	  http->length = strlen (dir);
-	  xfree (sock->send_buffer);
+	  svz_free (sock->send_buffer);
 	  sock->send_buffer = dir;
 	  sock->send_buffer_size = http_dirlist_size;
 	  sock->send_buffer_fill = strlen (dir);
 	  sock->userflags |= HTTP_FLAG_DONE;
-	  xfree (file);
+	  svz_free (file);
 	  return 0;
 	}
       close (fd);
@@ -1208,21 +1066,21 @@ http_get_response (socket_t sock, char *request, int flags)
   /* check if there are '..' in the requested file's path */
   if (strstr (request, "..") != NULL)
     {
-      sock_printf (sock, HTTP_ACCESS_DENIED "\r\n");
+      svz_sock_printf (sock, HTTP_ACCESS_DENIED "\r\n");
       http_error_response (sock, 403);
       sock->userflags |= HTTP_FLAG_DONE;
-      xfree (file);
+      svz_free (file);
       return -1;
     }
 
   /* get length of file and other properties */
   if (stat (file, &buf) == -1)
     {
-      log_printf (LOG_ERROR, "stat: %s (%s)\n", SYS_ERROR, file);
-      sock_printf (sock, HTTP_FILE_NOT_FOUND "\r\n");
+      svz_log (LOG_ERROR, "stat: %s (%s)\n", SYS_ERROR, file);
+      svz_sock_printf (sock, HTTP_FILE_NOT_FOUND "\r\n");
       http_error_response (sock, 404);
       sock->userflags |= HTTP_FLAG_DONE;
-      xfree (file);
+      svz_free (file);
       return -1;
     }
 
@@ -1233,11 +1091,11 @@ http_get_response (socket_t sock, char *request, int flags)
 #endif /* S_ISLNK */
 	S_ISDIR (buf.st_mode)))
     {
-      log_printf (LOG_ERROR, "http: %s is not a regular file\n", file);
-      sock_printf (sock, HTTP_ACCESS_DENIED "\r\n");
+      svz_log (LOG_ERROR, "http: %s is not a regular file\n", file);
+      svz_sock_printf (sock, HTTP_ACCESS_DENIED "\r\n");
       http_error_response (sock, 403);
       sock->userflags |= HTTP_FLAG_DONE;
-      xfree (file);
+      svz_free (file);
       return -1;
     }
 
@@ -1251,18 +1109,17 @@ http_get_response (socket_t sock, char *request, int flags)
 		       host ? "http://" : "", host ? host : "", request);
       http_send_header (sock);
       sock->userflags |= HTTP_FLAG_DONE;
-      xfree (file);
+      svz_free (file);
       return 0;
     }
 
   /* open the file for reading */
-  if ((fd = open (file, O_RDONLY | O_BINARY)) == -1)
+  if ((fd = svz_open (file, O_RDONLY | O_BINARY, 0)) == -1)
     {
-      log_printf (LOG_ERROR, "open: %s\n", SYS_ERROR);
-      sock_printf (sock, HTTP_FILE_NOT_FOUND "\r\n");
+      svz_sock_printf (sock, HTTP_FILE_NOT_FOUND "\r\n");
       http_error_response (sock, 404);
       sock->userflags |= HTTP_FLAG_DONE;
-      xfree (file);
+      svz_free (file);
       return -1;
     }
 
@@ -1282,15 +1139,15 @@ http_get_response (socket_t sock, char *request, int flags)
       if (date >= buf.st_mtime)
 	{
 #if ENABLE_DEBUG
-	  log_printf (LOG_DEBUG, "http: %s not changed\n", file);
+	  svz_log (LOG_DEBUG, "http: %s not changed\n", file);
 #endif
 	  http->response = 304;
 	  http_set_header (HTTP_NOT_MODIFIED);
 	  http_check_keepalive (sock);
 	  http_send_header (sock);
-	  close (fd);
+	  svz_close (fd);
 	  sock->userflags |= HTTP_FLAG_DONE;
-	  xfree (file);
+	  svz_free (file);
 	  return 0;
 	}
     }
@@ -1317,8 +1174,8 @@ http_get_response (socket_t sock, char *request, int flags)
 
       http->range.length = buf.st_size;
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "http: partial content: %ld-%ld/%ld\n",
-		  http->range.first, http->range.last, http->range.length);
+      svz_log (LOG_DEBUG, "http: partial content: %ld-%ld/%ld\n",
+	       http->range.first, http->range.last, http->range.length);
 #endif
     }
 
@@ -1340,14 +1197,14 @@ http_get_response (socket_t sock, char *request, int flags)
   /* just a HEAD response handled by this GET handler */
   if (flags & HTTP_FLAG_NOFILE)
     {
-      close (fd);
+      svz_close (fd);
       sock->userflags |= HTTP_FLAG_DONE;
-      xfree (file);
+      svz_free (file);
       return 0;
     }
 
   /* create a cache structure for the http socket structure */
-  cache = xmalloc (sizeof (http_cache_t));
+  cache = svz_calloc (sizeof (http_cache_t));
   http->cache = cache;
       
   /* return the file's current cache status */
@@ -1361,7 +1218,7 @@ http_get_response (socket_t sock, char *request, int flags)
 	{
 	  /* the file on disk has changed ? */
 #if ENABLE_DEBUG
-	  log_printf (LOG_DEBUG, "cache: %s has changed\n", file);
+	  svz_log (LOG_DEBUG, "cache: %s has changed\n", file);
 #endif
 	  http_refresh_cache (cache);
 	  cache->entry->date = buf.st_mtime;
@@ -1382,7 +1239,7 @@ http_get_response (socket_t sock, char *request, int flags)
 	      sock->send_buffer_fill = 42;
 	      sock->write_socket = http_cache_write;
 	    }
-	  close (fd);
+	  svz_close (fd);
 	}
     }
   /* the file is not in the cache structures yet */
@@ -1410,19 +1267,29 @@ http_get_response (socket_t sock, char *request, int flags)
        */
       else
 	{
-#if HAVE_SENDFILE
+#if ENABLE_SENDFILE && (HAVE_SENDFILE || defined (__MINGW32__))
+# ifdef __MINGW32__
+	  if (svz_os_version >= WinNT4x)
+	    {
+	      sock->read_socket = NULL;
+	      sock->flags &= ~SOCK_FLAG_FILE;
+	      sock->userflags |= HTTP_FLAG_SENDFILE;
+	    }
+	  else
+	    sock->read_socket = http_file_read;
+# else
 	  sock->read_socket = NULL;
 	  sock->flags &= ~SOCK_FLAG_FILE;
 	  sock->userflags |= HTTP_FLAG_SENDFILE;
-	  http_tcp_cork (sock->sock_desc, 1);
+	  svz_tcp_cork (sock->sock_desc, 1);
+# endif
 #else /* not HAVE_SENDFILE */
 	  sock->read_socket = http_file_read;
-#endif /* not HAVE_SENDFILE */
+#endif /* HAVE_SENDFILE || __MINGW32__ && ENABLE_SENDFILE */
 	}
     }
 
-  xfree (file);
-
+  svz_free (file);
   return 0;
 }
 
@@ -1432,7 +1299,7 @@ http_get_response (socket_t sock, char *request, int flags)
  * the file info.
  */
 int
-http_head_response (socket_t sock, char *request, int flags)
+http_head_response (svz_socket_t *sock, char *request, int flags)
 {
   http_get_response (sock, request, flags | HTTP_FLAG_NOFILE);
   return 0;
@@ -1442,9 +1309,9 @@ http_head_response (socket_t sock, char *request, int flags)
  * The default http response (Bad Request).
  */
 int
-http_default_response (socket_t sock, char *request, int flags)
+http_default_response (svz_socket_t *sock, char *request, int flags)
 {
-  sock_printf (sock, HTTP_NOT_IMPLEMENTED "\r\n");
+  svz_sock_printf (sock, HTTP_NOT_IMPLEMENTED "\r\n");
   http_error_response (sock, 501);
   sock->userflags |= HTTP_FLAG_DONE;
   return 0;
