@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: irc-proto.h,v 1.14 2000/09/28 21:12:54 ela Exp $
+ * $Id: irc-proto.h,v 1.20 2001/06/27 20:38:36 ela Exp $
  *
  */
 
@@ -32,9 +32,6 @@
 #define _GNU_SOURCE
 #include <time.h>
 
-#include "hash.h"
-#include "util.h"
-#include "server.h"
 #include "irc-core/irc-core.h"
 
 /* Some restrictions. */
@@ -147,7 +144,7 @@ struct irc_client
   char *server;            /* the server the client is connected to */
   irc_channel_t **channel; /* array of channels this client joined */
   int channels;            /* amount of channels the client joined */
-  socket_t sock;           /* this clients socket structure */
+  svz_socket_t *sock;      /* this clients socket structure */
   int flag;                /* this client's user flags */
   byte key;                /* the key */
   char *pass;              /* the given password */
@@ -232,7 +229,6 @@ struct irc_server
  */
 struct irc_configuration
 {
-  portcfg_t *netport;             /* port configuration */
   int operators;                  /* amount of logged in server operators */
   int users;                      /* amount of logged in users */
   int unknowns;                   /* unknown connections */
@@ -280,7 +276,7 @@ struct irc_configuration
    * :maximum number of links in this class 
    * :send queue size
    */
-  char **YLine;
+  svz_array_t *YLine;
 
   /*
    * I [Mandatory] -- authorize client, wildcards permitted, a valid client
@@ -292,7 +288,7 @@ struct irc_configuration
    * :password (optional)
    * :connection class number (YLine)
    */
-  char **ILine;
+  svz_array_t *ILine;
 
   /*
    * O [Optional] -- authorize operator, wildcards allowed
@@ -300,13 +296,13 @@ struct irc_configuration
    * :password
    * :nick
    */
-  char **OLine;
+  svz_array_t *OLine;
 
   /*
    * o [Optional] -- authorize local operator, see above at the O lines for
    *                 description
    */
-  char **oLine;
+  svz_array_t *oLine;
 
   /*
    * Note: + C and N lines can also use the user@ combination in order
@@ -322,7 +318,7 @@ struct irc_configuration
    * :port (if not given we will not connect)
    * :connection class number (YLine)
    */
-  char **CLine;
+  svz_array_t *CLine;
 
   /*
    * N [Networked] -- server which may connect
@@ -334,7 +330,7 @@ struct irc_configuration
    *  front and be replaced with a *.
    * :connection class number (YLine)
    */
-  char **NLine;
+  svz_array_t *NLine;
 
   /*
    * K [Optional] -- kill user, wildcards allowed
@@ -342,7 +338,7 @@ struct irc_configuration
    * :time of day
    * :user
    */
-  char **KLine;
+  svz_array_t *KLine;
 
   char *pass;                     /* server password */
   char *info;                     /* server info */
@@ -352,8 +348,8 @@ struct irc_configuration
   char *location1;                /* city, state, country */
   char *location2;                /* university, department */
 
-  hash_t *channels;               /* channel hash */
-  hash_t *clients;                /* client hash */
+  svz_hash_t *channels;           /* channel hash */
+  svz_hash_t *clients;            /* client hash */
   irc_server_t *servers;          /* server list root */
   irc_client_history_t *history;  /* client history list root */
   irc_class_t *classes;           /* connection classes list */
@@ -370,34 +366,32 @@ typedef struct
 {
   int count;     /* the command has been count times processed */
   char *request; /* name of the command */
-  int (* func)(socket_t, irc_client_t *, irc_request_t *);
+  int (* func)(svz_socket_t *, irc_client_t *, irc_request_t *);
 }
 irc_callback_t;
 
 extern irc_callback_t irc_callback[];
 
 /* Export the irc server definition to `server.c'. */
-extern server_definition_t irc_server_definition;
+extern svz_servertype_t irc_server_definition;
 
 /* these functions can be used by all of the IRC event subsections */
-int irc_client_in_channel (socket_t, irc_client_t *, irc_channel_t *);
-int irc_check_args (socket_t, irc_client_t *, irc_config_t *, 
+int irc_client_in_channel (svz_socket_t *, irc_client_t *, irc_channel_t *);
+int irc_check_args (svz_socket_t *, irc_client_t *, irc_config_t *, 
 		    irc_request_t *, int);
 int irc_client_absent (irc_client_t *, irc_client_t *);
 #ifndef __STDC__
 int irc_printf ();
 #else
-int irc_printf (socket_t, const char *, ...);
+int irc_printf (svz_socket_t *, const char *, ...);
 #endif
 
 /* serveez callbacks */
-int irc_handle_request (socket_t sock, char *request, int len);
-int irc_disconnect (socket_t sock);
-int irc_idle (socket_t sock);
+int irc_handle_request (svz_socket_t *sock, char *request, int len);
+int irc_disconnect (svz_socket_t *sock);
+int irc_idle (svz_socket_t *sock);
 
 /* channel operations */
-static int irc_delete_channel (irc_config_t *cfg, irc_channel_t *);
-static irc_channel_t *irc_add_channel (irc_config_t *cfg, char *channel);
 irc_channel_t *irc_find_channel (irc_config_t *cfg, char *channel);
 irc_channel_t **irc_regex_channel (irc_config_t *cfg, char *regex);
 int irc_join_channel (irc_config_t *cfg, irc_client_t *client, char *chan);
@@ -417,10 +411,10 @@ irc_client_history_t *irc_find_nick_history (irc_config_t *,
 					     irc_client_history_t *, char *);
 
 /* irc server functions */
-int irc_init (server_t *server);
-int irc_global_init (void);
-int irc_finalize (server_t *server);
-int irc_global_finalize (void);
+int irc_init (svz_server_t *server);
+int irc_global_init (svz_servertype_t *server);
+int irc_finalize (svz_server_t *server);
+int irc_global_finalize (svz_servertype_t *server);
 
 #define IRC_CLOSING_LINK    "Closing Link: %s (%s)"
 #define IRC_CONNECTION_LOST "Connection reset by peer"
