@@ -1,7 +1,7 @@
 /*
  * gnutella.c - gnutella protocol implementation
  *
- * Copyright (C) 2000 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2000, 2001 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: gnutella.c,v 1.28 2000/12/30 01:59:33 ela Exp $
+ * $Id: gnutella.c,v 1.41 2001/06/27 20:38:37 ela Exp $
  *
  */
 
@@ -49,7 +49,7 @@
 #endif
 
 #ifdef __MINGW32__
-# include <winsock.h>
+# include <winsock2.h>
 # include <io.h>
 #endif
 
@@ -60,12 +60,7 @@
 # define mkdir(path, mode) mkdir (path)
 #endif
 
-#include "alloc.h"
-#include "util.h"
-#include "socket.h"
-#include "connect.h"
-#include "server.h"
-#include "server-core.h"
+#include "libserveez.h"
 #include "gnutella.h"
 #include "nut-transfer.h"
 #include "nut-route.h"
@@ -73,20 +68,6 @@
 #include "nut-hostlist.h"
 #include "nut-request.h"
 
-/*
- * This port configuration is the default port for the gnutella server to
- * listen on.
- */
-portcfg_t nut_port = 
-{
-  PROTO_TCP,      /* tcp port */
-  NUT_PORT,       /* standard port to listen on */
-  "*",            /* bind all local addresses */
-  NULL,           /* calculated later */
-  NULL,           /* no inpipe */
-  NULL            /* no outpipe */
-};
-
 /*
  * Default search patterns.
  */
@@ -102,7 +83,6 @@ char *nut_search_patterns[] =
  */
 nut_config_t nut_config = 
 {
-  &nut_port,           /* port configuration */
   0,                   /* if set we do not listen on the above port cfg */
   NUT_MAX_TTL,         /* maximum ttl for a gnutella packet */
   NUT_TTL,             /* initial ttl for a gnutella packet */
@@ -110,7 +90,7 @@ nut_config_t nut_config =
   NUT_GUID,            /* this servers GUID */
   NULL,                /* routing table */
   NULL,                /* connected hosts hash */
-  nut_search_patterns, /* default search pattern */
+  NULL,                /* default search pattern */
   0,                   /* current search pattern index */
   30,                  /* limit amount of search reply records */
   NULL,                /* this servers created packets */
@@ -146,32 +126,38 @@ nut_config_t nut_config =
 /*
  * Defining configuration file associations with key-value-pairs.
  */
-key_value_pair_t nut_config_prototype[] = 
+svz_key_value_pair_t nut_config_prototype[] = 
 {
-  REGISTER_PORTCFG ("port", nut_config.netport, DEFAULTABLE),
-  REGISTER_STRARRAY ("hosts", nut_config.hosts, NOTDEFAULTABLE),
-  REGISTER_STRARRAY ("search", nut_config.search, DEFAULTABLE),
-  REGISTER_INT ("search-limit", nut_config.search_limit, DEFAULTABLE),
-  REGISTER_INT ("max-ttl", nut_config.max_ttl, DEFAULTABLE),
-  REGISTER_INT ("ttl", nut_config.ttl, DEFAULTABLE),
-  REGISTER_STR ("download-path", nut_config.save_path, DEFAULTABLE),
-  REGISTER_STR ("share-path", nut_config.share_path, DEFAULTABLE),
-  REGISTER_INT ("max-downloads", nut_config.max_dnloads, DEFAULTABLE),
-  REGISTER_INT ("connection-speed", nut_config.speed, DEFAULTABLE),
-  REGISTER_INT ("min-speed", nut_config.min_speed, DEFAULTABLE),
-  REGISTER_STRARRAY ("file-extensions", nut_config.extensions, DEFAULTABLE),
-  REGISTER_INT ("connections", nut_config.connections, DEFAULTABLE),
-  REGISTER_STR ("force-ip", nut_config.force_ip, DEFAULTABLE),
-  REGISTER_INT ("force-port", nut_config.force_port, DEFAULTABLE),
-  REGISTER_INT ("max-uploads", nut_config.max_uploads, DEFAULTABLE),
-  REGISTER_STR ("net-url", nut_config.net_url, DEFAULTABLE),
-  REGISTER_END ()
+  SVZ_REGISTER_STRARRAY ("hosts", nut_config.hosts, SVZ_ITEM_NOTDEFAULTABLE),
+  SVZ_REGISTER_STRARRAY ("search", nut_config.search, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_INT ("search-limit", nut_config.search_limit, 
+		    SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_INT ("max-ttl", nut_config.max_ttl, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_INT ("ttl", nut_config.ttl, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_STR ("download-path", nut_config.save_path, 
+		    SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_STR ("share-path", nut_config.share_path, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_INT ("max-downloads", nut_config.max_dnloads, 
+		    SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_INT ("connection-speed", nut_config.speed, 
+		    SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_INT ("min-speed", nut_config.min_speed, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_STRARRAY ("file-extensions", nut_config.extensions, 
+			 SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_INT ("connections", nut_config.connections, 
+		    SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_STR ("force-ip", nut_config.force_ip, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_INT ("force-port", nut_config.force_port, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_INT ("max-uploads", nut_config.max_uploads, 
+		    SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_STR ("net-url", nut_config.net_url, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_END ()
 };
 
 /*
  * Definition of this server.
  */
-server_definition_t nut_server_definition =
+svz_servertype_t nut_server_definition =
 {
   "gnutella spider version " NUT_VERSION, /* long description */
   "nut",                                  /* instance description */
@@ -227,7 +213,7 @@ nut_hash_code (char *id)
  * certain time.
  */
 int
-nut_connect_timeout (socket_t sock)
+nut_connect_timeout (svz_socket_t *sock)
 {
   /* 
    * Did we try to connect to another host in order to download something,
@@ -249,7 +235,7 @@ static int
 nut_connect_host (nut_config_t *cfg, char *host)
 {
   nut_host_t *client;
-  socket_t sock;
+  svz_socket_t *sock;
   unsigned long ip;
   unsigned short port;
   int ret = -1;
@@ -257,25 +243,25 @@ nut_connect_host (nut_config_t *cfg, char *host)
   /* try getting ip address and port */
   if (nut_parse_addr (host, &ip, &port) == -1)
     {
-      log_printf (LOG_WARNING, "nut: invalid host `%s'\n", host);
+      svz_log (LOG_WARNING, "nut: invalid host `%s'\n", host);
       return ret;
     }
 
   /* get client from host catcher hash */
-  client = (nut_host_t *) hash_get (cfg->net, host);
+  client = (nut_host_t *) svz_hash_get (cfg->net, host);
 
   /* try to connect to this host */
-  if ((sock = sock_connect (ip, port)) != NULL)
+  if ((sock = svz_tcp_connect (ip, port)) != NULL)
     {
-      log_printf (LOG_NOTICE, "nut: connecting %s:%u\n",
-		  util_inet_ntoa (ip), ntohs (port));
+      svz_log (LOG_NOTICE, "nut: connecting %s:%u\n",
+	       svz_inet_ntoa (ip), ntohs (port));
       
       sock->cfg = cfg;
       sock->flags |= SOCK_FLAG_NOFLOOD;
       sock->check_request = nut_detect_connect;
       sock->idle_func = nut_connect_timeout;
       sock->idle_counter = NUT_CONNECT_TIMEOUT;
-      sock_printf (sock, NUT_CONNECT);
+      svz_sock_printf (sock, NUT_CONNECT);
       ret = 0;
     }
 
@@ -285,8 +271,8 @@ nut_connect_host (nut_config_t *cfg, char *host)
    */
   if (client)
     {
-      hash_delete (cfg->net, host);
-      xfree (client);
+      svz_hash_delete (cfg->net, host);
+      svz_free (client);
     }
   return ret;
 }
@@ -296,7 +282,7 @@ nut_connect_host (nut_config_t *cfg, char *host)
  * functions pings it to get all further servers behind this server.
  */
 int
-nut_init_ping (socket_t sock)
+nut_init_ping (svz_socket_t *sock)
 {
   nut_config_t *cfg = sock->cfg;
   nut_client_t *client = sock->data;
@@ -313,10 +299,10 @@ nut_init_ping (socket_t sock)
   header = nut_put_header (&hdr);
 
   /* put into sent packet hash */
-  pkt = xmalloc (sizeof (nut_packet_t));
+  pkt = svz_malloc (sizeof (nut_packet_t));
   pkt->sock = sock;
   pkt->sent = time (NULL);
-  hash_put (cfg->packet, (char *) hdr.id, pkt);
+  svz_hash_put (cfg->packet, (char *) hdr.id, pkt);
 
   /* update client and server statistics */
   cfg->nodes -= client->nodes;
@@ -326,14 +312,14 @@ nut_init_ping (socket_t sock)
   client->files = 0;
   client->size = 0;
 
-  return sock_write (sock, (char *) header, SIZEOF_NUT_HEADER);
+  return svz_sock_write (sock, (char *) header, SIZEOF_NUT_HEADER);
 }
 
 /*
  * The gnutella servers global initializer.
  */
 int
-nut_global_init (void)
+nut_global_init (svz_servertype_t *server)
 {
 #ifdef __MINGW32__
   /* try getting M$'s GUID creation routine */
@@ -347,6 +333,9 @@ nut_global_init (void)
   /* initialize random seed */
   srand (time (NULL));
 
+  /* initialize configuration default values */
+  nut_config.search = svz_config_strarray_create (nut_search_patterns);
+
 #if 0
   /* Print structure sizes. */
   printf ("header     : %d\n", sizeof (nut_header_t));
@@ -370,7 +359,7 @@ nut_global_init (void)
  * Gnutella spider server's instance initializer.
  */
 int
-nut_init (server_t *server)
+nut_init (svz_server_t *server)
 {
   nut_config_t *cfg = server->cfg;
   int n = 0;
@@ -380,7 +369,7 @@ nut_init (server_t *server)
   /* check the download and share path first */
   if (strlen (cfg->save_path) == 0 || strlen (cfg->share_path) == 0)
     {
-      log_printf (LOG_ERROR, "nut: no download/share path given\n");
+      svz_log (LOG_ERROR, "nut: no download/share path given\n");
       return -1;
     }
   p = cfg->save_path + strlen (cfg->save_path) - 1;
@@ -398,22 +387,22 @@ nut_init (server_t *server)
 	  /* create the download directory */
 	  if (mkdir (cfg->save_path, 0755) == -1)
 	    {
-	      log_printf (LOG_ERROR, "nut: mkdir: %s\n", SYS_ERROR);
+	      svz_log (LOG_ERROR, "nut: mkdir: %s\n", SYS_ERROR);
 	      return -1;
 	    }
 	}
       /* check if the given path is a directory already */
       else if (!S_ISDIR (buf.st_mode))
 	{
-	  log_printf (LOG_ERROR, "nut: %s is not a directory\n", 
-		      cfg->save_path);
+	  svz_log (LOG_ERROR, "nut: %s is not a directory\n", 
+		   cfg->save_path);
 	  return -1;
 	}
     }
 
   /* read shared files */
   nut_read_database (cfg, cfg->share_path[0] ? cfg->share_path : "/");
-  log_printf (LOG_NOTICE, "nut: %d files in database\n", cfg->db_files);
+  svz_log (LOG_NOTICE, "nut: %d files in database\n", cfg->db_files);
 
   /* calculate forced local ip and port if necessary */
   if (cfg->force_ip)
@@ -423,31 +412,31 @@ nut_init (server_t *server)
   cfg->port = htons ((unsigned short) cfg->force_port);
 
   /* create and modify packet hash */
-  cfg->packet = hash_create (4);
+  cfg->packet = svz_hash_create (4);
   cfg->packet->code = nut_hash_code;
   cfg->packet->equals = nut_hash_equals;
   cfg->packet->keylen = nut_hash_keylen;
 
   /* create and modify reply hash */
-  cfg->reply = hash_create (4);
+  cfg->reply = svz_hash_create (4);
   cfg->reply->code = nut_hash_code;
   cfg->reply->equals = nut_hash_equals;
   cfg->reply->keylen = nut_hash_keylen;
 
   /* create current connection hash */
-  cfg->conn = hash_create (4);
+  cfg->conn = svz_hash_create (4);
 
   /* create host catcher hash */
-  cfg->net = hash_create (4);
+  cfg->net = svz_hash_create (4);
 
   /* create recent query hash */
-  cfg->query = hash_create (4);
+  cfg->query = svz_hash_create (4);
 
   /* create push request hash */
-  cfg->push = hash_create (4);
+  cfg->push = svz_hash_create (4);
 
   /* create and modify the routing table hash */
-  cfg->route = hash_create (4);
+  cfg->route = svz_hash_create (4);
   cfg->route->code = nut_hash_code;
   cfg->route->equals = nut_hash_equals;
   cfg->route->keylen = nut_hash_keylen;
@@ -456,22 +445,18 @@ nut_init (server_t *server)
   nut_calc_guid (cfg->guid);
 
   /* create detection string for gnutella host list */
-  cfg->net_detect = xmalloc (strlen (NUT_HOSTS) + strlen (cfg->net_url) + 1);
+  cfg->net_detect = svz_malloc (strlen (NUT_HOSTS) + 
+				strlen (cfg->net_url) + 1);
   sprintf (cfg->net_detect, NUT_HOSTS, cfg->net_url);
 
   /* go through all given hosts and try to connect to them */
-  if (cfg->hosts)
-    {
-      while (cfg->hosts[n])
-	{
-	  nut_connect_host (cfg, cfg->hosts[n]);
-	  n++;
-	}
-    }
+  svz_array_foreach (cfg->hosts, p, n)
+    nut_connect_host (cfg, p);
 
-  /* bind listening server to configurable port address */
+  /* bind listening server to configurable port address 
+     FIXME: how can we disable bindings ?
   if (!cfg->disable)
-    server_bind (server, cfg->netport);
+    server_bind (server, cfg->netport);*/
   return 0;
 }
 
@@ -479,7 +464,7 @@ nut_init (server_t *server)
  * Instance finalizer.
  */
 int
-nut_finalize (server_t *server)
+nut_finalize (svz_server_t *server)
 {
   nut_config_t *cfg = server->cfg;
   nut_host_t **client;
@@ -490,47 +475,47 @@ nut_finalize (server_t *server)
   /* destroy sharing files */
   nut_destroy_database (cfg);
 
-  hash_destroy (cfg->conn);
-  hash_destroy (cfg->route);
-  hash_destroy (cfg->query);
-  hash_destroy (cfg->reply);
+  svz_hash_destroy (cfg->conn);
+  svz_hash_destroy (cfg->route);
+  svz_hash_destroy (cfg->query);
+  svz_hash_destroy (cfg->reply);
 
   /* destroy sent packet hash */
-  if ((pkt = (nut_packet_t **) hash_values (cfg->packet)) != NULL)
+  if ((pkt = (nut_packet_t **) svz_hash_values (cfg->packet)) != NULL)
     {
-      for (n = 0; n < hash_size (cfg->packet); n++)
+      for (n = 0; n < svz_hash_size (cfg->packet); n++)
 	{
-	  xfree (pkt[n]);
+	  svz_free (pkt[n]);
 	}
-      hash_xfree (pkt);
+      svz_hash_xfree (pkt);
     }
-  hash_destroy (cfg->packet);
+  svz_hash_destroy (cfg->packet);
 
   /* destroy host catcher hash */
-  if ((client = (nut_host_t **) hash_values (cfg->net)) != NULL)
+  if ((client = (nut_host_t **) svz_hash_values (cfg->net)) != NULL)
     {
-      for (n = 0; n < hash_size (cfg->net); n++)
+      for (n = 0; n < svz_hash_size (cfg->net); n++)
 	{
-	  xfree (client[n]);
+	  svz_free (client[n]);
 	}
-      hash_xfree (client);
+      svz_hash_xfree (client);
     }
-  hash_destroy (cfg->net);
+  svz_hash_destroy (cfg->net);
 
   /* destroy push request hash */
-  if ((transfer = (nut_transfer_t **) hash_values (cfg->push)) != NULL)
+  if ((transfer = (nut_transfer_t **) svz_hash_values (cfg->push)) != NULL)
     {
-      for (n = 0; n < hash_size (cfg->push); n++)
+      for (n = 0; n < svz_hash_size (cfg->push); n++)
 	{
-	  xfree (transfer[n]->file);
-	  xfree (transfer[n]);
+	  svz_free (transfer[n]->file);
+	  svz_free (transfer[n]);
 	}
-      hash_xfree (transfer);
+      svz_hash_xfree (transfer);
     }
-  hash_destroy (cfg->push);
+  svz_hash_destroy (cfg->push);
 
   /* free detection string */
-  xfree (cfg->net_detect);
+  svz_free (cfg->net_detect);
 
   return 0;
 }
@@ -539,13 +524,16 @@ nut_finalize (server_t *server)
  * Global gnutella finalizer.
  */
 int
-nut_global_finalize (void)
+nut_global_finalize (svz_servertype_t *server)
 {
 #ifdef __MINGW32__
   if (oleHandle) 
     FreeLibrary (oleHandle);
 #endif /* __MINGW32__ */
 
+  /* destroy confguration defaults */
+  svz_config_strarray_destroy (nut_config.search);
+
   return 0;
 }
 
@@ -554,7 +542,7 @@ nut_global_finalize (void)
  * connections.
  */
 int
-nut_disconnect (socket_t sock)
+nut_disconnect (svz_socket_t *sock)
 {
   nut_config_t *cfg = sock->cfg;
   nut_host_t *host;
@@ -565,36 +553,36 @@ nut_disconnect (socket_t sock)
   nut_client_t *client = sock->data;
 
   /* delete all push request routing information for this connection */
-  while ((id = (byte *) hash_contains (cfg->reply, sock)) != NULL)
-    hash_delete (cfg->reply, (char *) id);
+  while ((id = (byte *) svz_hash_contains (cfg->reply, sock)) != NULL)
+    svz_hash_delete (cfg->reply, (char *) id);
 
   /* delete all routing information for this connection */
-  while ((id = (byte *) hash_contains (cfg->route, sock)) != NULL)
-    hash_delete (cfg->route, (char *) id);
+  while ((id = (byte *) svz_hash_contains (cfg->route, sock)) != NULL)
+    svz_hash_delete (cfg->route, (char *) id);
 
   /* drop all packet information for this connection */
-  if ((keys = (char **) hash_keys (cfg->packet)) != NULL)
+  if ((keys = (char **) svz_hash_keys (cfg->packet)) != NULL)
     {
-      size = hash_size (cfg->packet);
+      size = svz_hash_size (cfg->packet);
       for (n = 0; n < size; n++)
 	{
-	  pkt = (nut_packet_t *) hash_get (cfg->packet, keys[n]);
+	  pkt = (nut_packet_t *) svz_hash_get (cfg->packet, keys[n]);
 	  if (pkt->sock == sock)
 	    {
-	      hash_delete (cfg->packet, keys[n]);
-	      xfree (pkt);
+	      svz_hash_delete (cfg->packet, keys[n]);
+	      svz_free (pkt);
 	    }
 	}
-      hash_xfree (keys);
+      svz_hash_xfree (keys);
     }
   
   /* remove this socket from the current connection hash */
   key = nut_client_key (sock->remote_addr, sock->remote_port);
-  hash_delete (cfg->conn, key);
+  svz_hash_delete (cfg->conn, key);
 
   /* remove the connection from the host catcher */
-  if ((host = hash_delete (cfg->net, key)) != NULL)
-    xfree (host);
+  if ((host = svz_hash_delete (cfg->net, key)) != NULL)
+    svz_free (host);
 
   /* free client structure */
   if (client)
@@ -602,7 +590,7 @@ nut_disconnect (socket_t sock)
       cfg->nodes -= client->nodes;
       cfg->files -= client->files;
       cfg->size -= client->size;
-      xfree (client);
+      svz_free (client);
       sock->data = NULL;
     }
 
@@ -614,7 +602,7 @@ nut_disconnect (socket_t sock)
  * routine. Here we try connecting to more gnutella hosts.
  */
 int
-nut_server_notify (server_t *server)
+nut_server_notify (svz_server_t *server)
 {
   nut_config_t *cfg = server->cfg;
   static int count = NUT_CONNECT_INTERVAL;
@@ -628,57 +616,57 @@ nut_server_notify (server_t *server)
     return 0;
     
   /* do we have enough connections ? */
-  connect = cfg->connections - hash_size (cfg->conn);
+  connect = cfg->connections - svz_hash_size (cfg->conn);
   if (connect > 0)
     {
       /* are there hosts in the host catcher hash ? */
-      if ((keys = (char **) hash_keys (cfg->net)) != NULL)
+      if ((keys = (char **) svz_hash_keys (cfg->net)) != NULL)
 	{
 	  /* go through all caught hosts */
-	  for (n = 0; n < hash_size (cfg->net) && connect; n++)
+	  for (n = 0; n < svz_hash_size (cfg->net) && connect; n++)
 	    {
 	      /* check if we are not already connected */
-	      if (hash_get (cfg->conn, keys[n]) == NULL)
+	      if (svz_hash_get (cfg->conn, keys[n]) == NULL)
 		{
 		  if (nut_connect_host (cfg, keys[n]) != -1)
 		    connect--;
 		}
 	    }
-	  hash_xfree (keys);
+	  svz_hash_xfree (keys);
 	}
     }
 
   /* go through the sent packet hash and drop old entries */
-  if ((keys = (char **) hash_keys (cfg->packet)) != NULL)
+  if ((keys = (char **) svz_hash_keys (cfg->packet)) != NULL)
     {
       t = time (NULL);
-      size = hash_size (cfg->packet);
+      size = svz_hash_size (cfg->packet);
       for (n = 0; n < size; n++)
 	{
-	  pkt = (nut_packet_t *) hash_get (cfg->packet, keys[n]);
+	  pkt = (nut_packet_t *) svz_hash_get (cfg->packet, keys[n]);
 	  if (t - pkt->sent > NUT_ENTRY_AGE)
 	    {
-	      hash_delete (cfg->packet, keys[n]);
-	      xfree (pkt);
+	      svz_hash_delete (cfg->packet, keys[n]);
+	      svz_free (pkt);
 	    }
 	}
-      hash_xfree (keys);
+      svz_hash_xfree (keys);
     }
 
   /* drop older entries from the recent query hash */
-  if ((keys = (char **) hash_keys (cfg->query)) != NULL)
+  if ((keys = (char **) svz_hash_keys (cfg->query)) != NULL)
     {
       t = time (NULL);
-      size = hash_size (cfg->query);
+      size = svz_hash_size (cfg->query);
       for (n = 0; n < size; n++)
 	{
-	  received = (time_t) hash_get (cfg->query, keys[n]);
+	  received = (time_t) (long) svz_hash_get (cfg->query, keys[n]);
 	  if (t - received > NUT_ENTRY_AGE)
 	    {
-	      hash_delete (cfg->query, keys[n]);
+	      svz_hash_delete (cfg->query, keys[n]);
 	    }
 	}
-      hash_xfree (keys);
+      svz_hash_xfree (keys);
     }
 
   /* wake up in a certain time */
@@ -691,9 +679,8 @@ nut_server_notify (server_t *server)
  * routine.
  */
 int
-nut_check_request (socket_t sock)
+nut_check_request (svz_socket_t *sock)
 {
-  nut_config_t *cfg = sock->cfg;
   nut_client_t *client = sock->data;
   nut_header_t *hdr;
   byte *packet;
@@ -712,8 +699,8 @@ nut_check_request (socket_t sock)
 	  packet = (byte *) sock->recv_buffer + SIZEOF_NUT_HEADER;
 	  client->packets++;
 #if 0
-	  util_hexdump (stdout, "gnutella packet", sock->sock_desc,
-			sock->recv_buffer, len, 0);
+	  svz_hexdump (stdout, "gnutella packet", sock->sock_desc,
+		       sock->recv_buffer, len, 0);
 #endif
 	  
 	  /* try to route the packet */
@@ -749,7 +736,7 @@ nut_check_request (socket_t sock)
 	    return -1;
 
 	  /* cut this packet from the send buffer queue */
-	  sock_reduce_recv (sock, len);
+	  svz_sock_reduce_recv (sock, len);
 	}
       else
 	break;
@@ -762,7 +749,7 @@ nut_check_request (socket_t sock)
  * connection. We will regularly search for specific files.
  */
 int
-nut_idle_searching (socket_t sock)
+nut_idle_searching (svz_socket_t *sock)
 {
   nut_config_t *cfg = sock->cfg;
   nut_packet_t *pkt;
@@ -772,15 +759,18 @@ nut_idle_searching (socket_t sock)
   char *text;
 
   /* search strings given ? */
-  if (cfg->search && cfg->search[0])
+  if (cfg->search && svz_array_size (cfg->search) > 0)
     {
       /* get next search string */
-      if ((text = cfg->search[cfg->search_index]) != NULL)
-	cfg->search_index++;
+      if (svz_array_size (cfg->search) > (unsigned long) cfg->search_index)
+	{
+	  text = svz_array_get (cfg->search, cfg->search_index);
+	  cfg->search_index++;
+	}
       else
 	{
 	  cfg->search_index = 0;
-	  text = cfg->search[0];
+	  text = svz_array_get (cfg->search, 0);
 	}
 
       /* create new gnutella packet */
@@ -794,18 +784,18 @@ nut_idle_searching (socket_t sock)
       search = nut_put_query (&query);
 
       /* try sending this packet to this connection */
-      if (sock_write (sock, (char *) header, SIZEOF_NUT_HEADER) == -1 ||
-	  sock_write (sock, (char *) search, SIZEOF_NUT_QUERY) == -1 ||
-	  sock_write (sock, text, strlen (text) + 1) == -1)
+      if (svz_sock_write (sock, (char *) header, SIZEOF_NUT_HEADER) == -1 ||
+	  svz_sock_write (sock, (char *) search, SIZEOF_NUT_QUERY) == -1 ||
+	  svz_sock_write (sock, text, strlen (text) + 1) == -1)
 	{
 	  return -1;
 	}
       
       /* save this packet for later routing */
-      pkt = xmalloc (sizeof (nut_packet_t));
+      pkt = svz_malloc (sizeof (nut_packet_t));
       pkt->sock = sock;
       pkt->sent = time (NULL);
-      hash_put (cfg->packet, (char *) hdr.id, pkt);
+      svz_hash_put (cfg->packet, (char *) hdr.id, pkt);
     }
 
   /* wake up in a certain time */
@@ -817,38 +807,35 @@ nut_idle_searching (socket_t sock)
  * Gnutella server info callback.
  */
 char *
-nut_info_server (server_t *server)
+nut_info_server (svz_server_t *server)
 {
   nut_config_t *cfg = server->cfg;
   static char info[80 * 19];
-  char *ext = NULL;
+  char *e, *ext = NULL;
   int n;
 
   /* create file extension list */
-  n = 0;
   if (cfg->extensions)
     {
-      while (cfg->extensions[n])
+      svz_array_foreach (cfg->extensions, e, n)
 	{
 	  if (!ext)
 	    {
-	      ext = xmalloc (strlen (cfg->extensions[n]) + 2);
-	      strcpy (ext, cfg->extensions[n]);
+	      ext = svz_malloc (strlen (e) + 2);
+	      strcpy (ext, e);
 	    }
 	  else
 	    {
-	      ext = xrealloc (ext, strlen (ext) + 
-			      strlen (cfg->extensions[n]) + 2);
-	      strcat (ext, cfg->extensions[n]);
+	      ext = svz_realloc (ext, strlen (ext) + strlen (e) + 2);
+	      strcat (ext, e);
 	    }
-	  n++;
 	  strcat (ext, ";");
 	}
       ext[strlen (ext) - 1] = '\0';
     }
 
   sprintf (info,
-	   " tcp port        : %u\r\n"
+	   " tcp bindings    : %s\r\n"
 	   " force ip        : %s\r\n"
 	   " force port      : %s\r\n"
 	   " maximum ttl     : %u\r\n"
@@ -869,31 +856,33 @@ nut_info_server (server_t *server)
 	   " downloads       : %u/%u\r\n"
 	   " uploads         : %u/%u\r\n"
 	   " recent queries  : %u",
-	   cfg->netport->port,
-	   cfg->ip ? util_inet_ntoa (cfg->ip) : "no specified",
-	   cfg->port ? util_itoa (ntohs (cfg->port)) : "no specified",
+	   svz_server_bindings (server),
+	   cfg->ip ? svz_inet_ntoa (cfg->ip) : "no specified",
+	   cfg->port ? svz_itoa (ntohs (cfg->port)) : "no specified",
 	   cfg->max_ttl,
 	   cfg->ttl,
 	   cfg->speed,
 	   nut_print_guid (cfg->guid),
 	   cfg->save_path,
 	   cfg->share_path,
-	   cfg->search && cfg->search[0] ? (cfg->search[cfg->search_index] ? 
-			  cfg->search[cfg->search_index] : cfg->search[0])
-	   : "none given",
+	   cfg->search && svz_array_size (cfg->search) > 0 ? 
+	   (unsigned long) cfg->search_index < svz_array_size (cfg->search) ? 
+	   (char *) svz_array_get (cfg->search, cfg->search_index) :
+	   (char *) svz_array_get (cfg->search, 0) :
+	   "none given",
 	   ext ? ext : "no extensions",
-	   hash_size (cfg->route),
-	   hash_size (cfg->conn), cfg->connections,
-	   hash_size (cfg->packet),
+	   svz_hash_size (cfg->route),
+	   svz_hash_size (cfg->conn), cfg->connections,
+	   svz_hash_size (cfg->packet),
 	   cfg->errors,
-	   hash_size (cfg->net),
+	   svz_hash_size (cfg->net),
 	   cfg->size / 1024, cfg->files, cfg->nodes,
 	   cfg->db_size / 1024 / 1024, cfg->db_files,
 	   cfg->dnloads, cfg->max_dnloads,
 	   cfg->uploads, cfg->max_uploads,
-	   hash_size (cfg->query));
+	   svz_hash_size (cfg->query));
 
-  xfree (ext);
+  svz_free (ext);
   return info;
 }
 
@@ -901,9 +890,8 @@ nut_info_server (server_t *server)
  * Gnutella client info callback.
  */
 char *
-nut_info_client (void *nut_cfg, socket_t sock)
+nut_info_client (svz_server_t *server, svz_socket_t *sock)
 {
-  nut_config_t *cfg = nut_cfg;
   static char info[80 * 3];
   static char text[128];
   nut_transfer_t *transfer = sock->data;
@@ -969,7 +957,7 @@ nut_info_client (void *nut_cfg, socket_t sock)
  * push requests (download).
  */
 int
-nut_detect_connect (socket_t sock)
+nut_detect_connect (svz_socket_t *sock)
 {
   nut_config_t *cfg = sock->cfg;
   int len = strlen (NUT_OK);
@@ -979,12 +967,12 @@ nut_detect_connect (socket_t sock)
       !memcmp (sock->recv_buffer, NUT_OK, len))
     {
       sock->userflags |= (NUT_FLAG_CLIENT | NUT_FLAG_SELF);
-      log_printf (LOG_NOTICE, "nut: host %s:%u connected\n",
-		  util_inet_ntoa (sock->remote_addr),
-		  ntohs (sock->remote_port));
-      sock_reduce_recv (sock, len);
+      svz_log (LOG_NOTICE, "nut: host %s:%u connected\n",
+	       svz_inet_ntoa (sock->remote_addr),
+	       ntohs (sock->remote_port));
+      svz_sock_reduce_recv (sock, len);
 
-      if (nut_connect_socket (cfg, sock) == -1) 
+      if (nut_connect_socket (svz_server_find (cfg), sock) == -1) 
 	return -1;
       return sock->check_request (sock);
     }
@@ -996,9 +984,9 @@ nut_detect_connect (socket_t sock)
  * Incoming connections will be protocol checked.
  */
 int 
-nut_detect_proto (void *nut_cfg, socket_t sock)
+nut_detect_proto (svz_server_t *server, svz_socket_t *sock)
 {
-  nut_config_t *cfg = nut_cfg;
+  nut_config_t *cfg = server->cfg;
   int len = strlen (NUT_CONNECT);
 
   /* detect normal connect */
@@ -1007,8 +995,8 @@ nut_detect_proto (void *nut_cfg, socket_t sock)
       !memcmp (sock->recv_buffer, NUT_CONNECT, len))
     {
       sock->userflags |= NUT_FLAG_CLIENT;
-      log_printf (LOG_NOTICE, "gnutella protocol detected (client)\n");
-      sock_reduce_recv (sock, len);
+      svz_log (LOG_NOTICE, "gnutella protocol detected (client)\n");
+      svz_sock_reduce_recv (sock, len);
       return -1;
     }
 
@@ -1018,7 +1006,7 @@ nut_detect_proto (void *nut_cfg, socket_t sock)
       !memcmp (sock->recv_buffer, NUT_GET, len))
     {
       sock->userflags |= NUT_FLAG_UPLOAD;
-      log_printf (LOG_NOTICE, "gnutella protocol detected (upload)\n");
+      svz_log (LOG_NOTICE, "gnutella protocol detected (upload)\n");
       return -1;
     }
 
@@ -1028,8 +1016,8 @@ nut_detect_proto (void *nut_cfg, socket_t sock)
       !memcmp (sock->recv_buffer, cfg->net_detect, len))
     {
       sock->userflags |= NUT_FLAG_HOSTS;
-      log_printf (LOG_NOTICE, "gnutella protocol detected (host list)\n");
-      sock_reduce_recv (sock, len);
+      svz_log (LOG_NOTICE, "gnutella protocol detected (host list)\n");
+      svz_sock_reduce_recv (sock, len);
       return -1;
     }
 
@@ -1039,7 +1027,7 @@ nut_detect_proto (void *nut_cfg, socket_t sock)
       !memcmp (sock->recv_buffer, NUT_GIVE, len))
     {
       sock->userflags |= NUT_FLAG_GIVEN;
-      log_printf (LOG_NOTICE, "gnutella protocol detected (giving)\n");
+      svz_log (LOG_NOTICE, "gnutella protocol detected (giving)\n");
       return -1;
     }
 
@@ -1051,9 +1039,9 @@ nut_detect_proto (void *nut_cfg, socket_t sock)
  * success.
  */
 int 
-nut_connect_socket (void *nut_cfg, socket_t sock)
+nut_connect_socket (svz_server_t *server, svz_socket_t *sock)
 {
-  nut_config_t *cfg = nut_cfg;
+  nut_config_t *cfg = server->cfg;
 
   /* assign download callbacks */
   if (sock->userflags & NUT_FLAG_GIVEN)
@@ -1067,7 +1055,7 @@ nut_connect_socket (void *nut_cfg, socket_t sock)
     {
       sock->check_request = nut_hosts_check;
       sock->write_socket = nut_hosts_write;
-      sock_resize_buffers (sock, NUT_SEND_BUFSIZE, sock->recv_buffer_size);
+      svz_sock_resize_buffers (sock, NUT_SEND_BUFSIZE, sock->recv_buffer_size);
       return 0;
     }
 
@@ -1086,12 +1074,12 @@ nut_connect_socket (void *nut_cfg, socket_t sock)
   if (sock->userflags & NUT_FLAG_CLIENT)
     {
       /* check if we got enough clients already */
-      if (hash_size (cfg->conn) > cfg->connections)
+      if (svz_hash_size (cfg->conn) > cfg->connections)
 	return -1;
 
       /* send the first reply if necessary */
       if (!(sock->userflags & NUT_FLAG_SELF))
-	if (sock_printf (sock, NUT_OK) == -1)
+	if (svz_sock_printf (sock, NUT_OK) == -1)
 	  return -1;
 
       /* assign gnutella specific callbacks */
@@ -1107,8 +1095,9 @@ nut_connect_socket (void *nut_cfg, socket_t sock)
 	return -1;
 
       /* put this client to the current connection hash */
-      hash_put (cfg->conn, 
-		nut_client_key (sock->remote_addr, sock->remote_port), sock);
+      svz_hash_put (cfg->conn, 
+		    nut_client_key (sock->remote_addr, sock->remote_port), 
+		    sock);
 
       return 0;
     }
