@@ -18,25 +18,22 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: nut-request.h,v 1.1 2000/10/05 18:01:46 ela Exp $
+ * $Id: nut-request.h,v 1.3 2001/05/19 23:04:58 ela Exp $
  *
  */
 
 #ifndef __NUT_REQUEST_H__
-#define __NUT_REQUEST_H__
+#define __NUT_REQUEST_H__ 1
 
 #if HAVE_CONFIG_H
 # include <config.h>
 #endif
 
-#define _GNU_SOURCE
-#include "socket.h"
-
 /* Exported functions. */
-int nut_reply (socket_t sock, nut_header_t *hdr, byte *packet);
-int nut_push_request (socket_t sock, nut_header_t *hdr, byte *packet);
-int nut_query (socket_t sock, nut_header_t *hdr, byte *packet);
-int nut_pong (socket_t sock, nut_header_t *hdr, byte *packet);
-int nut_ping (socket_t sock, nut_header_t *hdr, byte *null);
+int nut_reply (svz_socket_t *sock, nut_header_t *hdr, byte *packet);
+int nut_push_request (svz_socket_t *sock, nut_header_t *hdr, byte *packet);
+int nut_query (svz_socket_t *sock, nut_header_t *hdr, byte *packet);
+int nut_pong (svz_socket_t *sock, nut_header_t *hdr, byte *packet);
+int nut_ping (svz_socket_t *sock, nut_header_t *hdr, byte *null);
 
-#endif /* __NUT_REQUEST_H__ */
+#endif /* not __NUT_REQUEST_H__ */
