@@ -1,7 +1,7 @@
 /*
  * nut-transfer.c - gnutella file transfer implementation
  *
- * Copyright (C) 2000 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2000, 2001 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: nut-transfer.c,v 1.26 2000/12/30 01:59:34 ela Exp $
+ * $Id: nut-transfer.c,v 1.37 2001/07/03 20:02:42 ela Exp $
  *
  */
 
@@ -68,7 +68,7 @@
 
 #ifdef __MINGW32__
 # include <windows.h>
-# include <winsock.h>
+# include <winsock2.h>
 # include <io.h>
 #endif
 
@@ -83,13 +83,7 @@
 # define closedir(dir) FindClose (dir)
 #endif
 
-#include "alloc.h"
-#include "util.h"
-#include "snprintf.h"
-#include "socket.h"
-#include "connect.h"
-#include "server.h"
-#include "server-core.h"
+#include "libserveez.h"
 #include "gnutella.h"
 #include "nut-core.h"
 #include "nut-transfer.h"
@@ -106,10 +100,10 @@ nut_string_regex (char *text, char *regex)
   /* first check if text tokens are in text */
   if (!strchr (regex, '*') && !strchr (regex, '?'))
     {
-      str = xstrdup (text);
-      reg = xstrdup (regex);
-      util_tolower (str);
-      util_tolower (reg);
+      str = svz_strdup (text);
+      reg = svz_strdup (regex);
+      svz_tolower (str);
+      svz_tolower (reg);
 
       /* all tokens must be in the text */
       for (token = strtok (reg, " "); token; token = strtok (NULL, " "))
@@ -117,8 +111,8 @@ nut_string_regex (char *text, char *regex)
 	  if (!strstr (str, token))
 	    break;
 	}
-      xfree (str);
-      xfree (reg);
+      svz_free (str);
+      svz_free (reg);
       if (!token)
 	return -1;
       return 0;
@@ -178,9 +172,8 @@ nut_string_regex (char *text, char *regex)
  * Within this callback the actual file transfer is done.
  */
 static int
-nut_save_transfer (socket_t sock)
+nut_save_transfer (svz_socket_t *sock)
 {
-  nut_config_t *cfg = sock->cfg;
   int fill = sock->recv_buffer_fill;
   nut_transfer_t *transfer = sock->data;
   int num_written;
@@ -194,18 +187,18 @@ nut_save_transfer (socket_t sock)
       /* seems like an error occurred */
       if (num_written < 0)
 	{
-	  log_printf (LOG_ERROR, "nut: write: %s\n", SYS_ERROR);
+	  svz_log (LOG_ERROR, "nut: write: %s\n", SYS_ERROR);
 	  return -1;
 	}
       
       /* crop written data from receive buffer */
-      sock_reduce_recv (sock, num_written);
+      svz_sock_reduce_recv (sock, num_written);
 
       /* did we get all data */
       if ((transfer->size -= num_written) <= 0)
 	{
 #if ENABLE_DEBUG
-	  log_printf (LOG_DEBUG, "nut: file successfully received\n");
+	  svz_log (LOG_DEBUG, "nut: file successfully received\n");
 #endif
 	  /* yes, shutdown the connection */
 	  return -1;
@@ -219,9 +212,8 @@ nut_save_transfer (socket_t sock)
  * Whenever there is data within the receive queue it will be called.
  */
 static int
-nut_check_transfer (socket_t sock)
+nut_check_transfer (svz_socket_t *sock)
 {
-  nut_config_t *cfg = sock->cfg;
   int fill = sock->recv_buffer_fill;
   int len = strlen (NUT_GET_OK);
   char *p = sock->recv_buffer, *length;
@@ -239,7 +231,7 @@ nut_check_transfer (socket_t sock)
       if (p < sock->recv_buffer + (fill - 3) && !memcmp (p, NUT_SEPERATOR, 4))
 	{
 #if ENABLE_DEBUG
-	  log_printf (LOG_DEBUG, "nut: download header received\n");
+	  svz_log (LOG_DEBUG, "nut: download header received\n");
 #endif
 
 	  len = p - sock->recv_buffer + 1;
@@ -247,7 +239,7 @@ nut_check_transfer (socket_t sock)
 	  if (length == NULL)
 	    {
 #if ENABLE_DEBUG
-	      log_printf (LOG_DEBUG, "nut: no content length given\n");
+	      svz_log (LOG_DEBUG, "nut: no content length given\n");
 #endif
 	      return -1;
 	    }
@@ -257,13 +249,13 @@ nut_check_transfer (socket_t sock)
 	   * corresponds to the content length of this HTTP header
 	   */
 	  sock->userflags |= NUT_FLAG_HDR;
-	  transfer->size = util_atoi (length);
-	  xfree (length);
+	  transfer->size = svz_atoi (length);
+	  svz_free (length);
 	  if (transfer->original_size != transfer->size)
 	    {
-	      log_printf (LOG_WARNING,
-			  "nut: transfer sizes differ (%u!=%u)\n",
-			  transfer->original_size, transfer->size);
+	      svz_log (LOG_WARNING,
+		       "nut: transfer sizes differ (%u!=%u)\n",
+		       transfer->original_size, transfer->size);
 	    }
 
 	  /* assign the appropriate gnutella transfer callbacks */
@@ -273,7 +265,7 @@ nut_check_transfer (socket_t sock)
 
 	  /* crop header from receive buffer */
 	  len = (p - sock->recv_buffer) + 4;
-	  sock_reduce_recv (sock, len);
+	  svz_sock_reduce_recv (sock, len);
 	}
     }
 
@@ -285,7 +277,7 @@ nut_check_transfer (socket_t sock)
  * or successfully exited.
  */
 static int
-nut_disconnect_transfer (socket_t sock)
+nut_disconnect_transfer (svz_socket_t *sock)
 {
   nut_config_t *cfg = sock->cfg;
   nut_transfer_t *transfer = sock->data;
@@ -295,7 +287,7 @@ nut_disconnect_transfer (socket_t sock)
 
   /* finally close the received file */
   if (close (sock->file_desc) == -1)
-    log_printf (LOG_ERROR, "nut: close: %s\n", SYS_ERROR);
+    svz_log (LOG_ERROR, "nut: close: %s\n", SYS_ERROR);
 
   /* free the transfer data */
   if (transfer)
@@ -304,11 +296,11 @@ nut_disconnect_transfer (socket_t sock)
       if (transfer->size > 0 || !(sock->userflags & NUT_FLAG_HDR))
 	{
 #if ENABLE_DEBUG
-	  log_printf (LOG_DEBUG, "nut: downloading `%s' aborted\n",
-		      transfer->file);
+	  svz_log (LOG_DEBUG, "nut: downloading `%s' aborted\n",
+		   transfer->file);
 #endif
 	  if (unlink (transfer->file) == -1)
-	    log_printf (LOG_ERROR, "nut: unlink: %s\n", SYS_ERROR);
+	    svz_log (LOG_ERROR, "nut: unlink: %s\n", SYS_ERROR);
 	}
       
       /* 
@@ -321,8 +313,8 @@ nut_disconnect_transfer (socket_t sock)
 	  nut_send_push (sock->cfg, sock->data);
 	}
 
-      xfree (transfer->file);
-      xfree (transfer);
+      svz_free (transfer->file);
+      svz_free (transfer);
       sock->data = NULL;
     }
 
@@ -334,13 +326,12 @@ nut_disconnect_transfer (socket_t sock)
  * get a certain file.
  */
 int
-nut_init_transfer (socket_t sock, nut_reply_t *reply, 
+nut_init_transfer (svz_socket_t *sock, nut_reply_t *reply, 
 		   nut_record_t *record, char *savefile)
 {
   nut_config_t *cfg = sock->cfg;
-  nut_client_t *client = sock->data;
-  socket_t xsock;
-  char *file;
+  svz_socket_t *xsock;
+  char *ext, *file, *pattern;
   struct stat buf;
   int fd;
   nut_transfer_t *transfer;
@@ -350,69 +341,65 @@ nut_init_transfer (socket_t sock, nut_reply_t *reply,
   if (cfg->extensions)
     {
       /* go through all file extensions */
-      while (cfg->extensions[n])
+      svz_array_foreach (cfg->extensions, ext, n)
 	{
-	  if (strlen (savefile) > strlen (cfg->extensions[n]))
+	  if (strlen (savefile) > strlen (ext))
 	    {
-	      pos = strlen (savefile) - strlen (cfg->extensions[n]);
-	      if (pos < 0 ||
-		  !util_strcasecmp (&savefile[pos], cfg->extensions[n]))
+	      pos = strlen (savefile) - strlen (ext);
+	      if (pos < 0 || !svz_strcasecmp (&savefile[pos], ext))
 		break;
 	    }
-	  n++;
 	}
       /* did the above code "break" ? */
-      if (!cfg->extensions[n])
+      if ((unsigned long) n >= svz_array_size (cfg->extensions))
 	{
-	  log_printf (LOG_WARNING, "nut: not a valid extension: %s\n",
-		      savefile);
+	  svz_log (LOG_WARNING, "nut: not a valid extension: %s\n",
+		   savefile);
 	  return -1;
 	}
     }
 
   /* first check if the requested file is not already created */
-  file = xmalloc (strlen (cfg->save_path) + strlen (savefile) + 2);
+  file = svz_malloc (strlen (cfg->save_path) + strlen (savefile) + 2);
   sprintf (file, "%s/%s", cfg->save_path, savefile);
   if (stat (file, &buf) != -1)
     {
-      log_printf (LOG_NOTICE, "nut: %s already exists\n", savefile);
-      xfree (file);
+      svz_log (LOG_NOTICE, "nut: %s already exists\n", savefile);
+      svz_free (file);
       return -1;
     }
 
   /* second check if the file matches the original search patterns */
-  if (cfg->search)
+  svz_array_foreach (cfg->search, pattern, n)
     {
-      for (n = 0; cfg->search[n]; n++)
-	{
-	  if (nut_string_regex (savefile, cfg->search[n]))
-	    break;
-	}
-      if (!cfg->search[n])
-	{
-	  log_printf (LOG_NOTICE, "nut: no search pattern for %s\n",
-		      savefile);
-	  xfree (file);
-	  return -1;
-	}
+      if (nut_string_regex (savefile, pattern))
+	break;
+    }
+  if ((unsigned long) n >= svz_array_size (cfg->search))
+    {
+      svz_log (LOG_NOTICE, "nut: no search pattern for %s\n",
+	       savefile);
+      svz_free (file);
+      return -1;
     }
 
   /* try creating local file */
   if ((fd = open (file, O_RDWR | O_CREAT | O_BINARY, 0644)) == -1)
     {
-      log_printf (LOG_ERROR, "nut: open: %s\n", SYS_ERROR);
-      xfree (file);
+      svz_log (LOG_ERROR, "nut: open: %s\n", SYS_ERROR);
+      svz_free (file);
       return -1;
     }
 
   /* try to connect to the host */
-  if ((xsock = sock_connect (reply->ip, reply->port)) != NULL)
+  if ((xsock = svz_tcp_connect (reply->ip, reply->port)) != NULL)
     {
-      log_printf (LOG_NOTICE, "nut: connecting %s:%u\n",
-		  util_inet_ntoa (reply->ip), ntohs (reply->port));
+      svz_log (LOG_NOTICE, "nut: connecting %s:%u\n",
+	       svz_inet_ntoa (reply->ip), ntohs (reply->port));
       cfg->dnloads++;
       xsock->cfg = cfg;
       xsock->flags |= SOCK_FLAG_NOFLOOD;
+      svz_sock_setparent (xsock, svz_sock_getparent (sock));
       xsock->disconnected_socket = nut_disconnect_transfer;
       xsock->check_request = nut_check_transfer;
       xsock->userflags = NUT_FLAG_DNLOAD;
@@ -421,10 +408,10 @@ nut_init_transfer (socket_t sock, nut_reply_t *reply,
       xsock->idle_counter = NUT_CONNECT_TIMEOUT;
 
       /* initialize transfer data */
-      transfer = xmalloc (sizeof (nut_transfer_t));
+      transfer = svz_malloc (sizeof (nut_transfer_t));
       memset (transfer, 0, sizeof (nut_transfer_t));
       transfer->original_size = record->size;
-      transfer->file = xstrdup (file);
+      transfer->file = svz_strdup (file);
       transfer->start = time (NULL);
       xsock->data = transfer;
 
@@ -435,17 +422,17 @@ nut_init_transfer (socket_t sock, nut_reply_t *reply,
       transfer->version = sock->version;
 
       /* send HTTP request to the listening gnutella host */
-      sock_printf (xsock, NUT_GET "%d/%s " NUT_HTTP "1.0\r\n",
-		   record->index, savefile);
-      sock_printf (xsock, NUT_AGENT);
-      sock_printf (xsock, NUT_RANGE ": bytes=0-\r\n");
-      sock_printf (xsock, "\r\n");
-      xfree (file);
+      svz_sock_printf (xsock, NUT_GET "%d/%s " NUT_HTTP "1.0\r\n",
+		       record->index, savefile);
+      svz_sock_printf (xsock, NUT_AGENT);
+      svz_sock_printf (xsock, NUT_RANGE ": bytes=0-\r\n");
+      svz_sock_printf (xsock, "\r\n");
+      svz_free (file);
       return 0;
     }
 
   close (fd);
-  xfree (file);
+  svz_free (file);
   return 0;
 }
 
@@ -453,7 +440,7 @@ nut_init_transfer (socket_t sock, nut_reply_t *reply,
  * This is the check_request callback for given files.
  */
 int
-nut_check_given (socket_t sock)
+nut_check_given (svz_socket_t *sock)
 {
   nut_config_t *cfg = sock->cfg;
   int fill = sock->recv_buffer_fill;
@@ -477,22 +464,22 @@ nut_check_given (socket_t sock)
 	p++;
       if (p >= sock->recv_buffer + fill || *p != '/')
 	{
-	 log_printf (LOG_ERROR, "nut: invalid GIV line\n");
+	  svz_log (LOG_ERROR, "nut: invalid GIV line\n");
 	  return -1;
 	}
 
       /* get original push request */
       *p = '\0';
-      transfer = (nut_transfer_t *) hash_get (cfg->push, pushkey);
+      transfer = (nut_transfer_t *) svz_hash_get (cfg->push, pushkey);
       if (transfer == NULL)
 	{
-	  log_printf (LOG_ERROR, "nut: no such push request sent\n");
+	  svz_log (LOG_ERROR, "nut: no such push request sent\n");
 	  return -1;
 	}
 
       /* delete key and data from push request hash */
-      hash_delete (cfg->push, pushkey);
-      sock_reduce_recv (sock, len);
+      svz_hash_delete (cfg->push, pushkey);
+      svz_sock_reduce_recv (sock, len);
 
       /* assign all necessary callbacks for downloading the file */
       sock->data = transfer;
@@ -508,14 +495,14 @@ nut_check_given (socket_t sock)
       file = transfer->file;
       if (stat (file, &buf) != -1)
 	{
-	  log_printf (LOG_NOTICE, "nut: %s already exists\n", file);
+	  svz_log (LOG_NOTICE, "nut: %s already exists\n", file);
 	  return -1;
 	}
       
       /* try creating local file */
       if ((fd = open (file, O_RDWR | O_CREAT | O_BINARY, 0644)) == -1)
 	{
-	  log_printf (LOG_ERROR, "nut: open: %s\n", SYS_ERROR);
+	  svz_log (LOG_ERROR, "nut: open: %s\n", SYS_ERROR);
 	  return -1;
 	}
 
@@ -528,11 +515,11 @@ nut_check_given (socket_t sock)
 	file++;
 
       /* send HTTP request to the listening gnutella host */
-      sock_printf (sock, NUT_GET "%d/%s " NUT_HTTP "1.0\r\n",
-		   transfer->index, file);
-      sock_printf (sock, NUT_AGENT);
-      sock_printf (sock, NUT_RANGE ": bytes=0-\r\n");
-      sock_printf (sock, "\r\n");
+      svz_sock_printf (sock, NUT_GET "%d/%s " NUT_HTTP "1.0\r\n",
+		       transfer->index, file);
+      svz_sock_printf (sock, NUT_AGENT);
+      svz_sock_printf (sock, NUT_RANGE ": bytes=0-\r\n");
+      svz_sock_printf (sock, "\r\n");
     }
   return 0;
 }
@@ -547,15 +534,17 @@ nut_check_given (socket_t sock)
 int
 nut_send_push (nut_config_t *cfg, nut_transfer_t *transfer)
 {
-  socket_t sock;
+  svz_socket_t *sock;
   nut_header_t hdr;
   nut_push_t push;
   nut_packet_t *pkt;
   nut_transfer_t *trans;
   char *pushkey;
+  struct sockaddr_in *addr = NULL;
+  svz_portcfg_t *port;
 
   /* find original socket connection */
-  if ((sock = sock_find (transfer->id, transfer->version)) != NULL)
+  if ((sock = svz_sock_find (transfer->id, transfer->version)) != NULL)
     {
       /* create new gnutella header */
       nut_calc_guid (hdr.id);
@@ -567,51 +556,54 @@ nut_send_push (nut_config_t *cfg, nut_transfer_t *transfer)
       /* create push request */
       memcpy (push.id, transfer->guid, NUT_GUID_SIZE);
       push.index = transfer->index;
-      push.ip = cfg->ip ? cfg->ip : sock->local_addr;
-      push.port = (unsigned short) (cfg->port ? cfg->port : 
-				    htons (cfg->netport->port));
+      if ((port = svz_sock_portcfg (sock)) != NULL)
+	addr = svz_portcfg_addr (port);
+      push.ip = cfg->ip ? cfg->ip : addr ? 
+	addr->sin_addr.s_addr : sock->local_addr;
+      push.port = (unsigned short) (cfg->port ? cfg->port : addr ?
+				    addr->sin_port : sock->local_port);
       
       /* create push request key and check if it was already sent */
-      pushkey = xmalloc (16 + NUT_GUID_SIZE * 2);
+      pushkey = svz_malloc (16 + NUT_GUID_SIZE * 2);
       sprintf (pushkey, "%d:%s", push.index, nut_text_guid (push.id));
-      if ((trans = hash_get (cfg->push, pushkey)) != NULL)
+      if ((trans = svz_hash_get (cfg->push, pushkey)) != NULL)
 	{
 #if ENABLE_DEBUG
-	  log_printf (LOG_DEBUG, "nut: push request already sent\n");
+	  svz_log (LOG_DEBUG, "nut: push request already sent\n");
 #endif
-	  xfree (pushkey);
+	  svz_free (pushkey);
 	  return -1;
 	}
 
       /* try sending header and push request */
-      if (sock_write (sock, (char *) nut_put_header (&hdr), 
-		      SIZEOF_NUT_HEADER) == -1 ||
-	  sock_write (sock, (char *) nut_put_push (&push), 
-		      SIZEOF_NUT_PUSH) == -1)
+      if (svz_sock_write (sock, (char *) nut_put_header (&hdr), 
+			  SIZEOF_NUT_HEADER) == -1 ||
+	  svz_sock_write (sock, (char *) nut_put_push (&push), 
+			  SIZEOF_NUT_PUSH) == -1)
 	{
-	  sock_schedule_for_shutdown (sock);
-	  xfree (pushkey);
+	  svz_sock_schedule_for_shutdown (sock);
+	  svz_free (pushkey);
 	  return -1;
 	}
 
       /* put push request into hash for later reply detection */
-      trans = xmalloc (sizeof (nut_transfer_t));
+      trans = svz_malloc (sizeof (nut_transfer_t));
       memcpy (trans, transfer, sizeof (nut_transfer_t));
-      trans->file = xstrdup (transfer->file);
-      hash_put (cfg->push, pushkey, trans);
-      xfree (pushkey);
+      trans->file = svz_strdup (transfer->file);
+      svz_hash_put (cfg->push, pushkey, trans);
+      svz_free (pushkey);
 
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "nut: sent push request to %s:%u\n",
-		  util_inet_ntoa (sock->remote_addr), 
+      svz_log (LOG_DEBUG, "nut: sent push request to %s:%u\n",
+		  svz_inet_ntoa (sock->remote_addr), 
 		  ntohs (sock->remote_port));
 #endif
 
       /* put into sent packet hash */
-      pkt = xmalloc (sizeof (nut_packet_t));
+      pkt = svz_malloc (sizeof (nut_packet_t));
       pkt->sock = sock;
       pkt->sent = time (NULL);
-      hash_put (cfg->packet, (char *) hdr.id, pkt);
+      svz_hash_put (cfg->packet, (char *) hdr.id, pkt);
     }
   return 0;
 }
@@ -627,9 +619,9 @@ nut_destroy_database (nut_config_t *cfg)
   while ((entry = cfg->database) != NULL)
     {
       cfg->database = entry->next;
-      xfree (entry->file);
-      xfree (entry->path);
-      xfree (entry);
+      svz_free (entry->file);
+      svz_free (entry->path);
+      svz_free (entry);
     }
   cfg->db_files = 0;
   cfg->db_size = 0;
@@ -643,9 +635,9 @@ nut_add_database (nut_config_t *cfg, char *path, char *file, off_t size)
 {
   nut_file_t *entry;
 
-  entry = xmalloc (sizeof (nut_file_t));
-  entry->file = xstrdup (file);
-  entry->path = xstrdup (path);
+  entry = svz_malloc (sizeof (nut_file_t));
+  entry->file = svz_strdup (file);
+  entry->path = svz_strdup (path);
   entry->size = size;
   entry->index = cfg->db_files;
   entry->next = cfg->database;
@@ -726,7 +718,7 @@ nut_read_database_r (nut_config_t *cfg, char *dirname, int depth)
 
       /* open the directory */
 #ifdef __MINGW32__
-      if (snprintf (filename, NUT_PATH_SIZE - 1, "%s/*", dirname) == -1)
+      if (svz_snprintf (filename, NUT_PATH_SIZE - 1, "%s/*", dirname) == -1)
 	return;
       
       if ((dir = FindFirstFile (filename, &de)) != INVALID_HANDLE_VALUE)
@@ -741,8 +733,8 @@ nut_read_database_r (nut_config_t *cfg, char *dirname, int depth)
 	  do
 #endif
 	    {
-	      if (snprintf (filename, NUT_PATH_SIZE - 1,
-			    "%s/%s", dirname, FILENAME) == -1)
+	      if (svz_snprintf (filename, NUT_PATH_SIZE - 1,
+				"%s/%s", dirname, FILENAME) == -1)
 		continue;
 
 	      /* stat the given file */
@@ -756,9 +748,9 @@ nut_read_database_r (nut_config_t *cfg, char *dirname, int depth)
 		  /* recurse into directories */
 		  else if (S_ISDIR (buf.st_mode) && FILENAME[0] != '.')
 		    {
-		      path = xstrdup (filename);
+		      path = svz_strdup (filename);
 		      nut_read_database_r (cfg, path, depth);
-		      xfree (path);
+		      svz_free (path);
 		    }
 		}
 	    }
@@ -775,7 +767,7 @@ nut_read_database_r (nut_config_t *cfg, char *dirname, int depth)
  * requests.
  */
 int
-nut_check_upload (socket_t sock)
+nut_check_upload (svz_socket_t *sock)
 {
   char *p = sock->recv_buffer;
   int len, fill = strlen (NUT_GET);
@@ -799,7 +791,7 @@ nut_check_upload (socket_t sock)
   if (p < sock->recv_buffer + (fill - 3) && !memcmp (p, NUT_SEPERATOR, 4))
     {
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "nut: upload header received\n");
+      svz_log (LOG_DEBUG, "nut: upload header received\n");
 #endif
       /* parse first (GET) line */
       len = p - sock->recv_buffer + 1;
@@ -828,7 +820,7 @@ nut_check_upload (socket_t sock)
 	return -1;
       /* parsed file itself */
       fill = p - f;
-      file = xmalloc (fill + 1);
+      file = svz_malloc (fill + 1);
       memcpy (file, f, fill);
       file[fill] = '\0';
 
@@ -838,14 +830,14 @@ nut_check_upload (socket_t sock)
       if ((entry = nut_get_database (sock->cfg, file, index)) == NULL)
 	{
 #if ENABLE_DEBUG
-	  log_printf (LOG_DEBUG, "nut: no such file: %s, %u\n", file, index);
+	  svz_log (LOG_DEBUG, "nut: no such file: %s, %u\n", file, index);
 #endif
-	  xfree (file);
+	  svz_free (file);
 	  return -1;
 	}
       len = end - sock->recv_buffer + 3;
-      sock_reduce_recv (sock, len);
-      xfree (file);
+      svz_sock_reduce_recv (sock, len);
+      svz_free (file);
 
       /* disable connection timeout */
       sock->idle_func = NULL;
@@ -863,7 +855,7 @@ nut_check_upload (socket_t sock)
  * initializes the file upload.
  */
 int
-nut_init_upload (socket_t sock, nut_file_t *entry)
+nut_init_upload (svz_socket_t *sock, nut_file_t *entry)
 {
   char *file;
   nut_config_t *cfg = sock->cfg;
@@ -872,29 +864,29 @@ nut_init_upload (socket_t sock, nut_file_t *entry)
   nut_transfer_t *transfer;
 
   /* create filename */
-  file = xmalloc (strlen (entry->path) + strlen (entry->file) + 2);
+  file = svz_malloc (strlen (entry->path) + strlen (entry->file) + 2);
   sprintf (file, "%s/%s", entry->path, entry->file);
   
   /* check file */
   if (stat (file, &buf) == -1 || !S_ISREG (buf.st_mode) || buf.st_size <= 0)
     {
-      log_printf (LOG_ERROR, "nut: invalid file: %s %s\n", file);
-      xfree (file);
+      svz_log (LOG_ERROR, "nut: invalid file: %s %s\n", file);
+      svz_free (file);
       return -1;
     }
   
   /* open the file for reading */
   if ((fd = open (file, O_RDONLY | O_BINARY)) == -1)
     {
-      log_printf (LOG_ERROR, "nut: open: %s\n", SYS_ERROR);
-      xfree (file);
+      svz_log (LOG_ERROR, "nut: open: %s\n", SYS_ERROR);
+      svz_free (file);
       return -1;
     }
 
-  sock_printf (sock, NUT_GET_OK  NUT_AGENT);
-  sock_printf (sock, NUT_CONTENT ": application/binary\r\n");
-  sock_printf (sock, NUT_LENGTH ": %d\r\n", buf.st_size);
-  sock_printf (sock, "\r\n");
+  svz_sock_printf (sock, NUT_GET_OK  NUT_AGENT);
+  svz_sock_printf (sock, NUT_CONTENT ": application/binary\r\n");
+  svz_sock_printf (sock, NUT_LENGTH ": %d\r\n", buf.st_size);
+  svz_sock_printf (sock, "\r\n");
 
   sock->file_desc = fd;
   sock->read_socket = nut_file_read;
@@ -903,15 +895,15 @@ nut_init_upload (socket_t sock, nut_file_t *entry)
   sock->flags |= SOCK_FLAG_FILE;
   cfg->uploads++;
 
-  transfer = xmalloc (sizeof (nut_transfer_t));
+  transfer = svz_malloc (sizeof (nut_transfer_t));
   memset (transfer, 0, sizeof (nut_transfer_t));
   transfer->size = buf.st_size;
   transfer->original_size = buf.st_size;
-  transfer->file = xstrdup (file);
+  transfer->file = svz_strdup (file);
   transfer->start = time (NULL);
   sock->data = transfer;
 
-  xfree (file);
+  svz_free (file);
   return 0;
 }
 
@@ -919,7 +911,7 @@ nut_init_upload (socket_t sock, nut_file_t *entry)
  * Disconnection callback for gnutella uploads.
  */
 int
-nut_disconnect_upload (socket_t sock)
+nut_disconnect_upload (svz_socket_t *sock)
 {
   nut_config_t *cfg = sock->cfg;
   nut_transfer_t *transfer = sock->data;
@@ -929,13 +921,13 @@ nut_disconnect_upload (socket_t sock)
 
   /* finally close the received file */
   if (close (sock->file_desc) == -1)
-    log_printf (LOG_ERROR, "nut: close: %s\n", SYS_ERROR);
+    svz_log (LOG_ERROR, "nut: close: %s\n", SYS_ERROR);
 
   /* free the transfer data */
   if (transfer)
     {
-      xfree (transfer->file);
-      xfree (transfer);
+      svz_free (transfer->file);
+      svz_free (transfer);
       sock->data = NULL;
     }
 
@@ -947,7 +939,7 @@ nut_disconnect_upload (socket_t sock)
  * file uploads.
  */
 int
-nut_file_read (socket_t sock)
+nut_file_read (svz_socket_t *sock)
 {
   int num_read;
   int do_read;
@@ -973,7 +965,7 @@ nut_file_read (socket_t sock)
   /* Read error occurred. */
   if (num_read < 0)
     {
-      log_printf (LOG_ERROR, "nut: read: %s\n", SYS_ERROR);
+      svz_log (LOG_ERROR, "nut: read: %s\n", SYS_ERROR);
       return -1;
     }
 
@@ -991,13 +983,13 @@ nut_file_read (socket_t sock)
   if (transfer->size <= 0)
     {
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "nut: file successfully read\n");
+      svz_log (LOG_DEBUG, "nut: file successfully read\n");
 #endif
       /* 
        * no further read()s from the file descriptor, signaling 
        * the writers there will not be additional data from now on
        */
-      sock->read_socket = default_read;
+      sock->read_socket = svz_tcp_read_socket;
       sock->flags &= ~SOCK_FLAG_FILE;
     }
 
@@ -1009,7 +1001,7 @@ nut_file_read (socket_t sock)
  * throttles its network output to a configured value.
  */
 int
-nut_file_write (socket_t sock)
+nut_file_write (svz_socket_t *sock)
 {
   int num_written, do_write;
   nut_transfer_t *transfer = sock->data;
@@ -1050,8 +1042,8 @@ nut_file_write (socket_t sock)
   /* write error occurred */
   else if (num_written < 0)
     {
-      log_printf (LOG_ERROR, "nut: send: %s\n", NET_ERROR);
-      if (last_errno == SOCK_UNAVAILABLE)
+      svz_log (LOG_ERROR, "nut: send: %s\n", NET_ERROR);
+      if (svz_errno == SOCK_UNAVAILABLE)
         {
           sock->unavailable = t + RELAX_FD_TIME;
           num_written = 0;
@@ -1061,7 +1053,7 @@ nut_file_write (socket_t sock)
   if (sock->send_buffer_fill == 0 && transfer->size <= 0)
     {
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "nut: file successfully sent\n");
+      svz_log (LOG_DEBUG, "nut: file successfully sent\n");
 #endif
       return -1;
     }
