@@ -1,7 +1,7 @@
 /*
  * tunnel.c - port forward implementations
  *
- * Copyright (C) 2000 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2000, 2001 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: tunnel.c,v 1.13 2001/01/08 23:27:21 ela Exp $
+ * $Id: tunnel.c,v 1.26 2001/07/03 10:43:31 ela Exp $
  *
  */
 
@@ -36,7 +36,7 @@
 #include <time.h>
 
 #ifdef __MINGW32__
-# include <winsock.h>
+# include <winsock2.h>
 #endif
 
 #ifndef __MINGW32__
@@ -45,16 +45,7 @@
 # include <arpa/inet.h>
 #endif
 
-#include "util.h"
-#include "hash.h"
-#include "alloc.h"
-#include "socket.h"
-#include "connect.h"
-#include "udp-socket.h"
-#include "icmp-socket.h"
-#include "pipe-socket.h"
-#include "server-core.h"
-#include "server.h"
+#include "libserveez.h"
 #include "tunnel.h"
 
 /*
@@ -70,17 +61,17 @@ tnl_config_t tnl_config =
 /*
  * Defining configuration file associations with key-value-pairs.
  */
-key_value_pair_t tnl_config_prototype [] = 
+svz_key_value_pair_t tnl_config_prototype [] = 
 {
-  REGISTER_PORTCFG ("source", tnl_config.source, NOTDEFAULTABLE),
-  REGISTER_PORTCFG ("target", tnl_config.target, NOTDEFAULTABLE),
-  REGISTER_END ()
+  SVZ_REGISTER_PORTCFG ("source", tnl_config.source, SVZ_ITEM_NOTDEFAULTABLE),
+  SVZ_REGISTER_PORTCFG ("target", tnl_config.target, SVZ_ITEM_NOTDEFAULTABLE),
+  SVZ_REGISTER_END ()
 };
 
 /*
  * Definition of this server.
  */
-server_definition_t tnl_server_definition =
+svz_servertype_t tnl_server_definition =
 {
   "tunnel server",
   "tunnel",
@@ -103,7 +94,7 @@ server_definition_t tnl_server_definition =
  * The tunnel server's global initializer.
  */
 int
-tnl_global_init (void)
+tnl_global_init (svz_servertype_t *server)
 {
   return 0;
 }
@@ -112,7 +103,7 @@ tnl_global_init (void)
  * The tunnel server's global finalizer.
  */
 int
-tnl_global_finalize (void)
+tnl_global_finalize (svz_servertype_t *server)
 {
   return 0;
 }
@@ -121,37 +112,39 @@ tnl_global_finalize (void)
  * Tunnel server instance initializer. Check the configuration.
  */
 int
-tnl_init (server_t *server)
+tnl_init (svz_server_t *server)
 {
   tnl_config_t *cfg = server->cfg;
+  struct sockaddr_in *addr;
 
   /* protocol supported ? */
   if (!(cfg->source->proto & (PROTO_TCP|PROTO_ICMP|PROTO_UDP|PROTO_PIPE)) ||
       !(cfg->target->proto & (PROTO_TCP|PROTO_ICMP|PROTO_UDP|PROTO_PIPE)))
     {
-      log_printf (LOG_ERROR, "tunnel: protocol not supported\n");
+      svz_log (LOG_ERROR, "tunnel: protocol not supported\n");
       return -1;
     }
 
   /* check identity of source and target port configurations */
-  if (server_portcfg_equal (cfg->source, cfg->target))
+  if (svz_portcfg_equal (cfg->source, cfg->target))
     {
-      log_printf (LOG_ERROR, "tunnel: source and target identical\n");
+      svz_log (LOG_ERROR, "tunnel: source and target identical\n");
       return -1;
     }
 
   if (!(cfg->target->proto & PROTO_PIPE))
     {
       /* broadcast target ip address not allowed */
-      if (cfg->target->addr->sin_addr.s_addr == INADDR_ANY)
+      addr = svz_portcfg_addr (cfg->target);
+      if (addr->sin_addr.s_addr == INADDR_ANY)
 	{
-	  log_printf (LOG_ERROR, "tunnel: broadcast target ip not allowed\n");
+	  svz_log (LOG_ERROR, "tunnel: broadcast target ip not allowed\n");
 	  return -1;
 	}
     }
 
   /* create source client hash (for UDP and ICMP only) */
-  cfg->client = hash_create (4);
+  cfg->client = svz_hash_create (4);
   
   /* assign the appropriate handle request routine of the server */
   if (cfg->source->proto & PROTO_UDP)
@@ -159,8 +152,6 @@ tnl_init (server_t *server)
   if (cfg->source->proto & PROTO_ICMP)
     server->handle_request = tnl_handle_request_icmp_source;
 
-  /* bind the source port */
-  server_bind (server, cfg->source);
   return 0;
 }
 
@@ -168,22 +159,22 @@ tnl_init (server_t *server)
  * The tunnel server instance finalizer.
  */
 int
-tnl_finalize (server_t *server)
+tnl_finalize (svz_server_t *server)
 {
   tnl_config_t *cfg = server->cfg;
   tnl_connect_t **source;
   int n;
 
   /* release source connection hash if necessary */
-  if ((source = (tnl_connect_t **) hash_values (cfg->client)) != NULL)
+  if ((source = (tnl_connect_t **) svz_hash_values (cfg->client)) != NULL)
     {
-      for (n = 0; n < hash_size (cfg->client); n++)
+      for (n = 0; n < svz_hash_size (cfg->client); n++)
 	{
-	  xfree (source[n]);
+	  svz_free (source[n]);
 	}
-      hash_xfree (source);
+      svz_hash_xfree (source);
     }
-  hash_destroy (cfg->client);
+  svz_hash_destroy (cfg->client);
 
   return 0;
 }
@@ -193,11 +184,11 @@ tnl_finalize (server_t *server)
  * are the remote ip address and port.
  */
 static char *
-tnl_addr (socket_t sock)
+tnl_addr (svz_socket_t *sock)
 {
   static char addr[24];
 
-  sprintf (addr, "%s:%u", util_inet_ntoa (sock->remote_addr), 
+  sprintf (addr, "%s:%u", svz_inet_ntoa (sock->remote_addr), 
 	   ntohs (sock->remote_port));
   return addr;
 }
@@ -206,11 +197,11 @@ tnl_addr (socket_t sock)
  * Release referring tunnel structure.
  */
 static void
-tnl_free_connect (socket_t sock)
+tnl_free_connect (svz_socket_t *sock)
 {
   if (sock->data)
     {
-      xfree (sock->data);
+      svz_free (sock->data);
       sock->data = NULL;
     }
 }
@@ -223,7 +214,7 @@ tnl_create_connect (void)
 {
   tnl_connect_t *source;
 
-  source = xmalloc (sizeof (tnl_connect_t));
+  source = svz_malloc (sizeof (tnl_connect_t));
   memset (source, 0, sizeof (tnl_connect_t));
   return source;
 }
@@ -233,19 +224,21 @@ tnl_create_connect (void)
  * tries to connect to the servers target configuration and delivers a
  * new socket structure or NULL if it failed.
  */ 
-static socket_t
-tnl_create_socket (socket_t sock, int source)
+static svz_socket_t *
+tnl_create_socket (svz_socket_t *sock, int source)
 {
   tnl_config_t *cfg = sock->cfg;
   unsigned long ip = 0;
   unsigned short port = 0;
-  socket_t xsock = NULL;
+  svz_socket_t *xsock = NULL;
+  struct sockaddr_in *addr;
 
   /* get host and target ip if necessary */
   if (!(cfg->target->proto & PROTO_PIPE))
     {
-      ip = cfg->target->addr->sin_addr.s_addr;
-      port = cfg->target->addr->sin_port;
+      addr = svz_portcfg_addr (cfg->target);
+      ip = addr->sin_addr.s_addr;
+      port = addr->sin_port;
     }
 
   /*
@@ -267,41 +260,41 @@ tnl_create_socket (socket_t sock, int source)
       sock->userflags |= TNL_FLAG_TGT_PIPE;
       break;
     default:
-      log_printf (LOG_ERROR, "tunnel: invalid target configuration\n");
+      svz_log (LOG_ERROR, "tunnel: invalid target configuration\n");
       return NULL;
     }
 
   /* target is a TCP connection */
   if (sock->userflags & TNL_FLAG_TGT_TCP)
     {
-      if ((xsock = sock_connect (ip, port)) == NULL)
+      if ((xsock = svz_tcp_connect (ip, port)) == NULL)
 	{
-	  log_printf (LOG_ERROR, "tunnel: tcp: cannot connect to %s:%u\n",
-		      util_inet_ntoa (ip), ntohs (port));
+	  svz_log (LOG_ERROR, "tunnel: tcp: cannot connect to %s:%u\n",
+		   svz_inet_ntoa (ip), ntohs (port));
 	  return NULL;
 	}
 
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "tunnel: tcp: connecting to %s:%u\n",
-		  util_inet_ntoa (ip), ntohs (port));
+      svz_log (LOG_DEBUG, "tunnel: tcp: connecting to %s:%u\n",
+	       svz_inet_ntoa (ip), ntohs (port));
 #endif /* ENABLE_DEBUG */
       xsock->check_request = tnl_check_request_tcp_target;
-      sock_resize_buffers (xsock, UDP_BUF_SIZE, UDP_BUF_SIZE);
+      svz_sock_resize_buffers (xsock, UDP_BUF_SIZE, UDP_BUF_SIZE);
     }
 
   /* target is an UDP connection */
   else if (sock->userflags & TNL_FLAG_TGT_UDP)
     {
-      if ((xsock = udp_connect (ip, port)) == NULL)
+      if ((xsock = svz_udp_connect (ip, port)) == NULL)
 	{
-	  log_printf (LOG_ERROR, "tunnel: udp: cannot connect to %s:%u\n",
-		      util_inet_ntoa (ip), ntohs (port));
+	  svz_log (LOG_ERROR, "tunnel: udp: cannot connect to %s:%u\n",
+		   svz_inet_ntoa (ip), ntohs (port));
 	  return NULL;
 	}
 
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "tunnel: udp: connecting to %s:%u\n",
-		  util_inet_ntoa (ip), ntohs (port));
+      svz_log (LOG_DEBUG, "tunnel: udp: connecting to %s:%u\n",
+	       svz_inet_ntoa (ip), ntohs (port));
 #endif /* ENABLE_DEBUG */
       xsock->handle_request = tnl_handle_request_udp_target;
       xsock->idle_func = tnl_idle;
@@ -311,16 +304,17 @@ tnl_create_socket (socket_t sock, int source)
   /* target is an ICMP connection */
   else if (sock->userflags & TNL_FLAG_TGT_ICMP)
     {
-      if ((xsock = icmp_connect (ip, port)) == NULL)
+      if ((xsock = svz_icmp_connect (ip, port, cfg->target->icmp_type)) 
+	  == NULL)
 	{
-	  log_printf (LOG_ERROR, "tunnel: icmp: cannot connect to %s\n",
-		      util_inet_ntoa (ip));
+	  svz_log (LOG_ERROR, "tunnel: icmp: cannot connect to %s\n",
+		   svz_inet_ntoa (ip));
 	  return NULL;
 	}
 
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "tunnel: icmp: connecting to %s\n",
-		  util_inet_ntoa (ip));
+      svz_log (LOG_DEBUG, "tunnel: icmp: connecting to %s\n",
+	       svz_inet_ntoa (ip));
 #endif /* ENABLE_DEBUG */
       xsock->handle_request = tnl_handle_request_icmp_target;
       xsock->idle_func = tnl_idle;
@@ -330,20 +324,20 @@ tnl_create_socket (socket_t sock, int source)
   /* target is a pipe connection */
   else if (sock->userflags & TNL_FLAG_TGT_PIPE)
     {
-      if ((xsock = pipe_connect (cfg->target->inpipe, 
-				 cfg->target->outpipe)) == NULL)
+      if ((xsock = svz_pipe_connect (&cfg->target->pipe_recv, 
+				     &cfg->target->pipe_send)) == NULL)
 	{
-	  log_printf (LOG_ERROR, "tunnel: pipe: cannot connect to %s\n",
-		      cfg->target->outpipe);
+	  svz_log (LOG_ERROR, "tunnel: pipe: cannot connect to %s\n",
+		   cfg->target->pipe_send.name);
 	  return NULL;
 	}
 
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "tunnel: pipe: connecting to %s\n",
-		  cfg->target->outpipe);
+      svz_log (LOG_DEBUG, "tunnel: pipe: connecting to %s\n",
+	       cfg->target->pipe_send.name);
 #endif /* ENABLE_DEBUG */
       xsock->check_request = tnl_check_request_pipe_target;
-      sock_resize_buffers (xsock, UDP_BUF_SIZE, UDP_BUF_SIZE);
+      svz_sock_resize_buffers (xsock, UDP_BUF_SIZE, UDP_BUF_SIZE);
     }
 
   xsock->cfg = cfg;
@@ -360,24 +354,24 @@ tnl_create_socket (socket_t sock, int source)
  * the targets socket and its own userflags.
  */
 static int
-tnl_send_request_source (socket_t sock, char *packet, int len, int flag)
+tnl_send_request_source (svz_socket_t *sock, char *packet, int len, int flag)
 {
   /* target is TCP or PIPE */
   if (flag & (TNL_FLAG_TGT_TCP | TNL_FLAG_TGT_PIPE))
     {
-      if (sock_write (sock, packet, len) == -1)
+      if (svz_sock_write (sock, packet, len) == -1)
 	return -1;
     }
   /* target is UDP */
   else if (flag & TNL_FLAG_TGT_UDP)
     {
-      if (udp_write (sock, packet, len) == -1)
+      if (svz_udp_write (sock, packet, len) == -1)
 	return -1;
     }
   /* target is ICMP */
   else if (flag & TNL_FLAG_TGT_ICMP)
     {
-      if (icmp_write (sock, packet, len) == -1)
+      if (svz_icmp_write (sock, packet, len) == -1)
 	return -1;
     }
 
@@ -390,24 +384,24 @@ tnl_send_request_source (socket_t sock, char *packet, int len, int flag)
  * the sources socket and its own userflags.
  */
 static int
-tnl_send_request_target (socket_t sock, char *packet, int len, int flag)
+tnl_send_request_target (svz_socket_t *sock, char *packet, int len, int flag)
 {
   /* source is TCP or PIPE */
   if (flag & (TNL_FLAG_SRC_TCP | TNL_FLAG_SRC_PIPE))
     {
-      if (sock_write (sock, packet, len) == -1)
+      if (svz_sock_write (sock, packet, len) == -1)
 	return -1;
     }
   /* source is UDP */
   else if (flag & TNL_FLAG_SRC_UDP)
     {
-      if (udp_write (sock, packet, len) == -1)
+      if (svz_udp_write (sock, packet, len) == -1)
 	return -1;
     }
   /* source is ICMP */
   else if (flag & TNL_FLAG_SRC_ICMP)
     {
-      if (icmp_write (sock, packet, len) == -1)
+      if (svz_icmp_write (sock, packet, len) == -1)
 	return -1;
     }
 
@@ -419,10 +413,10 @@ tnl_send_request_target (socket_t sock, char *packet, int len, int flag)
  * share the port configuration with other TCP or pipe servers.
  */
 int
-tnl_detect_proto (void *cfg, socket_t sock)
+tnl_detect_proto (svz_server_t *server, svz_socket_t *sock)
 {
-  log_printf (LOG_NOTICE, "tunnel: %s connection accepted\n",
-	      sock->flags & SOCK_FLAG_PIPE ? "pipe" : "tcp");
+  svz_log (LOG_NOTICE, "tunnel: %s connection accepted\n",
+	   sock->flags & SOCK_FLAG_PIPE ? "pipe" : "tcp");
   return -1;
 }
 
@@ -431,17 +425,16 @@ tnl_detect_proto (void *cfg, socket_t sock)
  * to setup the tunnel server specific callbacks.
  */
 int
-tnl_connect_socket (void *config, socket_t sock)
+tnl_connect_socket (svz_server_t *server, svz_socket_t *sock)
 {
-  tnl_config_t *cfg = config;
-  socket_t xsock = NULL;
+  svz_socket_t *xsock = NULL;
   tnl_connect_t *source;
   
   sock->flags |= SOCK_FLAG_NOFLOOD;
   sock->check_request = sock->flags & SOCK_FLAG_PIPE ?
     tnl_check_request_pipe_source : tnl_check_request_tcp_source;
   sock->disconnected_socket = tnl_disconnect_source;
-  sock_resize_buffers (sock, UDP_BUF_SIZE, UDP_BUF_SIZE);
+  svz_sock_resize_buffers (sock, UDP_BUF_SIZE, UDP_BUF_SIZE);
 
   /* try connecting to target */
   xsock = tnl_create_socket (sock, sock->flags & SOCK_FLAG_PIPE ?
@@ -467,15 +460,15 @@ tnl_connect_socket (void *config, socket_t sock)
  * send data back to its source connection.
  */
 int
-tnl_check_request_tcp_target (socket_t sock)
+tnl_check_request_tcp_target (svz_socket_t *sock)
 {
-  socket_t xsock = NULL;
+  svz_socket_t *xsock = NULL;
   tnl_connect_t *source = sock->data;
 
 #if ENABLE_DEBUG
   if (source == NULL || source->source_sock == NULL)
     {
-      log_printf (LOG_FATAL, "tunnel: tcp target has no source connection\n");
+      svz_log (LOG_FATAL, "tunnel: tcp target has no source connection\n");
       return -1;
     }
 #endif /* ENABLE_DEBUG */
@@ -489,7 +482,7 @@ tnl_check_request_tcp_target (socket_t sock)
   if (tnl_send_request_target (xsock, sock->recv_buffer, 
 			       sock->recv_buffer_fill, sock->userflags) == -1)
     {
-      sock_schedule_for_shutdown (xsock);
+      svz_sock_schedule_for_shutdown (xsock);
       return -1;
     }
 
@@ -504,16 +497,15 @@ tnl_check_request_tcp_target (socket_t sock)
  * connection.
  */
 int
-tnl_check_request_tcp_source (socket_t sock)
+tnl_check_request_tcp_source (svz_socket_t *sock)
 {
-  tnl_config_t *cfg = sock->cfg;
   tnl_connect_t *target = sock->data;
-  socket_t xsock;
+  svz_socket_t *xsock;
 
 #if ENABLE_DEBUG
   if (target == NULL || target->target_sock == NULL)
     {
-      log_printf (LOG_FATAL, "tunnel: tcp source has no target connection\n");
+      svz_log (LOG_FATAL, "tunnel: tcp source has no target connection\n");
       return -1;
     }
 #endif /* ENABLE_DEBUG */
@@ -536,16 +528,15 @@ tnl_check_request_tcp_source (socket_t sock)
  * This function is the handle_request() routine for target UDP sockets.
  */
 int
-tnl_handle_request_udp_target (socket_t sock, char *packet, int len)
+tnl_handle_request_udp_target (svz_socket_t *sock, char *packet, int len)
 {
-  tnl_config_t *cfg = sock->cfg;
   tnl_connect_t *source = sock->data;
-  socket_t xsock = NULL;
+  svz_socket_t *xsock = NULL;
 
 #if ENABLE_DEBUG
   if (source == NULL || source->source_sock == NULL)
     {
-      log_printf (LOG_FATAL, "tunnel: udp target has no source connection\n");
+      svz_log (LOG_FATAL, "tunnel: udp target has no source connection\n");
       return -1;
     }
 #endif /* ENABLE_DEBUG */
@@ -560,7 +551,7 @@ tnl_handle_request_udp_target (socket_t sock, char *packet, int len)
     {
       /* shutdown the source connection if it is TCP or pipe */
       if (sock->userflags & (TNL_FLAG_SRC_TCP | TNL_FLAG_SRC_PIPE))
-	sock_schedule_for_shutdown (xsock);
+	svz_sock_schedule_for_shutdown (xsock);
       return -1;
     }
 
@@ -574,14 +565,14 @@ tnl_handle_request_udp_target (socket_t sock, char *packet, int len)
  * to existing target sockets.
  */
 int
-tnl_handle_request_udp_source (socket_t sock, char *packet, int len)
+tnl_handle_request_udp_source (svz_socket_t *sock, char *packet, int len)
 {
   tnl_config_t *cfg = sock->cfg;
   tnl_connect_t *source;
-  socket_t xsock = NULL;
+  svz_socket_t *xsock = NULL;
 
   /* check if there is such a connection in the source hash already */
-  source = hash_get (cfg->client, tnl_addr (sock));
+  source = svz_hash_get (cfg->client, tnl_addr (sock));
   if (source)
     {
       /* get existing target socket */
@@ -598,7 +589,7 @@ tnl_handle_request_udp_source (socket_t sock, char *packet, int len)
       source->source_sock = sock;
       source->ip = sock->remote_addr;
       source->port = sock->remote_port;
-      hash_put (cfg->client, tnl_addr (sock), source);
+      svz_hash_put (cfg->client, tnl_addr (sock), source);
 
       /* put the source connection into data field of target */
       xsock->data = source;
@@ -608,7 +599,7 @@ tnl_handle_request_udp_source (socket_t sock, char *packet, int len)
   /* forward packet data to target connection */
   if (tnl_send_request_source (xsock, packet, len, sock->userflags) == -1)
     {
-      sock_schedule_for_shutdown (xsock);
+      svz_sock_schedule_for_shutdown (xsock);
       return 0;
     }
 
@@ -619,17 +610,16 @@ tnl_handle_request_udp_source (socket_t sock, char *packet, int len)
  * This function is the handle_request routine for target ICMP sockets.
  */
 int
-tnl_handle_request_icmp_target (socket_t sock, char *packet, int len)
+tnl_handle_request_icmp_target (svz_socket_t *sock, char *packet, int len)
 {
-  tnl_config_t *cfg = sock->cfg;
   tnl_connect_t *source = sock->data;
-  socket_t xsock = NULL;
+  svz_socket_t *xsock = NULL;
 
 #if ENABLE_DEBUG
   if (source == NULL || source->source_sock == NULL)
     {
-      log_printf (LOG_FATAL,
-		  "tunnel: icmp target has no source connection\n");
+      svz_log (LOG_FATAL,
+	       "tunnel: icmp target has no source connection\n");
       return -1;
     }
 #endif /* ENABLE_DEBUG */
@@ -644,7 +634,7 @@ tnl_handle_request_icmp_target (socket_t sock, char *packet, int len)
     {
       /* shutdown source connection if it is TCP or pipe */
       if (sock->userflags & (TNL_FLAG_SRC_TCP | TNL_FLAG_SRC_PIPE))
-	sock_schedule_for_shutdown (xsock);
+	svz_sock_schedule_for_shutdown (xsock);
       return -1;
     }
 
@@ -658,14 +648,14 @@ tnl_handle_request_icmp_target (socket_t sock, char *packet, int len)
  * to existing target sockets.
  */
 int
-tnl_handle_request_icmp_source (socket_t sock, char *packet, int len)
+tnl_handle_request_icmp_source (svz_socket_t *sock, char *packet, int len)
 {
   tnl_config_t *cfg = sock->cfg;
   tnl_connect_t *source;
-  socket_t xsock = NULL;
+  svz_socket_t *xsock = NULL;
 
   /* check if there is such a connection in the source hash already */
-  source = hash_get (cfg->client, tnl_addr (sock));
+  source = svz_hash_get (cfg->client, tnl_addr (sock));
   if (source)
     {
       /* get existing target socket */
@@ -682,7 +672,7 @@ tnl_handle_request_icmp_source (socket_t sock, char *packet, int len)
       source->source_sock = sock;
       source->ip = sock->remote_addr;
       source->port = sock->remote_port;
-      hash_put (cfg->client, tnl_addr (sock), source);
+      svz_hash_put (cfg->client, tnl_addr (sock), source);
 
       /* put the source connection into data field */
       xsock->data = source;
@@ -692,7 +682,7 @@ tnl_handle_request_icmp_source (socket_t sock, char *packet, int len)
   /* forward packet data to target connection */
   if (tnl_send_request_source (xsock, packet, len, sock->userflags) == -1)
     {
-      sock_schedule_for_shutdown (xsock);
+      svz_sock_schedule_for_shutdown (xsock);
       return 0;
     }
 
@@ -704,19 +694,22 @@ tnl_handle_request_icmp_source (socket_t sock, char *packet, int len)
  * ICMP and UDP).
  */
 int
-tnl_disconnect_target (socket_t sock)
+tnl_disconnect_target (svz_socket_t *sock)
 {
   tnl_config_t *cfg = sock->cfg;
   tnl_connect_t *source = sock->data;
-  socket_t xsock;
+  svz_socket_t *xsock;
   char *key;
 
   /* do not do anything if we are shutting down */
-  if (server_nuke_happened)
+  if (svz_nuke_happened)
     {
       /* if source is TCP or PIPE then shutdown referring connection */
       if (sock->userflags & (TNL_FLAG_SRC_TCP | TNL_FLAG_SRC_PIPE))
 	{
+	  xsock = source ? source->source_sock : NULL;
+	  if (xsock)
+	    xsock->data = NULL;
 	  tnl_free_connect (sock);
 	}
       return 0;
@@ -725,7 +718,7 @@ tnl_disconnect_target (socket_t sock)
 #if ENABLE_DEBUG
   if (source == NULL || source->source_sock == NULL)
     {
-      log_printf (LOG_FATAL, "tunnel: target has no source connection\n");
+      svz_log (LOG_FATAL, "tunnel: target has no source connection\n");
       return -1;
     }
 #endif /* ENABLE_DEBUG */
@@ -737,27 +730,27 @@ tnl_disconnect_target (socket_t sock)
   if (sock->userflags & TNL_FLAG_SRC_ICMP)
     {
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG,
-		  "tunnel: sending icmp disconnect on socket id %d\n",
-		  xsock->id);
+      svz_log (LOG_DEBUG,
+	       "tunnel: sending icmp disconnect on socket id %d\n",
+	       xsock->id);
 #endif /* ENABLE_DEBUG */
-      icmp_send_control (xsock, ICMP_SERVEEZ_CLOSE);
+      svz_icmp_send_control (xsock, ICMP_SERVEEZ_CLOSE);
     }
 
   /* if source is TCP or PIPE then shutdown referring connection */
   if (sock->userflags & (TNL_FLAG_SRC_TCP | TNL_FLAG_SRC_PIPE))
     {
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "tunnel: shutdown referrer id %d\n", xsock->id);
+      svz_log (LOG_DEBUG, "tunnel: shutdown referrer id %d\n", xsock->id);
 #endif /* ENABLE_DEBUG */
-      sock_schedule_for_shutdown (xsock);
+      svz_sock_schedule_for_shutdown (xsock);
       xsock->data = NULL;
       tnl_free_connect (sock);
     }
   /* else delete target connection from its hash */
-  else if ((key = hash_contains (cfg->client, source)) != NULL)
+  else if ((key = svz_hash_contains (cfg->client, source)) != NULL)
     {
-      hash_delete (cfg->client, key);
+      svz_hash_delete (cfg->client, key);
       tnl_free_connect (sock);
     }
 
@@ -768,11 +761,10 @@ tnl_disconnect_target (socket_t sock)
  * What happens if a source connection gets lost.
  */
 int
-tnl_disconnect_source (socket_t sock)
+tnl_disconnect_source (svz_socket_t *sock)
 {
-  tnl_config_t *cfg = sock->cfg;
   tnl_connect_t *target = sock->data;
-  socket_t xsock;
+  svz_socket_t *xsock;
 
   /* if target is TCP or PIPE shutdown referring connection */
   if (sock->userflags & (TNL_FLAG_TGT_TCP | TNL_FLAG_TGT_PIPE))
@@ -780,8 +772,8 @@ tnl_disconnect_source (socket_t sock)
 #if ENABLE_DEBUG
       if (target == NULL || target->target_sock == NULL)
 	{
-	  log_printf (LOG_DEBUG, "tunnel: tcp/pipe source has no "
-		      "target connection\n");
+	  svz_log (LOG_DEBUG, "tunnel: tcp/pipe source has no "
+		   "target connection\n");
 	  return -1;
 	}
 #endif /* ENABLE_DEBUG */
@@ -790,9 +782,9 @@ tnl_disconnect_source (socket_t sock)
       xsock = target->target_sock;
 
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "tunnel: shutdown referrer id %d\n", xsock->id);
+      svz_log (LOG_DEBUG, "tunnel: shutdown referrer id %d\n", xsock->id);
 #endif /* ENABLE_DEBUG */
-      sock_schedule_for_shutdown (xsock);
+      svz_sock_schedule_for_shutdown (xsock);
       xsock->data = NULL;
       tnl_free_connect (sock);
     }
@@ -805,7 +797,7 @@ tnl_disconnect_source (socket_t sock)
  * we need to shutdown target sockets ourselves.
  */
 int
-tnl_idle (socket_t sock)
+tnl_idle (svz_socket_t *sock)
 {
   time_t t = time (NULL);
 
